@@ -17,12 +17,12 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 #
 
-import logging
+import gc, logging
 logger = logging.getLogger(__name__)
 
 from time import time
 from copy import copy
-from collections import deque
+from collections import deque, namedtuple, defaultdict
 
 from spyne.const.xml_ns import DEFAULT_NS
 from spyne.util.oset import oset
@@ -30,6 +30,22 @@ from spyne.util.oset import oset
 class BODY_STYLE_WRAPPED: pass
 class BODY_STYLE_EMPTY: pass
 class BODY_STYLE_BARE: pass
+class BODY_STYLE_OUT_BARE: pass
+
+
+# When spyne.server.twisted gets imported, this type gets a static method named
+# `from_twisted_address`. Dark magic.
+Address = namedtuple("Address", ["type", "host", "port"])
+
+
+class _add_address_types():
+    Address.TCP4 = 'TCP4'
+    Address.TCP6 = 'TCP6'
+    Address.UDP4 = 'UDP4'
+    Address.UDP6 = 'UDP6'
+    def address_str(self):
+        return ":".join((self.type, self.host, str(self.port)))
+    Address.__str__ = address_str
 
 
 class AuxMethodContext(object):
@@ -45,7 +61,7 @@ class AuxMethodContext(object):
 class TransportContext(object):
     """Generic object that holds transport-specific context information"""
     def __init__(self, parent, transport, type=None):
-        self.parent = parent;
+        self.parent = parent
         """The MethodContext this object belongs to"""
 
         self.itself = transport
@@ -60,19 +76,32 @@ class TransportContext(object):
         """General purpose variable to hold the string identifier of a request
         encoding. It's nowadays usually 'utf-8', especially with http data"""
 
+        self.remote_addr = None
+        """The address of the other end of the connection."""
+
+        self.sessid = ''
+        """The session id."""
+
 
 class ProtocolContext(object):
-    """Generic object that holds transport-specific context information"""
+    """Generic object that holds protocol-specific context information"""
     def __init__(self, parent, transport, type=None):
-        self.parent = parent;
+        self.parent = parent
         """The MethodContext this object belongs to"""
 
         self.itself = transport
-        """The transport itself; i.e. a ServerBase instance."""
+        """The protocol itself as passed to the `Application` init. This is a
+        `ProtocolBase` instance."""
 
         self.type = type
         """The protocol the transport uses."""
 
+        self._subctx = defaultdict(
+                               lambda: self.__class__(parent, transport, type))
+
+    def __getitem__(self, item):
+        return self._subctx[item]
+
 
 class EventContext(object):
     """Generic object that holds event-specific context information"""
@@ -86,6 +115,9 @@ class MethodContext(object):
     current state of execution of a remote procedure call.
     """
 
+    SERVER = type("SERVER", (object,), {})
+    CLIENT = type("CLIENT", (object,), {})
+
     frozen = False
 
     def copy(self):
@@ -93,8 +125,10 @@ class MethodContext(object):
 
         if retval.transport is not None:
             retval.transport.parent = retval
-        if retval.protocol is not None:
-            retval.protocol.parent = retval
+        if retval.inprot_ctx is not None:
+            retval.inprot_ctx.parent = retval
+        if retval.outprot_ctx is not None:
+            retval.outprot_ctx.parent = retval
         if retval.event is not None:
             retval.event.parent = retval
         if retval.aux is not None:
@@ -113,7 +147,7 @@ class MethodContext(object):
         else:
             return self.descriptor.name
 
-    def __init__(self, transport):
+    def __init__(self, transport, way):
         # metadata
         self.call_start = time()
         """The time the rpc operation was initiated in seconds-since-epoch
@@ -127,6 +161,8 @@ class MethodContext(object):
 
         Useful for benchmarking purposes."""
 
+        self.is_closed = False
+
         self.app = transport.app
         """The parent application."""
 
@@ -137,9 +173,31 @@ class MethodContext(object):
         """The transport-specific context. Transport implementors can use this
         to their liking."""
 
-        self.protocol = ProtocolContext(self, transport)
-        """The protocol-specific context. Protocol implementors can use this
-        to their liking."""
+        self.outprot_ctx = None
+        """The output-protocol-specific context. Protocol implementors can use
+        this to their liking."""
+
+        if self.app.out_protocol is not None:
+            self.outprot_ctx = self.app.out_protocol.get_context(self, transport)
+
+        self.inprot_ctx = None
+        """The input-protocol-specific context. Protocol implementors can use
+        this to their liking."""
+
+        if self.app.in_protocol is not None:
+            self.inprot_ctx = self.app.in_protocol.get_context(self, transport)
+
+        self.protocol = None
+        """The protocol-specific context. This points to the in_protocol when an
+        incoming message is being processed and out_protocol when an outgoing
+        message is being processed."""
+
+        if way is MethodContext.SERVER:
+            self.protocol = self.inprot_ctx
+        elif way is MethodContext.CLIENT:
+            self.protocol = self.outprot_ctx
+        else:
+            raise ValueError(way)
 
         self.event = EventContext(self)
         """Event-specific context. Use this as you want, preferably only in
@@ -155,6 +213,10 @@ class MethodContext(object):
         """This is used to decide which native method to call. It is set by
         the protocol classes."""
 
+        self.files = []
+        """List of stuff to be closed when closing this context. Anything that
+        has a close() callable can go in."""
+
         self.__descriptor = None
 
         #
@@ -290,7 +352,7 @@ class MethodContext(object):
 
     def __setattr__(self, k, v):
         if not self.frozen or k in self.__dict__ or k in \
-                                                ('descriptor', 'out_protocol'):
+                                                 ('descriptor', 'out_protocol'):
             object.__setattr__(self, k, v)
         else:
             raise ValueError("use the udc member for storing arbitrary data "
@@ -317,6 +379,13 @@ class MethodContext(object):
     def close(self):
         self.call_end = time()
         self.app.event_manager.fire_event("method_context_closed", self)
+        for f in self.files:
+            f.close()
+
+        self.is_closed = True
+
+        # this is important to have file descriptors returned in a timely manner
+        gc.collect()
 
     def set_out_protocol(self, what):
         self._out_protocol = what
@@ -467,7 +536,6 @@ class MethodDescriptor(object):
 
     def translate(self, locale, default):
         """
-        :param cls: class
         :param locale: locale string
         :param default: default string if no translation found
         :returns: translated string
@@ -531,7 +599,7 @@ class EventManager(object):
         handlers.add(handler)
         self.handlers[event_name] = handlers
 
-    def fire_event(self, event_name, ctx):
+    def fire_event(self, event_name, ctx, *args, **kwargs):
         """Run all the handlers for a given event name.
 
         :param event_name: The event identifier, indicated by the documentation.
@@ -542,17 +610,42 @@ class EventManager(object):
 
         handlers = self.handlers.get(event_name, oset())
         for handler in handlers:
-            handler(ctx)
+            handler(ctx, *args, **kwargs)
 
 
 class FakeContext(object):
-    def __init__(self, app=None, descriptor=None, out_object=None, out_error=None,
-                                             out_document=None, out_string=None):
+    def __init__(self, app=None, descriptor=None,
+            in_object=None, in_error=None, in_document=None, in_string=None,
+            out_object=None, out_error=None, out_document=None, out_string=None,
+            in_protocol=None, out_protocol=None):
         self.app = app
         self.descriptor = descriptor
+        self.in_object = in_object
+        self.in_error = in_error
+        self.in_document = in_document
+        self.in_string = in_string
         self.out_error = out_error
         self.out_object = out_object
         self.out_document = out_document
         self.out_string = out_string
-        self.protocol = type("ProtocolContext", (object,), {})()
+        self.in_protocol = in_protocol
+        self.out_protocol = out_protocol
+
+        if self.in_protocol is not None:
+            self.inprot_ctx = self.in_protocol.get_context(self, None)
+        else:
+            self.inprot_ctx = type("ProtocolContext", (object,), {})()
+
+        from spyne.protocol.html._base import HtmlClothProtocolContext
+
+        if self.out_protocol is not None:
+            self.outprot_ctx = self.out_protocol.get_context(self, None)
+        else:
+            # The outprot_ctx here must contain properties from ALL tested
+            # protocols' context objects. That's why we use
+            # HtmlClothProtocolContext here, it's just the one with most
+            # attributes.
+            self.outprot_ctx = HtmlClothProtocolContext(self, None)
+
+        self.protocol = self.outprot_ctx
         self.transport = type("ProtocolContext", (object,), {})()
