@@ -20,11 +20,12 @@
 import logging
 logger = logging.getLogger(__name__)
 logger_client = logging.getLogger('.'.join([__name__, 'client']))
+logger_server = logging.getLogger('.'.join([__name__, 'server']))
 
 from spyne import BODY_STYLE_EMPTY
 from spyne import BODY_STYLE_BARE
 from spyne import BODY_STYLE_WRAPPED
-from spyne.model.fault import Fault
+from spyne.error import Fault, Redirect
 from spyne.interface import Interface
 from spyne import EventManager
 from spyne.util.appreg import register_application
@@ -64,7 +65,7 @@ class Application(object):
                          protocol. It's only optional for NullServer transport.
     :param out_protocol: A ProtocolBase instance that denotes the output
                          protocol. It's only optional for NullServer transport.
-    :param interface:    Ignored. Kept for backwards-compatibility purposes.
+    :param config:       An arbitrary python object to store random global data.
 
     Supported events:
         * ``method_call``:
@@ -90,10 +91,11 @@ class Application(object):
     transport = None
 
     def __init__(self, services, tns, name=None,
-                          in_protocol=None, out_protocol=None, interface=None):
+                          in_protocol=None, out_protocol=None, config=None):
         self.services = tuple(services)
         self.tns = tns
         self.name = name
+        self.config = config
 
         if self.name is None:
             self.name = self.__class__.__name__.split('.')[-1]
@@ -123,8 +125,6 @@ class Application(object):
 
         register_application(self)
 
-        self.reinitialize()
-
     def process_request(self, ctx):
         """Takes a MethodContext instance. Returns the response to the request
         as a native python object. If the function throws an exception, it
@@ -140,21 +140,59 @@ class Application(object):
             if ctx.service_class is not None:
                 ctx.service_class.event_manager.fire_event('method_call', ctx)
 
-            # call the method
+            # in object is always a sequence of incoming values. We need to fix
+            # that for bare mode.
+            if ctx.descriptor.body_style is BODY_STYLE_BARE:
+                ctx.in_object = [ctx.in_object]
+            elif ctx.descriptor.body_style is BODY_STYLE_EMPTY:
+                ctx.in_object = []
+
+            # call user method
             ctx.out_object = self.call_wrapper(ctx)
 
-            # out object is always an iterable of return values. see
+            # out object is always a sequence of return values. see
             # MethodContext docstrings for more info
             if ctx.descriptor.body_style is not BODY_STYLE_WRAPPED or \
                                 len(ctx.descriptor.out_message._type_info) <= 1:
-                # the return value should already be wrapped by a sequence.
+                # if it's not a wrapped method, OR there's just one return type
+                # we wrap it ourselves
                 ctx.out_object = [ctx.out_object]
 
+            # Now that the processing is switched to the outgoing message,
+            # point ctx.protocol to ctx.out_protocol
+            ctx.protocol = ctx.outprot_ctx
+
             # fire events
             self.event_manager.fire_event('method_return_object', ctx)
             if ctx.service_class is not None:
                 ctx.service_class.event_manager.fire_event(
                                                     'method_return_object', ctx)
+        except Redirect as e:
+            try:
+                e.do_redirect()
+
+                ctx.out_object = [None]
+
+                # Now that the processing is switched to the outgoing message,
+                # point ctx.protocol to ctx.out_protocol
+                ctx.protocol = ctx.outprot_ctx
+
+                # fire events
+                self.event_manager.fire_event('method_redirect', ctx)
+                if ctx.service_class is not None:
+                    ctx.service_class.event_manager.fire_event(
+                                                         'method_redirect', ctx)
+
+            except Exception as e:
+                logger_server.exception(e)
+                ctx.out_error = Fault('Server',
+                                             get_fault_string_from_exception(e))
+
+                # fire events
+                self.event_manager.fire_event('method_redirect_exception', ctx)
+                if ctx.service_class is not None:
+                    ctx.service_class.event_manager.fire_event(
+                                               'method_redirect_exception', ctx)
 
         except Fault as e:
             if e.faultcode == 'Client' or e.faultcode.startswith('Client.'):
@@ -170,8 +208,11 @@ class Application(object):
                 ctx.service_class.event_manager.fire_event(
                                                'method_exception_object', ctx)
 
+        # we don't catch BaseException because we're not interested in
+        # "system-exiting" exceptions. See:
+        # https://docs.python.org/2/library/exceptions.html#exceptions.Exception
         except Exception as e:
-            logger.exception(e)
+            logger_server.exception(e)
 
             ctx.out_error = Fault('Server', get_fault_string_from_exception(e))
 
@@ -187,11 +228,6 @@ class Application(object):
         management.
         """
 
-        if ctx.descriptor.body_style is BODY_STYLE_BARE:
-            ctx.in_object = [ctx.in_object]
-        elif ctx.descriptor.body_style is BODY_STYLE_EMPTY:
-            ctx.in_object = []
-
         retval = None
 
         # service rpc
@@ -240,15 +276,22 @@ class Application(object):
     def _has_callbacks(self):
         return self.interface._has_callbacks()
 
-    def reinitialize(self):
-        from spyne.server import ServerBase
+    def reinitialize(self, server):
+        """This is normally called on transport instantiation by ServerBase"""
 
-        server = ServerBase(self)
-        aux_memo = set()
+        seen = set()
+
+        from spyne import MethodDescriptor
         for d in self.interface.method_id_map.values():
-            if d.aux is not None and not id(d.aux) in aux_memo:
+            assert isinstance(d, MethodDescriptor)
+
+            if d.aux is not None and not id(d.aux) in seen:
                 d.aux.initialize(server)
-                aux_memo.add(id(d.aux))
+                seen.add(id(d.aux))
+
+            if d.service_class is not None and not id(d.service_class) in seen:
+                d.service_class.initialize(server)
+                seen.add(id(d.service_class))
 
     def __hash__(self):
         return hash(tuple((id(s) for s in self.services)))
