@@ -20,6 +20,6 @@
 """The ``spyne.client`` package contains the client transports."""
 
 from spyne.client._base import Factory
-from spyne.client._base import Service
+from spyne.client._base import RemoteService
 from spyne.client._base import ClientBase
 from spyne.client._base import RemoteProcedureBase
