@@ -22,6 +22,7 @@
 from spyne._base import MethodContext
 from spyne.model.primitive import string_encoding
 
+
 class Factory(object):
     def __init__(self, app):
         self.__app = app
@@ -29,7 +30,8 @@ class Factory(object):
     def create(self, object_name):
         return self.__app.interface.get_class_instance(object_name)
 
-class Service(object):
+
+class RemoteService(object):
     def __init__(self, rpc_class, url, app, *args, **kwargs):
         self.__app = app
         self.__url = url
@@ -42,6 +44,7 @@ class Service(object):
         return self.rpc_class(self.__url, self.__app, key, self.out_header,
                                                     *self.args, **self.kwargs)
 
+
 class RemoteProcedureBase(object):
     """Abstract base class that handles all (de)serialization.
 
@@ -60,11 +63,12 @@ class RemoteProcedureBase(object):
         self.url = url
         self.app = app
 
-        initial_ctx = MethodContext(self)
+        initial_ctx = MethodContext(self, MethodContext.CLIENT)
         initial_ctx.method_request_string = name
         initial_ctx.out_header = out_header
 
-        self.contexts = initial_ctx.out_protocol.generate_method_contexts(initial_ctx)
+        self.contexts = initial_ctx.out_protocol.generate_method_contexts(
+                                                                    initial_ctx)
 
     def __call__(self, *args, **kwargs):
         """Serializes its arguments, sends them, receives and deserializes the
