@@ -17,12 +17,9 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 #
 
-"""The ``spyne.const.xml`` module contains various XML-related constants like
-namespace prefixes, namespace values and schema uris.
-"""
-
-#SHOULDHAVEFIXEDME: These are constants, so should have been all UPPERCASE.
+# This module is DEPRECATED. Use ``spyne.const.xml``.
 
+# FIXME: These are constants, so should have been all UPPERCASE.
 xml = 'http://www.w3.org/XML/1998/namespace'
 xsd = 'http://www.w3.org/2001/XMLSchema'
 xsi = 'http://www.w3.org/2001/XMLSchema-instance'
@@ -30,11 +27,12 @@ wsa = 'http://schemas.xmlsoap.org/ws/2003/03/addressing'
 xop = 'http://www.w3.org/2004/08/xop/include'
 soap = 'http://schemas.xmlsoap.org/wsdl/soap/'
 wsdl = 'http://schemas.xmlsoap.org/wsdl/'
+xhtml = 'http://www.w3.org/1999/xhtml'
 plink = 'http://schemas.xmlsoap.org/ws/2003/05/partner-link/'
-soap_enc = 'http://schemas.xmlsoap.org/soap/encoding/'
-soap_env = 'http://schemas.xmlsoap.org/soap/envelope/'
-soap12_env = 'http://www.w3.org/2003/05/soap-envelope/'
-soap12_enc = 'http://www.w3.org/2003/05/soap-encoding/'
+soap11_enc = 'http://schemas.xmlsoap.org/soap/encoding/'
+soap11_env = 'http://schemas.xmlsoap.org/soap/envelope/'
+soap12_env = 'http://www.w3.org/2003/05/soap-envelope'
+soap12_enc = 'http://www.w3.org/2003/05/soap-encoding'
 
 const_nsmap = {
     'xml': xml,
@@ -43,8 +41,8 @@ const_nsmap = {
     'plink': plink,
     'soap': soap,
     'wsdl': wsdl,
-    'senc': soap_enc,
-    'senv': soap_env,
+    'soap11enc': soap11_enc,
+    'soap11env': soap11_env,
     'soap12env': soap12_env,
     'soap12enc': soap12_enc,
     'wsa': wsa,
