@@ -1,3 +1,4 @@
+
 #
 # spyne - Copyright (C) Spyne contributors.
 #
@@ -28,6 +29,7 @@ decorator is a simple example of this.
 import spyne.const.xml_ns
 
 from copy import copy
+from inspect import isclass
 
 from spyne import MethodDescriptor
 
@@ -36,17 +38,16 @@ from spyne import MethodDescriptor
 from spyne._base import BODY_STYLE_EMPTY
 from spyne._base import BODY_STYLE_WRAPPED
 from spyne._base import BODY_STYLE_BARE
+from spyne._base import BODY_STYLE_OUT_BARE
 
-from spyne.model.complex import ComplexModel
+from spyne.model import ModelBase, ComplexModel
 from spyne.model.complex import TypeInfo
 
 from spyne.const import add_request_suffix
 
 
-def _produce_input_message(f, params, kparams, in_message_name,
-                           in_variable_names, no_ctx, no_self, args):
-    _body_style = _validate_body_style(kparams)
-
+def _produce_input_message(f, params, in_message_name,
+                      in_variable_names, no_ctx, no_self, args, body_style_str):
     arg_start = 0
     if no_ctx is False:
         arg_start += 1
@@ -87,17 +88,27 @@ def _produce_input_message(f, params, kparams, in_message_name,
         ns, _, in_message_name = in_message_name[1:].partition("}")
 
     message = None
-    if _body_style == 'bare':
+    if body_style_str == 'bare':
         if len(in_params) > 1:
             raise Exception("body_style='bare' can handle at most one function "
                             "argument.")
+
         if len(in_params) == 0:
             message = ComplexModel.produce(type_name=in_message_name,
                                            namespace=ns, members=in_params)
         else:
             message, = in_params.values()
             message = message.customize(sub_name=in_message_name, sub_ns=ns)
-            assert message.Attributes.sub_name is not None
+
+            from spyne.model import ComplexModelBase
+            if issubclass(message, ComplexModelBase) and not message._type_info:
+                raise Exception("body_style='bare' does not allow empty "
+                                "model as param")
+
+            # there can't be multiple arguments here.
+            if message.__type_name__ is ModelBase.Empty:
+                message._fill_empty_type_name(ns, in_message_name,
+                                                    "%s_arg0" % in_message_name)
 
     else:
         message = ComplexModel.produce(type_name=in_message_name,
@@ -108,13 +119,14 @@ def _produce_input_message(f, params, kparams, in_message_name,
 
 
 def _validate_body_style(kparams):
-    _body_style = kparams.get('_body_style')
-    _soap_body_style = kparams.get('_soap_body_style')
+    _body_style = kparams.pop('_body_style', None)
+    _soap_body_style = kparams.pop('_soap_body_style', None)
 
+    allowed_body_styles = ('wrapped', 'bare', 'out_bare')
     if _body_style is None:
         _body_style = 'wrapped'
-    elif not (_body_style in ('wrapped', 'bare')):
-        raise ValueError("body_style must be one of ('wrapped', 'bare')")
+    elif not (_body_style in allowed_body_styles):
+        raise ValueError("body_style must be one of %r" % allowed_body_styles)
     elif _soap_body_style == 'document':
         _body_style = 'wrapped'
     elif _soap_body_style == 'rpc':
@@ -124,30 +136,30 @@ def _validate_body_style(kparams):
     else:
         raise ValueError("soap_body_style must be one of ('rpc', 'document')")
 
-    assert _body_style in ('wrapped', 'bare')
+    assert _body_style in ('wrapped', 'bare', 'out_bare')
 
     return _body_style
 
 
-def _produce_output_message(func_name, kparams):
+def _produce_output_message(func_name, body_style_str, kparams):
     """Generate an output message for "rpc"-style API methods.
 
     This message is a wrapper to the declared return type.
     """
 
-    _returns = kparams.get('_returns')
-    _body_style = _validate_body_style(kparams)
-    _out_message_name = kparams.get('_out_message_name', '%s%s' %
+    _returns = kparams.pop('_returns', None)
+
+    _out_message_name = kparams.pop('_out_message_name', '%s%s' %
                                        (func_name, spyne.const.RESPONSE_SUFFIX))
 
     out_params = TypeInfo()
 
-    if _returns and _body_style == 'wrapped':
+    if _returns and body_style_str == 'wrapped':
         if isinstance(_returns, (list, tuple)):
             default_names = ['%s%s%d'% (func_name, spyne.const.RESULT_SUFFIX, i)
                                                   for i in range(len(_returns))]
 
-            _out_variable_names = kparams.get('_out_variable_names',
+            _out_variable_names = kparams.pop('_out_variable_names',
                                                                   default_names)
 
             assert (len(_returns) == len(_out_variable_names))
@@ -156,7 +168,7 @@ def _produce_output_message(func_name, kparams):
             out_params = TypeInfo(var_pair)
 
         else:
-            _out_variable_name = kparams.get('_out_variable_name',
+            _out_variable_name = kparams.pop('_out_variable_name',
                                 '%s%s' % (func_name, spyne.const.RESULT_SUFFIX))
 
             out_params[_out_variable_name] = _returns
@@ -165,8 +177,10 @@ def _produce_output_message(func_name, kparams):
     if _out_message_name.startswith("{"):
         ns = _out_message_name[1:].partition("}")[0]
 
-    if _body_style == 'bare' and _returns is not None:
+    if body_style_str.endswith('bare') and _returns is not None:
         message = _returns.customize(sub_name=_out_message_name, sub_ns=ns)
+        if message.__type_name__ is ModelBase.Empty:
+            message.__type_name__ = _out_message_name
 
     else:
         message = ComplexModel.produce(type_name=_out_message_name,
@@ -178,6 +192,28 @@ def _produce_output_message(func_name, kparams):
     return message
 
 
+def _substitute_self_reference(params, kparams, kwargs, _no_self):
+    from spyne.model import SelfReference
+
+    for i, v in enumerate(params):
+        if isclass(v) and issubclass(v, SelfReference):
+            if _no_self:
+                raise ValueError("SelfReference can't be used in @rpc")
+            else:
+                params[i] = kwargs['_self_ref_replacement']
+        else:
+            params[i] = v
+
+    for k, v in kparams.items():
+        if isclass(v) and issubclass(v, SelfReference):
+            if _no_self:
+                raise ValueError("SelfReference can't be used in @rpc")
+            else:
+                kparams[k] = kwargs['_self_ref_replacement']
+        else:
+            kparams[k] = v
+
+
 def rpc(*params, **kparams):
     """Method decorator to tag a method as a remote procedure call in a
     :class:`spyne.service.ServiceBase` subclass.
@@ -196,7 +232,7 @@ def rpc(*params, **kparams):
 
     :param _returns: Denotes The return type of the function. It can be a
         type or a sequence of types for functions that have multiple return
-         values.
+        values.
     :param _in_header: A type or an iterable of types that that this method
         accepts as incoming header.
     :param _out_header: A type or an iterable of types that that this method
@@ -238,65 +274,75 @@ def rpc(*params, **kparams):
         overriding it.
     """
 
+    params = list(params)
+
     def explain(f):
-        def explain_method(*args, **kwargs):
+        def explain_method(**kwargs):
             function_name = kwargs['_default_function_name']
 
-            _is_callback = kparams.get('_is_callback', False)
-            _is_async = kparams.get('_is_async', False)
-            _mtom = kparams.get('_mtom', False)
-            _in_header = kparams.get('_in_header', None)
-            _out_header = kparams.get('_out_header', None)
-            _port_type = kparams.get('_soap_port_type', None)
-            _no_ctx = kparams.get('_no_ctx', False)
-            _no_self = kparams.get('_no_self', True)
-            _udp = kparams.get('_udp', None)
-            _aux = kparams.get('_aux', None)
-            _pattern = kparams.get("_pattern", None)
-            _patterns = kparams.get("_patterns", [])
-            _args = kparams.get("_args", None)
-            _translations = kparams.get("_translations", None)
-            _when = kparams.get("_when", None)
-            _service_class = kparams.get("_service_class", None)
-            _href = kparams.get("_href", None)
+            # this block is passed straight to the descriptor
+            _is_callback = kparams.pop('_is_callback', False)
+            _is_async = kparams.pop('_is_async', False)
+            _mtom = kparams.pop('_mtom', False)
+            _in_header = kparams.pop('_in_header', None)
+            _out_header = kparams.pop('_out_header', None)
+            _port_type = kparams.pop('_soap_port_type', None)
+            _no_ctx = kparams.pop('_no_ctx', False)
+            _no_self = kparams.pop('_no_self', True)
+            _udp = kparams.pop('_udp', None)
+            _aux = kparams.pop('_aux', None)
+            _pattern = kparams.pop("_pattern", None)
+            _patterns = kparams.pop("_patterns", [])
+            _args = kparams.pop("_args", None)
+            _translations = kparams.pop("_translations", None)
+            _when = kparams.pop("_when", None)
+            _service_class = kparams.pop("_service_class", None)
+            _href = kparams.pop("_href", None)
 
-            if _no_self:
-                from spyne.model import SelfReference
-
-                if SelfReference in params:
-                    raise ValueError("SelfReference can't be used in @rpc")
-                if SelfReference in kparams.values():
-                    raise ValueError("SelfReference can't be used in @rpc")
+            _substitute_self_reference(params, kparams, kwargs, _no_self)
 
             _faults = None
             if ('_faults' in kparams) and ('_throws' in kparams):
                 raise ValueError("only one of '_throws ' or '_faults' arguments"
-                                 "should be given, as they're synonyms.")
+                                 "must be given -- they're synonyms.")
+
             elif '_faults' in kparams:
-                _faults = kparams.get('_faults', None)
+                _faults = kparams.pop('_faults')
 
             elif '_throws' in kparams:
-                _faults = kparams.get('_throws', None)
+                _faults = kparams.pop('_throws')
 
             _in_message_name_override = not ('_in_message_name' in kparams)
-            _in_message_name = kparams.get('_in_message_name', function_name)
+            _in_message_name = kparams.pop('_in_message_name', function_name)
 
-            _operation_name = kparams.get('_operation_name', function_name)
+            _operation_name = kparams.pop('_operation_name', function_name)
 
-            if _operation_name != function_name and _in_message_name != function_name:
+            if _operation_name != function_name and \
+                                              _in_message_name != function_name:
                 raise ValueError(
                     "only one of '_operation_name' and '_in_message_name' "
                     "arguments should be given")
+
             if _in_message_name == function_name:
                 _in_message_name = add_request_suffix(_operation_name)
 
-            _in_variable_names = kparams.get('_in_variable_names', {})
+            _in_variable_names = kparams.pop('_in_variable_names', {})
 
-            in_message = _produce_input_message(f, params, kparams,
-                 _in_message_name, _in_variable_names, _no_ctx, _no_self, _args)
+            body_style = BODY_STYLE_WRAPPED
+            body_style_str = _validate_body_style(kparams)
+            if body_style_str.endswith('bare'):
+                if body_style_str == 'out_bare':
+                    body_style = BODY_STYLE_OUT_BARE
+                else:
+                    body_style = BODY_STYLE_BARE
+
+            in_message = _produce_input_message(f, params,
+                                           _in_message_name, _in_variable_names,
+                                       _no_ctx, _no_self, _args, body_style_str)
 
             _out_message_name_override = not ('_out_message_name' in kparams)
-            out_message = _produce_output_message(function_name, kparams)
+            out_message = _produce_output_message(function_name, body_style_str,
+                                                                        kparams)
 
             doc = getattr(f, '__doc__')
 
@@ -307,12 +353,10 @@ def rpc(*params, **kparams):
             if _pattern is not None:
                 _patterns = [_pattern]
 
-            body_style = BODY_STYLE_WRAPPED
-            if _validate_body_style(kparams) == 'bare':
-                body_style = BODY_STYLE_BARE
-                t = in_message
+            if body_style_str.endswith('bare'):
                 from spyne.model import ComplexModelBase
 
+                t = in_message
                 if issubclass(t, ComplexModelBase) and len(t._type_info) == 0:
                     body_style = BODY_STYLE_EMPTY
 
@@ -329,10 +373,12 @@ def rpc(*params, **kparams):
                 service_class=_service_class, href=_href,
             )
 
-            if _patterns is not None:
+            if _patterns is not None and _no_self:
                 for p in _patterns:
                     p.hello(retval)
 
+            if len(kparams) > 0:
+                raise Exception("Unknown kwarg(s) %r passed.", kparams)
             return retval
 
         explain_method.__doc__ = f.__doc__
