@@ -31,6 +31,7 @@ from spyne.model import ModelBase
 from spyne.model import Array, Iterable
 from spyne.model import ComplexModelBase
 from spyne.model.complex import XmlModifier
+from spyne.util.six import get_function_name
 
 
 def _generate_method_id(cls, descriptor):
@@ -59,6 +60,8 @@ class Interface(object):
         self.nsmap = {}
         self.prefmap = {}
         self.member_methods = deque()
+        self.method_descriptor_id_to_key = {}
+        self.service_attrs = defaultdict(dict)
 
         self.import_base_namespaces = import_base_namespaces
         self.app = app
@@ -124,7 +127,7 @@ class Interface(object):
 
     def get_class(self, key):
         """Returns the class definition that corresponds to the given key.
-        Keys are in '{namespace}class_name' form, a.k.a. XML QName format.
+        Keys are in '{namespace}class_name' form.
 
         Not meant to be overridden.
         """
@@ -174,8 +177,10 @@ class Interface(object):
                 in_header.resolve_namespace(in_header, self.get_tns())
                 if method.aux is None:
                     yield in_header
-                if in_header.get_namespace() != self.get_tns():
-                    self.imports[self.get_tns()].add(in_header.get_namespace())
+                in_header_ns = in_header.get_namespace()
+                if in_header_ns != self.get_tns() and \
+                                             self.is_valid_import(in_header_ns):
+                    self.imports[self.get_tns()].add(in_header_ns)
 
         if not (method.out_header is None):
             if not isinstance(method.out_header, (list, tuple)):
@@ -185,9 +190,10 @@ class Interface(object):
                 out_header.resolve_namespace(out_header, self.get_tns())
                 if method.aux is None:
                     yield out_header
-                if out_header.get_namespace() != self.get_tns():
-                    self.imports[self.get_tns()].add(
-                                             out_header.get_namespace())
+                out_header_ns = out_header.get_namespace()
+                if out_header_ns != self.get_tns() and \
+                                            self.is_valid_import(out_header_ns):
+                    self.imports[self.get_tns()].add(out_header_ns)
 
         if method.faults is None:
             method.faults = []
@@ -201,10 +207,22 @@ class Interface(object):
                 yield fault
 
         method.in_message.resolve_namespace(method.in_message, self.get_tns())
+        in_message_ns = method.in_message.get_namespace()
+        if in_message_ns != self.get_tns() and \
+                                            self.is_valid_import(in_message_ns):
+            self.imports[self.get_tns()].add(method.in_message.get_namespace())
+
         if method.aux is None:
             yield method.in_message
 
         method.out_message.resolve_namespace(method.out_message, self.get_tns())
+        assert not method.out_message.get_type_name() is method.out_message.Empty
+
+        out_message_ns = method.out_message.get_namespace()
+        if out_message_ns != self.get_tns() and \
+                                           self.is_valid_import(out_message_ns):
+            self.imports[self.get_tns()].add(out_message_ns)
+
         if method.aux is None:
             yield method.out_message
 
@@ -219,13 +237,12 @@ class Interface(object):
         if issubclass(s, ComplexModelBase) and method.in_message_name_override:
             method_key = '{%s}%s.%s' % (self.app.tns,
                                                  s.get_type_name(), method.name)
-
-        logger.debug('\tadding method %r to match %r tag.' %
-                                                      (method.name, method_key))
         key = _generate_method_id(s, method)
         if key in self.method_id_map:
             c = self.method_id_map[key].parent_class
-            if c.__orig__ is None:
+            if c is s:
+                pass
+            elif c.__orig__ is None:
                 assert c is s.__orig__, "%r.%s conflicts with %r.%s" % \
                                         (c, key, s.__orig__, key)
             elif s.__orig__ is None:
@@ -236,6 +253,9 @@ class Interface(object):
                                         (c.__orig__, key, s.__orig__, key)
             return
 
+        logger.debug('  adding method %s.%s to match %r tag.', s.__name__,
+                                 get_function_name(method.function), method_key)
+
         self.method_id_map[key] = method
 
         val = self.service_method_map.get(method_key, None)
@@ -261,6 +281,12 @@ class Interface(object):
                                     % (method.name, s.__module__,  s.__name__,
                                                    os.__module__, os.__name__))
 
+    def check_method(self, method):
+        """Override this if you need to cherry-pick methods added to the
+        interface document."""
+
+        return True
+
     def populate_interface(self, types=None):
         """Harvests the information stored in individual classes' _type_info
         dictionaries. It starts from function definitions and includes only
@@ -269,8 +295,8 @@ class Interface(object):
 
         # populate types
         for s in self.services:
-            logger.debug("populating '%s.%s (%s)' types..." % (s.__module__,
-                                                s.__name__, s.get_service_key()))
+            logger.debug("populating '%s.%s (%s)' types...", s.__module__,
+                                        s.__name__, s.get_service_key(self.app))
 
             for method in s.public_methods.values():
                 if method.in_header is None:
@@ -282,25 +308,35 @@ class Interface(object):
                 if method.aux is not None:
                     method.aux.methods.append(_generate_method_id(s, method))
 
+                if not self.check_method(method):
+                    logger.debug("method %s' discarded by check_method",
+                                                               method.class_key)
+                    continue
+
+                logger.debug("  enumerating classes for method '%s'",
+                                                               method.class_key)
                 for cls in self.add_method(method):
                     self.add_class(cls)
 
         # populate call routes for service methods
         for s in self.services:
-            s.__tns__ = self.get_tns()
-            logger.debug("populating '%s.%s' methods..." % (s.__module__,
-                                                                    s.__name__))
+            self.service_attrs[s]['tns'] = self.get_tns()
+            logger.debug("populating '%s.%s' routes...", s.__module__,
+                                                                     s.__name__)
             for method in s.public_methods.values():
                 self.process_method(s, method)
 
         # populate call routes for member methods
         for cls, descriptor in self.member_methods:
-            self.process_method(cls, descriptor)
+            self.process_method(cls.__orig__ or cls, descriptor)
+
+        # populate method descriptor id to method key map
+        self.method_descriptor_id_to_key = dict(((id(v[0]), k)
+                                    for k,v in self.service_method_map.items()))
 
         logger.debug("From this point on, you're not supposed to make any "
                      "changes to the class and method structure of the exposed "
-                     "services."
-                 )
+                     "services.")
 
     tns = property(get_tns)
 
@@ -340,25 +376,29 @@ class Interface(object):
         assert ns is not None, ('either assign a namespace to the class or call'
                         ' cls.resolve_namespace(cls, "some_default_ns") on it.')
 
-        if not (ns in self.imports):
+        if not (ns in self.imports) and self.is_valid_import(ns):
             self.imports[ns] = set()
 
         class_key = '{%s}%s' % (ns, tn)
-        logger.debug('\tadding class %r for %r' % (repr(cls), class_key))
+        logger.debug('    adding class %r for %r', repr(cls), class_key)
 
         assert class_key not in self.classes, ("Somehow, you're trying to "
             "overwrite %r by %r for class key %r." %
                                       (self.classes[class_key], cls, class_key))
 
-        assert not (cls.get_type_name() is cls.Empty)
+        assert not (cls.get_type_name() is cls.Empty), cls
 
-        self.deps[cls]
+        self.deps[cls]  # despite the appearances, this is not totally useless.
         self.classes[class_key] = cls
         if ns == self.get_tns():
             self.classes[tn] = cls
 
         # add parent class
         extends = getattr(cls, '__extends__', None)
+        while extends is not None and \
+                                   (extends.get_type_name() is ModelBase.Empty):
+            extends = getattr(extends, '__extends__', None)
+
         if add_parent and extends is not None:
             assert issubclass(extends, ModelBase)
             self.deps[cls].add(extends)
@@ -367,9 +407,9 @@ class Interface(object):
             if parent_ns != ns and not parent_ns in self.imports[ns] and \
                                                 self.is_valid_import(parent_ns):
                 self.imports[ns].add(parent_ns)
-                logger.debug("\timporting %r to %r because %r extends %r" % (
+                logger.debug("    importing %r to %r because %r extends %r",
                                             parent_ns, ns, cls.get_type_name(),
-                                            extends.get_type_name()))
+                                            extends.get_type_name())
 
         # add fields
         if issubclass(cls, ComplexModelBase):
@@ -379,7 +419,7 @@ class Interface(object):
 
                 self.deps[cls].add(v)
 
-                logger.debug("\tadding %s.%s = %r", cls.get_type_name(), k, v)
+                logger.debug("    adding %s.%s = %r", cls.get_type_name(), k, v)
                 if v.get_namespace() is None:
                     v.resolve_namespace(v, ns)
 
@@ -392,7 +432,7 @@ class Interface(object):
                 if child_ns != ns and not child_ns in self.imports[ns] and \
                                                  self.is_valid_import(child_ns):
                     self.imports[ns].add(child_ns)
-                    logger.debug("\timporting %r to %r for %s.%s(%r)",
+                    logger.debug("    importing %r to %r for %s.%s(%r)",
                                        child_ns, ns, cls.get_type_name(), k, v)
 
                 if issubclass(v, XmlModifier):
@@ -402,11 +442,11 @@ class Interface(object):
                     if child_ns != ns and not child_ns in self.imports[ns] and \
                                                  self.is_valid_import(child_ns):
                         self.imports[ns].add(child_ns)
-                        logger.debug("\timporting %r to %r for %s.%s(%r)",
+                        logger.debug("    importing %r to %r for %s.%s(%r)",
                                     child_ns, ns, v.get_type_name(), k, v.type)
 
             if cls.Attributes.methods is not None:
-                logger.debug("\tpopulating member methods for '%s.%s'...",
+                logger.debug("    populating member methods for '%s.%s'...",
                                        cls.get_namespace(), cls.get_type_name())
 
                 for method_key, descriptor in cls.Attributes.methods.items():
@@ -417,7 +457,7 @@ class Interface(object):
                         self.add_class(c)
 
             if cls.Attributes._subclasses is not None:
-                logger.debug("\tadding subclasses of '%s.%s'...",
+                logger.debug("    adding subclasses of '%s.%s'...",
                                        cls.get_namespace(), cls.get_type_name())
 
                 for c in cls.Attributes._subclasses:
@@ -429,10 +469,10 @@ class Interface(object):
                             self.add_class(c, add_parent=False)
                             self.deps[c].add(cls)
                     else:
-                        logger.debug("\tnot adding %r to %r because it would "
+                        logger.debug("    not adding %r to %r because it would "
                             "cause circular imports because %r extends %r and "
-                            "they don't have the same namespace" % (child_ns,
-                                   ns, c.get_type_name(), cls.get_type_name()))
+                            "they don't have the same namespace", child_ns,
+                                     ns, c.get_type_name(), cls.get_type_name())
 
     def is_valid_import(self, ns):
         """This will return False for base namespaces unless told otherwise."""
