@@ -30,19 +30,18 @@ REGEX_WSDL = re.compile('[.?]wsdl$')
 import spyne.const.xml_ns
 
 from lxml import etree
+from lxml.builder import E
+from lxml.etree import SubElement
 
+from spyne.const.xml import WSDL11, XSD, NS_WSA, WSDL11_SOAP, PLINK
 from spyne.interface.xml_schema import XmlSchema
 
-_ns_plink = spyne.const.xml_ns.plink
-_ns_xsd = spyne.const.xml_ns.xsd
-_ns_wsa = spyne.const.xml_ns.wsa
-_ns_wsdl = spyne.const.xml_ns.wsdl
-_ns_soap = spyne.const.xml_ns.soap
-_pref_wsa = spyne.const.xml_ns.const_prefmap[_ns_wsa]
+PREF_WSA = spyne.const.xml.PREFMAP[NS_WSA]
 
 _in_header_msg_suffix = 'InHeaderMsg'
 _out_header_msg_suffix = 'OutHeaderMsg'
 
+
 def check_method_port(service, method):
     if len(service.__port_types__) != 0 and method.port_type is None:
         raise ValueError("""
@@ -68,60 +67,6 @@ def check_method_port(service, method):
             the ports defined by the service class
             """)
 
-# FIXME: I don't think this is working.
-def _add_callbacks(service, root, types, service_name, url):
-    ns_tns = service.get_tns()
-    pref_tns = 'tns'
-
-    cb_port_type = None
-
-    # add necessary async headers
-    # WS-Addressing -> RelatesTo ReplyTo MessageID
-    # callback porttype
-    if service._has_callbacks():
-        wsa_schema = etree.SubElement(types, "{%s}schema" % _ns_xsd)
-        wsa_schema.set("targetNamespace", '%sCallback'  % ns_tns)
-        wsa_schema.set("elementFormDefault", "qualified")
-
-        import_ = etree.SubElement(wsa_schema, "{%s}import" % _ns_xsd)
-        import_.set("namespace", _ns_wsa)
-        import_.set("schemaLocation", _ns_wsa)
-
-        relt_message = etree.SubElement(root, '{%s}message' % _ns_wsdl)
-        relt_message.set('name', 'RelatesToHeader')
-        relt_part = etree.SubElement(relt_message, '{%s}part' % _ns_wsdl)
-        relt_part.set('name', 'RelatesTo')
-        relt_part.set('element', '%s:RelatesTo' % _pref_wsa)
-
-        reply_message = etree.SubElement(root, '{%s}message' % _ns_wsdl)
-        reply_message.set('name', 'ReplyToHeader')
-        reply_part = etree.SubElement(reply_message, '{%s}part' % _ns_wsdl)
-        reply_part.set('name', 'ReplyTo')
-        reply_part.set('element', '%s:ReplyTo' % _pref_wsa)
-
-        id_header = etree.SubElement(root, '{%s}message' % _ns_wsdl)
-        id_header.set('name', 'MessageIDHeader')
-        id_part = etree.SubElement(id_header, '{%s}part' % _ns_wsdl)
-        id_part.set('name', 'MessageID')
-        id_part.set('element', '%s:MessageID' % _pref_wsa)
-
-        # make portTypes
-        cb_port_type = etree.SubElement(root, '{%s}portType' % _ns_wsdl)
-        cb_port_type.set('name', '%sCallback' % service_name)
-
-        cb_service_name = '%sCallback' % service_name
-
-        cb_service = etree.SubElement(root, '{%s}service' % _ns_wsdl)
-        cb_service.set('name', cb_service_name)
-
-        cb_wsdl_port = etree.SubElement(cb_service, '{%s}port' % _ns_wsdl)
-        cb_wsdl_port.set('name', cb_service_name)
-        cb_wsdl_port.set('binding', '%s:%s' % (pref_tns, cb_service_name))
-
-        cb_address = etree.SubElement(cb_wsdl_port, '{%s}address' % _ns_soap)
-        cb_address.set('location', url)
-
-    return cb_port_type
 
 class Wsdl11(XmlSchema):
     """The implementation of the Wsdl 1.1 interface definition document
@@ -168,7 +113,7 @@ class Wsdl11(XmlSchema):
         pt = None
 
         if not pt_name in self.port_type_dict:
-            pt = etree.SubElement(self.root_elt, '{%s}portType' % _ns_wsdl)
+            pt = SubElement(self.root_elt, WSDL11("portType"))
             pt.set('name', pt_name)
             self.port_type_dict[pt_name] = pt
 
@@ -182,7 +127,7 @@ class Wsdl11(XmlSchema):
 
         ser = None
         if not service_name in self.service_elt_dict:
-            ser = etree.SubElement(self.root_elt, '{%s}service' % _ns_wsdl)
+            ser = SubElement(self.root_elt, WSDL11("service"))
             ser.set('name', service_name)
             self.service_elt_dict[service_name] = ser
 
@@ -204,14 +149,14 @@ class Wsdl11(XmlSchema):
         service_name = self.interface.get_name()
 
         # create wsdl root node
-        self.root_elt = root = etree.Element("{%s}definitions" % _ns_wsdl,
+        self.root_elt = root = etree.Element(WSDL11("definitions"),
                                                      nsmap=self.interface.nsmap)
 
         root.set('targetNamespace', self.interface.tns)
         root.set('name', service_name)
 
         # create types node
-        types = etree.SubElement(root, "{%s}types" % _ns_wsdl)
+        types = SubElement(root, WSDL11("types"))
         for s in self.schema_dict.values():
             types.append(s)
 
@@ -220,7 +165,7 @@ class Wsdl11(XmlSchema):
             self.add_messages_for_methods(s, root, messages)
 
         if self._with_plink:
-            plink = etree.SubElement(root, '{%s}partnerLinkType' % _ns_plink)
+            plink = SubElement(root, PLINK("partnerLinkType"))
             plink.set('name', service_name)
             self.__add_partner_link(service_name, plink)
 
@@ -257,17 +202,17 @@ class Wsdl11(XmlSchema):
         ns_tns = self.interface.tns
         pref_tns = self.interface.get_namespace_prefix(ns_tns)
 
-        role = etree.SubElement(plink, '{%s}role' % _ns_plink)
+        role = SubElement(plink, PLINK("role"))
         role.set('name', service_name)
 
-        plink_port_type = etree.SubElement(role, '{%s}portType' % _ns_plink)
+        plink_port_type = SubElement(role, PLINK("portType"))
         plink_port_type.set('name', '%s:%s' % (pref_tns, service_name))
 
         if self._has_callbacks():
-            role = etree.SubElement(plink, '{%s}role' % _ns_plink)
+            role = SubElement(plink, PLINK("role"))
             role.set('name', '%sCallback' % service_name)
 
-            plink_port_type = etree.SubElement(role, '{%s}portType' % _ns_plink)
+            plink_port_type = SubElement(role, PLINK("portType"))
             plink_port_type.set('name', '%s:%sCallback' %
                                                        (pref_tns, service_name))
 
@@ -276,11 +221,11 @@ class Wsdl11(XmlSchema):
 
         pref_tns = self.interface.get_namespace_prefix(self.interface.tns)
 
-        wsdl_port = etree.SubElement(service, '{%s}port' % _ns_wsdl)
+        wsdl_port = SubElement(service, WSDL11("port"))
         wsdl_port.set('name', port_name)
         wsdl_port.set('binding', '%s:%s' % (pref_tns, binding_name))
 
-        addr = etree.SubElement(wsdl_port, '{%s}address' % _ns_soap)
+        addr = SubElement(wsdl_port, WSDL11_SOAP("address"))
         addr.set('location', self.url)
 
     def _has_callbacks(self):
@@ -305,7 +250,8 @@ class Wsdl11(XmlSchema):
 
     def add_port_type(self, service, root, service_name, types, url):
         # FIXME: I don't think this call is working.
-        cb_port_type = _add_callbacks(service, root, types, service_name, url)
+        cb_port_type = self._add_callbacks(service, root, types,
+                                                              service_name, url)
         applied_service_name = self._get_applied_service_name(service)
 
         port_binding_names = []
@@ -329,35 +275,31 @@ class Wsdl11(XmlSchema):
             check_method_port(service, method)
 
             if method.is_callback:
-                operation = etree.SubElement(cb_port_type, '{%s}operation'
-                                                                    % _ns_wsdl)
+                operation = SubElement(cb_port_type, WSDL11("operation"))
             else:
-                operation = etree.SubElement(port_type, '{%s}operation'
-                                                                    % _ns_wsdl)
+                operation = SubElement(port_type, WSDL11("operation"))
 
             operation.set('name', method.operation_name)
 
             if method.doc is not None:
-                documentation = etree.SubElement(operation, '{%s}documentation'
-                                                                    % _ns_wsdl)
-                documentation.text = method.doc
+                operation.append(E(WSDL11("documentation"), method.doc))
 
             operation.set('parameterOrder', method.in_message.get_element_name())
 
-            op_input = etree.SubElement(operation, '{%s}input' % _ns_wsdl)
+            op_input = SubElement(operation, WSDL11("input"))
             op_input.set('name', method.in_message.get_element_name())
-            op_input.set('message', method.in_message.get_element_name_ns(self.interface))
+            op_input.set('message',
+                          method.in_message.get_element_name_ns(self.interface))
 
             if (not method.is_callback) and (not method.is_async):
-                op_output = etree.SubElement(operation, '{%s}output' % _ns_wsdl)
+                op_output = SubElement(operation, WSDL11("output"))
                 op_output.set('name', method.out_message.get_element_name())
                 op_output.set('message', method.out_message.get_element_name_ns(
                                                                 self.interface))
 
                 if not (method.faults is None):
                     for f in method.faults:
-                        fault = etree.SubElement(operation, '{%s}fault' %
-                                                                       _ns_wsdl)
+                        fault = SubElement(operation, WSDL11("fault"))
                         fault.set('name', f.get_type_name())
                         fault.set('message', '%s:%s' % (
                                         f.get_namespace_prefix(self.interface),
@@ -371,7 +313,7 @@ class Wsdl11(XmlSchema):
         if obj is not None and not (message_name in messages):
             messages.add(message_name)
 
-            message = etree.SubElement(root, '{%s}message' % _ns_wsdl)
+            message = SubElement(root, WSDL11("message"))
             message.set('name', message_name)
 
             if isinstance(obj, (list, tuple)):
@@ -380,7 +322,7 @@ class Wsdl11(XmlSchema):
                 objs = (obj,)
 
             for obj in objs:
-                part = etree.SubElement(message, '{%s}part' % _ns_wsdl)
+                part = SubElement(message, WSDL11("part"))
                 part.set('name', obj.get_element_name())
                 part.set('element', obj.get_element_name_ns(self.interface))
 
@@ -416,22 +358,21 @@ class Wsdl11(XmlSchema):
     def add_bindings_for_methods(self, service, root, service_name,
                                      cb_binding):
 
-        pref_tns = self.interface.get_namespace_prefix(service.get_tns())
+        pref_tns = self.interface.get_namespace_prefix(self.interface.get_tns())
 
         def inner(method, binding):
-            operation = etree.Element('{%s}operation' % _ns_wsdl)
+            operation = etree.Element(WSDL11("operation"))
             operation.set('name', method.operation_name)
 
-            soap_operation = etree.SubElement(operation, '{%s}operation'
-                                                                    % _ns_soap)
+            soap_operation = SubElement(operation, WSDL11_SOAP("operation"))
             soap_operation.set('soapAction', method.operation_name)
             soap_operation.set('style', 'document')
 
             # get input
-            input = etree.SubElement(operation, '{%s}input' % _ns_wsdl)
+            input = SubElement(operation, WSDL11("input"))
             input.set('name', method.in_message.get_element_name())
 
-            soap_body = etree.SubElement(input, '{%s}body' % _ns_soap)
+            soap_body = SubElement(input, WSDL11_SOAP("body"))
             soap_body.set('use', 'literal')
 
             # get input soap header
@@ -452,7 +393,7 @@ class Wsdl11(XmlSchema):
                     in_header_message_name = in_headers[0].get_type_name()
 
                 for header in in_headers:
-                    soap_header = etree.SubElement(input, '{%s}header' % _ns_soap)
+                    soap_header = SubElement(input, WSDL11_SOAP('header'))
                     soap_header.set('use', 'literal')
                     soap_header.set('message', '%s:%s' % (
                                 header.get_namespace_prefix(self.interface),
@@ -460,10 +401,10 @@ class Wsdl11(XmlSchema):
                     soap_header.set('part', header.get_type_name())
 
             if not (method.is_async or method.is_callback):
-                output = etree.SubElement(operation, '{%s}output' % _ns_wsdl)
+                output = SubElement(operation, WSDL11("output"))
                 output.set('name', method.out_message.get_element_name())
 
-                soap_body = etree.SubElement(output, '{%s}body' % _ns_soap)
+                soap_body = SubElement(output, WSDL11_SOAP("body"))
                 soap_body.set('use', 'literal')
 
                 # get output soap header
@@ -484,8 +425,7 @@ class Wsdl11(XmlSchema):
                         out_header_message_name = out_headers[0].get_type_name()
 
                     for header in out_headers:
-                        soap_header = etree.SubElement(output, '{%s}header'
-                                                                    % _ns_soap)
+                        soap_header = SubElement(output, WSDL11_SOAP("header"))
                         soap_header.set('use', 'literal')
                         soap_header.set('message', '%s:%s' % (
                                 header.get_namespace_prefix(self.interface),
@@ -494,17 +434,15 @@ class Wsdl11(XmlSchema):
 
                 if not (method.faults is None):
                     for f in method.faults:
-                        wsdl_fault = etree.SubElement(operation, '{%s}fault' %
-                                                                    _ns_wsdl)
+                        wsdl_fault = SubElement(operation, WSDL11("fault"))
                         wsdl_fault.set('name', f.get_type_name())
 
-                        soap_fault = etree.SubElement(wsdl_fault, '{%s}fault' %
-                                                                    _ns_soap)
+                        soap_fault = SubElement(wsdl_fault, WSDL11_SOAP("fault"))
                         soap_fault.set('name', f.get_type_name())
                         soap_fault.set('use', 'literal')
 
             if method.is_callback:
-                relates_to = etree.SubElement(input, '{%s}header' % _ns_soap)
+                relates_to = SubElement(input, WSDL11_SOAP("header"))
 
                 relates_to.set('message', '%s:RelatesToHeader' % pref_tns)
                 relates_to.set('part', 'RelatesTo')
@@ -514,12 +452,12 @@ class Wsdl11(XmlSchema):
 
             else:
                 if method.is_async:
-                    rt_header = etree.SubElement(input, '{%s}header' % _ns_soap)
+                    rt_header = SubElement(input, WSDL11_SOAP("header"))
                     rt_header.set('message', '%s:ReplyToHeader' % pref_tns)
                     rt_header.set('part', 'ReplyTo')
                     rt_header.set('use', 'literal')
 
-                    mid_header = etree.SubElement(input, '{%s}header'% _ns_soap)
+                    mid_header = SubElement(input, WSDL11_SOAP("header"))
                     mid_header.set('message', '%s:MessageIDHeader' % pref_tns)
                     mid_header.set('part', 'MessageID')
                     mid_header.set('use', 'literal')
@@ -531,11 +469,11 @@ class Wsdl11(XmlSchema):
             for port_type_name in port_type_list:
 
                 # create binding nodes
-                binding = etree.SubElement(root, '{%s}binding' % _ns_wsdl)
+                binding = SubElement(root, WSDL11("binding"))
                 binding.set('name', port_type_name)
                 binding.set('type', '%s:%s'% (pref_tns, port_type_name))
 
-                transport = etree.SubElement(binding, '{%s}binding' % _ns_soap)
+                transport = SubElement(binding, WSDL11_SOAP("binding"))
                 transport.set('style', 'document')
 
                 for m in service.public_methods.values():
@@ -545,11 +483,11 @@ class Wsdl11(XmlSchema):
         else:
             # here is the default port.
             if cb_binding is None:
-                cb_binding = etree.SubElement(root, '{%s}binding' % _ns_wsdl)
+                cb_binding = SubElement(root, WSDL11("binding"))
                 cb_binding.set('name', service_name)
                 cb_binding.set('type', '%s:%s'% (pref_tns, service_name))
 
-                transport = etree.SubElement(cb_binding, '{%s}binding' % _ns_soap)
+                transport = SubElement(cb_binding, WSDL11_SOAP("binding"))
                 transport.set('style', 'document')
                 transport.set('transport', self.interface.app.transport)
 
@@ -557,3 +495,58 @@ class Wsdl11(XmlSchema):
                 inner(m, cb_binding)
 
         return cb_binding
+
+    # FIXME: I don't think this is working.
+    def _add_callbacks(self, service, root, types, service_name, url):
+        ns_tns = self.interface.get_tns()
+        pref_tns = 'tns'
+
+        cb_port_type = None
+
+        # add necessary async headers
+        # WS-Addressing -> RelatesTo ReplyTo MessageID
+        # callback porttype
+        if service._has_callbacks():
+            wsa_schema = SubElement(types, XSD("schema"))
+            wsa_schema.set("targetNamespace", '%sCallback'  % ns_tns)
+            wsa_schema.set("elementFormDefault", "qualified")
+
+            import_ = SubElement(wsa_schema, XSD("import"))
+            import_.set("namespace", NS_WSA)
+            import_.set("schemaLocation", NS_WSA)
+
+            relt_message = SubElement(root, WSDL11("message"))
+            relt_message.set('name', 'RelatesToHeader')
+            relt_part = SubElement(relt_message, WSDL11("part"))
+            relt_part.set('name', 'RelatesTo')
+            relt_part.set('element', '%s:RelatesTo' % PREF_WSA)
+
+            reply_message = SubElement(root, WSDL11("message"))
+            reply_message.set('name', 'ReplyToHeader')
+            reply_part = SubElement(reply_message, WSDL11("part"))
+            reply_part.set('name', 'ReplyTo')
+            reply_part.set('element', '%s:ReplyTo' % PREF_WSA)
+
+            id_header = SubElement(root, WSDL11("message"))
+            id_header.set('name', 'MessageIDHeader')
+            id_part = SubElement(id_header, WSDL11("part"))
+            id_part.set('name', 'MessageID')
+            id_part.set('element', '%s:MessageID' % PREF_WSA)
+
+            # make portTypes
+            cb_port_type = SubElement(root, WSDL11("portType"))
+            cb_port_type.set('name', '%sCallback' % service_name)
+
+            cb_service_name = '%sCallback' % service_name
+
+            cb_service = SubElement(root, WSDL11("service"))
+            cb_service.set('name', cb_service_name)
+
+            cb_wsdl_port = SubElement(cb_service, WSDL11("port"))
+            cb_wsdl_port.set('name', cb_service_name)
+            cb_wsdl_port.set('binding', '%s:%s' % (pref_tns, cb_service_name))
+
+            cb_address = SubElement(cb_wsdl_port, WSDL11_SOAP("address"))
+            cb_address.set('location', url)
+
+        return cb_port_type
\ No newline at end of file
