@@ -21,20 +21,47 @@
 defining models.
 """
 
+import logging
+logger = logging.getLogger(__name__)
+
 import re
 
 import spyne.const.xml_ns
 
 from decimal import Decimal
 
-from spyne.util import Break
+from spyne import const
+from spyne.util import Break, six
+from spyne.util.cdict import cdict
 from spyne.util.odict import odict
-from spyne.util.six import add_metaclass
 
 from spyne.const.xml_ns import DEFAULT_NS
 
 
-# All this code to get rid of a one letter quirk: nillable vs nullable.
+def _decode_pa_dict(d):
+    """Decodes dict passed to prot_attrs.
+
+    >>> _decode_pa_dict({})
+    cdict({})
+    >>> _decode_pa_dict({1: 2)})
+    cdict({1: 2})
+    >>> _decode_pa_dict({(1,2): 3)})
+    cdict({1: 3, 2: 3})
+    """
+
+    retval = cdict()
+    for k, v in d.items():
+        if isinstance(k, tuple):
+            for subk in k:
+                retval[subk] = v
+
+    for k, v in d.items():
+        if not isinstance(k, tuple):
+            retval[k] = v
+
+    return retval
+
+
 class AttributesMeta(type(object)):
     NULLABLE_DEFAULT = True
 
@@ -46,6 +73,8 @@ class AttributesMeta(type(object)):
         return super(AttributesMeta, cls).__new__(cls, cls_name, cls_bases, cls_dict)
 
     def __init__(self, cls_name, cls_bases, cls_dict):
+        # you will probably want to look at ModelBase._s_customize as well.
+
         nullable = cls_dict.get('nullable', None)
         nillable = cls_dict.get('nillable', None)
         if nullable is not None:
@@ -56,9 +85,33 @@ class AttributesMeta(type(object)):
             assert nullable is None or nullable == nillable
             self._nullable = nillable
 
-        if getattr(self, '_nullable', None) is None:
+        if not hasattr(self, '_nullable'):
             self._nullable = None
 
+        if not hasattr(self, '_default_factory'):
+            self._default_factory = None
+
+        if not hasattr(self, '_html_cloth'):
+            self._html_cloth = None
+        if not hasattr(self, '_html_root_cloth'):
+            self._html_root_cloth = None
+
+        if 'html_cloth' in cls_dict:
+            self.set_html_cloth(cls_dict.pop('html_cloth'))
+        if 'html_root_cloth' in cls_dict:
+            self.set_html_cloth(cls_dict.pop('html_root_cloth'))
+
+        if not hasattr(self, '_xml_cloth'):
+            self._xml_cloth = None
+        if not hasattr(self, '_xml_root_cloth'):
+            self._xml_root_cloth = None
+
+        if 'xml_cloth' in cls_dict:
+            self.set_xml_cloth(cls_dict.pop('xml_cloth'))
+
+        if 'xml_root_cloth' in cls_dict:
+            self.set_xml_cloth(cls_dict.pop('xml_root_cloth'))
+
         super(AttributesMeta, self).__init__(cls_name, cls_bases, cls_dict)
 
     def get_nullable(self):
@@ -78,7 +131,63 @@ class AttributesMeta(type(object)):
 
     nillable = property(get_nillable, set_nillable)
 
+    def get_default_factory(self):
+        return self._default_factory
 
+    def set_default_factory(self, what):
+        self._default_factory = staticmethod(what)
+
+    default_factory = property(get_default_factory, set_default_factory)
+
+    def get_html_cloth(self):
+        return self._html_cloth
+    def set_html_cloth(self, what):
+        from spyne.protocol.cloth.to_cloth import ClothParserMixin
+        cm = ClothParserMixin.from_html_cloth(what)
+        if cm._root_cloth is not None:
+            self._html_root_cloth = cm._root_cloth
+        elif cm._cloth is not None:
+            self._html_cloth = cm._cloth
+        else:
+            raise Exception("%r is not a suitable cloth", what)
+    html_cloth = property(get_html_cloth, set_html_cloth)
+
+    def get_html_root_cloth(self):
+        return self._html_root_cloth
+    html_root_cloth = property(get_html_root_cloth)
+
+    def get_xml_cloth(self):
+        return self._xml_cloth
+    def set_xml_cloth(self, what):
+        from spyne.protocol.cloth.to_cloth import ClothParserMixin
+        cm = ClothParserMixin.from_xml_cloth(what)
+        if cm._root_cloth is not None:
+            self._xml_root_cloth = cm._root_cloth
+        elif cm._cloth is not None:
+            self._xml_cloth = cm._cloth
+        else:
+            raise Exception("%r is not a suitable cloth", what)
+    xml_cloth = property(get_xml_cloth, set_xml_cloth)
+
+    def get_xml_root_cloth(self):
+        return self._xml_root_cloth
+    xml_root_cloth = property(get_xml_root_cloth)
+
+
+class ModelBaseMeta(type(object)):
+    def __getitem__(self, item):
+        return self.customize(**item)
+
+    def customize(self, **kwargs):
+        """Duplicates cls and overwrites the values in ``cls.Attributes`` with
+        ``**kwargs`` and returns the new class."""
+
+        cls_name, cls_bases, cls_dict = ModelBase._s_customize(self, **kwargs)
+
+        return type(cls_name, cls_bases, cls_dict)
+
+
+@six.add_metaclass(ModelBaseMeta)
 class ModelBase(object):
     """The base class for type markers. It defines the model interface for the
     interface generators to use and also manages class customizations that are
@@ -102,13 +211,16 @@ class ModelBase(object):
     """The public type name of the class. Use ``get_type_name()`` instead of
     accessing it directly."""
 
+    Value = type(None)
+    """The value of this type is an instance of this class"""
+
     # These are not the xml schema defaults. The xml schema defaults are
     # considered in XmlSchema's add() method. the defaults here are to reflect
     # what people seem to want most.
     #
     # Please note that min_occurs and max_occurs must be validated in the
     # ComplexModelBase deserializer.
-    @add_metaclass(AttributesMeta)
+    @six.add_metaclass(AttributesMeta)
     class Attributes(object):
         """The class that holds the constraints for the given type."""
 
@@ -117,10 +229,10 @@ class ModelBase(object):
         # are skipped. just for internal use.
 
         default = None
-        """The default value if the input is None"""
+        """The default value if the input is None."""
 
         default_factory = None
-        """The default value if the input is None"""
+        """The callable that produces a default value if the input is None."""
 
         nillable = None
         """Set this to false to reject null values. Synonyms with
@@ -208,19 +320,22 @@ class ModelBase(object):
         * If the value is a string, the value will denote the indexing method
           used by the database. See: http://www.postgresql.org/docs/9.2/static/indexes-types.html
 
-        * If the vale is a tuple of two strings, the first value will denote the
-          index name and the second value will denote the indexing method as
+        * If the value is a tuple of two strings, the first value will denote
+          the index name and the second value will denote the indexing method as
           above.
         """
 
-        read_only= False
-        """If True, the attribute won't be initialized from outside values."""
+        read_only = False
+        """If True, the attribute won't be initialized from outside values.
+        Set this to ``True`` for e.g. read-only properties."""
 
         prot_attrs = None
-        """to be implemented."""
-        #"""Customize child attributes for protocols. It's a dict of dicts.
-        #The key is either a ProtocolBase subclass or a ProtocolBase instance.
-        #Instances override classes."""
+        """Customize child attributes for protocols. It's a dict of dicts.
+        The key is either a ProtocolBase subclass or a ProtocolBase instance.
+        Instances override classes."""
+
+        pa = None
+        """Alias for prot_attrs."""
 
         empty_is_none = False
         """When the incoming object is empty (e.g. '' for strings) treat it as
@@ -230,12 +345,17 @@ class ModelBase(object):
         """An integer that's passed to ``_type_info.insert()`` as first argument
          when not None. ``.append()`` is used otherwise."""
 
+        polymap = {}
+        """A dict of classes that override polymorphic substitions for classes
+        given as keys to classes given as values."""
+
     class Annotations(object):
         """The class that holds the annotations for the given type."""
 
         __use_parent_doc__ = False
-        """If set to True Annotations will use __doc__ from parent,
-        This is a convenience option"""
+        """If equal to True and doc is empty, Annotations will use __doc__
+        from parent. Set it to False to avoid this mechanism. This is a
+        convenience option"""
 
         doc = ""
         """The public documentation for the given type."""
@@ -272,6 +392,17 @@ class ModelBase(object):
 
         return cls.__namespace__
 
+    @classmethod
+    def _fill_empty_type_name(cls, parent_ns, parent_tn, k):
+        cls.__namespace__ = parent_ns
+
+        cls.__type_name__ = "%s_%s%s" % (parent_tn, k, const.TYPE_SUFFIX)
+        extends = cls.__extends__
+        while extends is not None and extends.__type_name__ is ModelBase.Empty:
+            cls.__extends__._fill_empty_type_name(cls.get_namespace(),
+                               cls.get_type_name(), k + const.PARENT_SUFFIX)
+            extends = extends.__extends__
+
     # TODO: rename to "resolve_identifier"
     @staticmethod
     def resolve_namespace(cls, default_ns, tags=None):
@@ -309,6 +440,9 @@ class ModelBase(object):
         if cls.__namespace__ is None or len(cls.__namespace__) == 0:
             raise ValueError("You need to explicitly set %r.__namespace__" % cls)
 
+        # too slow
+        # logger.debug("    resolve ns for %r to %r", cls, cls.__namespace__)
+
         if getattr(cls, '__extends__', None) != None:
             cls.__extends__.resolve_namespace(cls.__extends__, default_ns, tags)
 
@@ -325,6 +459,7 @@ class ModelBase(object):
 
         return retval
 
+    # FIXME: Rename this to get_type_name_with_ns_pref
     @classmethod
     def get_type_name_ns(cls, interface):
         """Returns the type name with a namespace prefix, separated by a column.
@@ -349,18 +484,26 @@ class ModelBase(object):
 
     @classmethod
     def to_string(cls, value):
-        """Returns str(value). This should be overridden if this is not enough.
+        """
+        Returns str(value). This should be overridden if this is not enough.
         """
         return str(value)
 
     @classmethod
-    def customize(cls, **kwargs):
-        """Duplicates cls and overwrites the values in ``cls.Attributes`` with
-        ``**kwargs`` and returns the new class."""
+    def to_unicode(cls, value):
+        """
+        Returns unicode(value). This should be overridden if this is not enough.
+        """
+        return six.text_type(value)
 
-        cls_name, cls_bases, cls_dict = cls._s_customize(cls, **kwargs)
-
-        return type(cls_name, cls_bases, cls_dict)
+    @classmethod
+    def get_documentation(cls):
+        if cls.Annotations.doc:
+            return cls.Annotations.doc
+        elif cls.Annotations.__use_parent_doc__:
+            return cls.__doc__
+        else:
+            return ''
 
     @staticmethod
     def _s_customize(cls, **kwargs):
@@ -370,10 +513,12 @@ class ModelBase(object):
         Not meant to be overridden.
         """
 
-        cls_dict = odict({'__module__': cls.__module__})
+        cls_dict = odict({'__module__': cls.__module__, '__doc__': cls.__doc__})
 
         if getattr(cls, '__orig__', None) is None:
             cls_dict['__orig__'] = cls
+        else:
+            cls_dict['__orig__'] = cls.__orig__
 
         class Attributes(cls.Attributes):
             pass
@@ -382,27 +527,52 @@ class ModelBase(object):
             Attributes.translations = {}
         if cls.Attributes.sqla_column_args is None:
             Attributes.sqla_column_args = (), {}
-
         cls_dict['Attributes'] = Attributes
 
-        # as nillable is a property, it gets reset everytime a new class is
-        # defined. So we need to reinitialize it explicitly.
-        Attributes.nillable = cls.Attributes.nillable
+        # properties get reset everytime a new class is defined. So we need
+        # to reinitialize them explicitly.
+        for k in ('nillable', '_xml_cloth', '_xml_root_cloth', '_html_cloth',
+                                                            '_html_root_cloth'):
+            v = getattr(cls.Attributes, k)
+            if v is not None:
+                setattr(Attributes, k, v)
 
         class Annotations(cls.Annotations):
             pass
         cls_dict['Annotations'] = Annotations
 
+        # get protocol attrs
+        prot = kwargs.get('protocol', None)
+        if prot is None:
+            prot = kwargs.get('prot', None)
+        if prot is None:
+            prot = kwargs.get('p', None)
+        if prot is not None and len(prot.type_attrs) > 0:
+            # if there is a class customization from protocol, do it
+
+            type_attrs = prot.type_attrs.copy()
+            type_attrs.update(kwargs)
+            logger.debug("%r: kwargs %r => %r from prot typeattr %r",
+                                       cls, kwargs, type_attrs, prot.type_attrs)
+            kwargs = type_attrs
+
         for k, v in kwargs.items():
             if k.startswith('_'):
                 continue
 
+            if k in ('protocol', 'prot', 'p'):
+                setattr(Attributes, 'prot', v)
+
             elif k in ("doc", "appinfo"):
                 setattr(Annotations, k, v)
 
             elif k in ('primary_key', 'pk'):
+                setattr(Attributes, 'primary_key', v)
                 Attributes.sqla_column_args[-1]['primary_key'] = v
 
+            elif k in ('protocol_attrs', 'prot_attrs', 'pa'):
+                setattr(Attributes, 'prot_attrs', _decode_pa_dict(v))
+
             elif k in ('foreign_key', 'fk'):
                 from sqlalchemy.schema import ForeignKey
                 t, d = Attributes.sqla_column_args
@@ -412,6 +582,13 @@ class ModelBase(object):
             elif k in ('autoincrement', 'onupdate', 'server_default'):
                 Attributes.sqla_column_args[-1][k] = v
 
+            elif k == 'values_dict':
+                assert not 'values' in v, "`values` and `values_dict` can't be" \
+                                          "specified at the same time"
+
+                Attributes.values = v.keys()
+                Attributes.values_dict = v
+
             elif k == 'max_occurs' and v in ('unbounded', 'inf', float('inf')):
                 setattr(Attributes, k, Decimal('inf'))
 
@@ -457,19 +634,36 @@ class SimpleModelAttributesMeta(AttributesMeta):
 
     pattern = property(get_pattern, set_pattern)
 
+    def get_unicode_pattern(self):
+        return self._pattern
+
+    def set_unicode_pattern(self, pattern):
+        self._pattern = pattern
+        if pattern is not None:
+            self._pattern_re = re.compile(pattern, re.UNICODE)
+
+    unicode_pattern = property(get_unicode_pattern, set_unicode_pattern)
+    upattern = property(get_unicode_pattern, set_unicode_pattern)
+
 
 class SimpleModel(ModelBase):
     """The base class for primitives."""
 
     __namespace__ = "http://www.w3.org/2001/XMLSchema"
 
-    @add_metaclass(SimpleModelAttributesMeta)
+    @six.add_metaclass(SimpleModelAttributesMeta)
     class Attributes(ModelBase.Attributes):
         """The class that holds the constraints for the given type."""
 
         values = set()
         """The set of possible values for this type."""
 
+        # some hacks are done in _s_customize to make `values_dict`
+        # behave like `values`
+        values_dict = dict()
+        """The dict of possible values for this type. Dict keys are values and
+        dict values are either a single string or a translation dict."""
+
         _pattern_re = None
 
     def __new__(cls, **kwargs):
@@ -504,12 +698,14 @@ class SimpleModel(ModelBase):
 
     @staticmethod
     def validate_native(cls, value):
-        return (     ModelBase.validate_native(cls, value)
-                and (cls.Attributes.values is None or
-                                            len(cls.Attributes.values) == 0 or (
-                     (value is None     and cls.Attributes.nillable) or
-                     (value is not None and value in cls.Attributes.values)
-                ))
+        return (ModelBase.validate_native(cls, value)
+                and (
+                    cls.Attributes.values is None or
+                    len(cls.Attributes.values) == 0 or (
+                        (value is None     and cls.Attributes.nillable) or
+                        (value is not None and value in cls.Attributes.values)
+                    )
+                )
             )
 
 
@@ -567,9 +763,10 @@ class xml:
         information. This is generally a stupid thing to do. Use with caution.
     """
 
-    def __init__(self, root_tag=None, no_ns=False):
+    def __init__(self, root_tag=None, no_ns=False, pretty_print=False):
         self.root_tag = root_tag
         self.no_ns = no_ns
+        self.pretty_print = pretty_print
 
 
 class table:
@@ -593,7 +790,10 @@ class table:
     """
 
     def __init__(self, multi=False, left=None, right=None, backref=None,
-            id_backref=None, cascade=False, lazy='select', back_populates=None):
+            id_backref=None, cascade=False, lazy='select', back_populates=None,
+                            fk_left_deferrable=None, fk_left_initially=None,
+                            fk_right_deferrable=None, fk_right_initially=None,
+                            explicit_join=False, order_by=False):
         self.multi = multi
         self.left = left
         self.right = right
@@ -602,6 +802,12 @@ class table:
         self.cascade = cascade
         self.lazy = lazy
         self.back_populates = back_populates
+        self.fk_left_deferrable = fk_left_deferrable
+        self.fk_left_initially = fk_left_initially
+        self.fk_right_deferrable = fk_right_deferrable
+        self.fk_right_initially = fk_right_initially
+        self.explicit_join = explicit_join
+        self.order_by = order_by
 
 
 class json:
