@@ -19,17 +19,21 @@
 
 """The ``spyne.model.binary`` package contains binary type markers."""
 
+import logging
+logger = logging.getLogger(__name__)
+
 import os
 import base64
 import tempfile
 
+from mmap import mmap, ACCESS_READ
 from base64 import b64encode
 from base64 import b64decode
 from base64 import urlsafe_b64encode
 from base64 import urlsafe_b64decode
 from binascii import hexlify
 from binascii import unhexlify
-from os.path import abspath, isdir
+from os.path import abspath, isdir, isfile, basename
 
 from spyne.util.six import StringIO
 from spyne.error import ValidationError
@@ -103,13 +107,14 @@ class ByteArray(SimpleModel):
 
     @classmethod
     def to_base64(cls, value):
-        return b64encode(_bytes_join(value))
+        return b64encode(b''.join(value))
 
     @classmethod
     def from_base64(cls, value):
+        joiner = type(value)()
         try:
-            return [b64decode(_bytes_join(value))]
-        except TypeError as e:
+            return [b64decode(joiner.join(value))]
+        except TypeError:
             raise ValidationError(value)
 
     @classmethod
@@ -148,8 +153,9 @@ binary_decoding_handlers = {
 
 
 class HybridFileStore(object):
-    def __init__(self, store_path, db_format='json'):
-        """Hybrid Sql/Filesystem store.
+    def __init__(self, store_path, db_format='json', type=None):
+        """Marker to be passed to File's store_as to denote a hybrid
+        Sql/Filesystem storage scheme.
 
         :param store_path: The path where the file contents are stored. This is
             converted to an absolute path if it's not already one.
@@ -159,6 +165,7 @@ class HybridFileStore(object):
 
         self.store = abspath(store_path)
         self.db_format = db_format
+        self.type = type
 
         if not isdir(self.store):
             os.makedirs(self.store)
@@ -166,6 +173,81 @@ class HybridFileStore(object):
         assert isdir(self.store)
 
 
+_BINARY = type('FileTypeBinary', (object,), {})
+_TEXT = type('FileTypeText', (object,), {})
+
+
+class _Value(ComplexModel):
+    """The class for values marked as ``File``.
+
+    :param name: Original name of the file
+    :param path: Current path to the file.
+    :param type: The mime type of the file's contents.
+    :param data: Optional sequence of ``str`` or ``bytes`` instances
+        that contain the file's data.
+    :param handle: :class:`file` object that contains the file's data.
+        It is ignored unless the ``path`` argument is ``None``.
+    """
+
+    _type_info = [
+        ('name', Unicode(encoding='utf8')),
+        ('type', Unicode),
+        ('data', ByteArray(logged='len')),
+    ]
+
+    def __init__(self, name=None, path=None, type='application/octet-stream',
+                                            data=None, handle=None, move=False):
+
+        self.name = name
+        if self.name is not None:
+            if not os.path.basename(self.name) == self.name:
+                raise ValidationError(self.name,
+                             "File name %r should not contain any '/' char")
+
+        self.path = path
+        self.type = type
+        self.data = data
+        self.handle = handle
+        self.move = move
+        self.abspath = None
+        if self.path is not None:
+            self.abspath = abspath(self.path)
+
+    def rollover(self):
+        """This method normalizes the file object by making ``path``,
+        ``name`` and ``handle`` properties consistent. It writes
+        incoming data to the file object and points the ``data`` iterable
+        to the contents of this file.
+        """
+
+        if self.data is not None:
+            if self.path is None:
+                self.handle = tempfile.NamedTemporaryFile()
+                self.name = self.path = self.handle.name
+            else:
+                self.handle = open(self.path, 'wb')
+
+            # data is a ByteArray, so a sequence of str/bytes objects
+            for d in self.data:
+                self.handle.write(d)
+
+        elif self.handle is not None:
+            self.data = mmap(self.handle.fileno(), 0)  # 0 = whole file
+
+        elif self.path is not None:
+            if not isfile(self.path):
+                logger.error("File path in %r not found", self)
+
+            self.handle = open(self.path, 'rb')
+            self.data = mmap(self.handle.fileno(), 0, access=ACCESS_READ)
+            self.abspath = abspath(self.path)
+            self.name = self.path = basename(self.path)
+
+        else:
+            raise ValueError("Invalid file object passed in. All of "
+                                   ".data, .handle and .path are None.")
+
+
 class File(SimpleModel):
     """A compact way of dealing with incoming files for protocols with a
     standard way of encoding file metadata along with binary data. (E.g. Http)
@@ -174,6 +256,10 @@ class File(SimpleModel):
     __type_name__ = 'base64Binary'
     __namespace__ = "http://www.w3.org/2001/XMLSchema"
 
+    BINARY = _BINARY
+    TEXT = _BINARY
+    Value = _Value
+
     class Attributes(SimpleModel.Attributes):
         encoding = BINARY_ENCODING_USE_DEFAULT
         """The binary encoding to use when the protocol does not enforce an
@@ -182,67 +268,17 @@ class File(SimpleModel):
         One of (None, 'base64', 'hex')
         """
 
-    class Value(ComplexModel):
-        """The class for values marked as ``File``.
-
-        :param name: Original name of the file
-        :param path: Current path to the file.
-        :param type: The mime type of the file's contents.
-        :param data: Optional sequence of ``str`` or ``bytes`` instances
-            that contain the file's data.
-        :param handle: :class:`file` object that contains the file's data.
-            It is ignored unless the ``path`` argument is ``None``.
+        type = _Value
+        """The native type used to serialize the information in the file object.
         """
 
-        _type_info = [
-            ('name', Unicode(encoding='utf8')),
-            ('type', Unicode),
-            ('data', ByteArray(logged=False)),
-        ]
+        contents = _BINARY
+        """Set this to type=File.TEXT if you're sure you're handling unicode
+        data. This lets serializers like HtmlCloth avoid base64 encoding. Do
+        note that you still need to set encoding attribute explicitly to None!..
 
-        def __init__(self, name=None, path=None, type='application/octet-stream',
-                                            data=None, handle=None, move=False):
-
-            self.name = name
-            if self.name is not None:
-                if not os.path.basename(self.name) == self.name:
-                    raise ValidationError(self.name,
-                                 "File name %r should not contain any '/' char")
-
-            self.path = path
-            self.type = type
-            self.data = data
-            self.handle = handle
-            self.move = move
-            self.abspath = None
-            if self.path is not None:
-                self.abspath = abspath(self.path)
-
-        def rollover(self):
-            """This method normalizes the file object by making ``path``,
-            ``name`` and ``handle`` properties consistent. It writes
-            incoming data to the file object and points the ``data`` iterable
-            to the contents of this file.
-            """
-
-            iter(self.data)
-
-            if self.path is None:
-                handle, self.path = tempfile.mkstemp()
-                f = os.fdopen(handle, 'wb')
-            else:
-                assert os.path.isabs(self.path)
-                f = open(self.path, 'wb')
-
-            if self.name is None:
-                self.name = os.path.basename(self.path)
-
-            for data in self.data:
-                f.write(data)
-
-            f.close()
-
-            self.data = None
+        One of (File.BINARY, File.TEXT)
+        """
 
     @classmethod
     def to_base64(cls, value):
