@@ -23,14 +23,18 @@ subclasses. These are mainly container classes for other simple or
 complex objects -- they don't carry any data by themselves.
 """
 
+from __future__ import print_function
+
 import logging
 logger = logging.getLogger(__name__)
 
 import decimal
 
+from copy import copy
 from weakref import WeakKeyDictionary
 from collections import deque
 from inspect import isclass
+from itertools import chain
 
 from spyne import BODY_STYLE_BARE, BODY_STYLE_WRAPPED, BODY_STYLE_EMPTY
 from spyne import const
@@ -55,11 +59,22 @@ from spyne.util.six import add_metaclass, with_metaclass, string_types
 
 PSSM_VALUES = {'json': json, 'xml': xml, 'msgpack': msgpack, 'table': table}
 
+
+def _is_under_pydev_debugger():
+    import inspect
+    for frame in inspect.stack():
+        if frame[1].endswith("pydevd.py"):
+            return True
+    return False
+
+
 def _get_flat_type_info(cls, retval):
+    assert isinstance(retval, TypeInfo)
     parent = getattr(cls, '__extends__', None)
     if parent != None:
         _get_flat_type_info(parent, retval)
     retval.update(cls._type_info)
+    retval.alt.update(cls._type_info_alt) # FIXME: move to cls._type_info.alt
     return retval
 
 
@@ -67,6 +82,7 @@ class TypeInfo(odict):
     def __init__(self, *args, **kwargs):
         super(TypeInfo, self).__init__(*args, **kwargs)
         self.attributes = {}
+        self.alt = {}
 
     def __setitem__(self, key, val):
         assert isinstance(key, string_types)
@@ -74,13 +90,14 @@ class TypeInfo(odict):
 
 
 class _SimpleTypeInfoElement(object):
-    __slots__ = ['path', 'parent', 'type', 'is_array']
+    __slots__ = ['path', 'parent', 'type', 'is_array', 'can_be_empty']
 
-    def __init__(self, path, parent, type_, is_array):
+    def __init__(self, path, parent, type_, is_array, can_be_empty):
         self.path = path
         self.parent = parent
         self.type = type_
         self.is_array = is_array
+        self.can_be_empty = can_be_empty
 
     def __repr__(self):
         return "SimpleTypeInfoElement(path=%r, parent=%r, type=%r, is_array=%r)" \
@@ -109,6 +126,24 @@ class XmlModifier(ModelBase):
         if cls.__namespace__ in xml_ns.const_prefmap:
             cls.__namespace__ = default_ns
 
+    @classmethod
+    def _fill_empty_type_name(cls, parent_ns, parent_tn, k):
+        cls.__namespace__ = parent_ns
+        tn = "%s_%s%s" % (parent_tn, k, const.TYPE_SUFFIX)
+
+        child_v = cls.type
+        child_v.__type_name__ = tn
+
+        cls._type_info = TypeInfo({tn: child_v})
+        cls.__type_name__ = '%s%s%s' % (const.ARRAY_PREFIX, tn,
+                                                         const.ARRAY_SUFFIX)
+
+        extends = child_v.__extends__
+        while extends is not None and extends.get_type_name() is cls.Empty:
+            extends._fill_empty_type_name(parent_ns, parent_tn,
+                                                    k + const.PARENT_SUFFIX)
+            extends = extends.__extends__
+
 
 class XmlData(XmlModifier):
     """Items which are marshalled as data of the parent element."""
@@ -174,10 +209,25 @@ class SelfReference(object):
     """Use this as a placeholder type in classes that contain themselves. See
     :func:`spyne.test.model.test_complex.TestComplexModel.test_self_reference`.
     """
+    customize_args = []
+    customize_kwargs = {}
+    __orig__ = None
 
     def __init__(self):
         raise NotImplementedError()
 
+    @classmethod
+    def customize(cls, *args, **kwargs):
+        args = chain(args, cls.customize_args)
+        kwargs = dict(chain(kwargs.items(), cls.customize_kwargs.items()))
+        if cls.__orig__ is None:
+            cls.__orig__ = cls
+
+        return type("SelfReference", (cls,), {
+            'customize_args': args,
+            'customize_kwargs': kwargs,
+        })
+
 
 def _get_spyne_type(cls_name, k, v):
     try:
@@ -230,29 +280,54 @@ def _gen_attrs(cls_bases, cls_dict):
 
 def _get_type_info(cls, cls_name, cls_bases, cls_dict, attrs):
     base_type_info = TypeInfo()
-    mixin = {}
+    mixin = TypeInfo()
     extends = cls_dict.get('__extends__', None)
+
+    # user did not specify explicit base class so let's try to derive it from
+    # the actual class hierarchy
     if extends is None:
-        for b in cls_bases:
+        # we don't want origs end up as base classes
+        orig = cls_dict.get("__orig__", None)
+        if orig is None:
+            orig = getattr(cls, '__orig__', None)
+
+        if orig is not None:
+            bases = orig.__bases__
+            logger.debug("Got bases for %s from orig: %r", cls_name, bases)
+        else:
+            bases = cls_bases
+            logger.debug("Got bases for %s from meta: %r", cls_name, bases)
+
+        for b in bases:
             base_types = getattr(b, "_type_info", None)
 
-            if base_types is not None:
-                if getattr(b, '__mixin__', False) == True:
-                    mixin = b.get_flat_type_info(b)
-                else:
-                    if not (extends in (None, b)):
-                        raise Exception("Spyne objects do not support multiple "
-                            "inheritance. Use mixins if you need to reuse "
-                            "fields from multiple classes.")
+            # we don't care about non-ComplexModel bases
+            if base_types is None:
+                continue
 
-                    try:
-                        if len(base_types) > 0 and issubclass(b, ModelBase):
-                            extends = cls_dict["__extends__"] = b
+            # mixins are simple
+            if getattr(b, '__mixin__', False) == True:
+                logger.debug("Adding fields from mixin %r to '%s'", b, cls_name)
+                mixin.update(b.get_flat_type_info(b))
 
-                    except Exception as e:
-                        logger.exception(e)
-                        logger.error(repr(extends))
-                        raise
+                if '__mixin__' not in cls_dict:
+                    cls_dict['__mixin__'] = False
+
+                continue
+
+            if not (extends in (None, b)):
+                raise Exception("Spyne objects do not support multiple "
+                    "inheritance. Use mixins if you need to reuse "
+                    "fields from multiple classes.")
+
+            if len(base_types) > 0 and issubclass(b, ModelBase):
+                extends = cls_dict["__extends__"] = b
+                assert extends.__orig__ is None, "You can't inherit from a " \
+                    "customized class. You should first get your class " \
+                    "hierarchy right, then start customizing classes."
+
+                b.get_subclasses.memo.clear()
+                logger.debug("Registering %r as base of '%s'", b, cls_name)
 
     if not ('_type_info' in cls_dict):
         cls_dict['_type_info'] = _type_info = TypeInfo()
@@ -274,6 +349,7 @@ def _get_type_info(cls, cls_name, cls_bases, cls_dict, attrs):
             _type_info = cls_dict['_type_info'] = TypeInfo(_type_info)
 
     _type_info.update(mixin)
+
     return _type_info
 
 
@@ -312,26 +388,34 @@ class _MethodsDict(dict):
                 d.in_message.insert_field(0, 'self', cls.novalidate_freq())
                 d.body_style = BODY_STYLE_WRAPPED
 
-                for k, v in d.in_message._type_info.items():
-                    # SelfReference is replaced by descriptor.in_message itself.
-                    # However, in the context of mrpc, SelfReference means
-                    # parent class. here, we do that substitution. It's a safe
-                    # hack but a hack nevertheless.
-                    if v is d.in_message:
-                        d.in_message._type_info[k] = cls
+                if issubclass(d.in_message, ComplexModelBase):
+                    for k, v in d.in_message._type_info.items():
+                        # SelfReference is replaced by descriptor.in_message
+                        # itself. However, in the context of mrpc, SelfReference
+                        # means parent class. here, we do that substitution.
+                        # It's a safe hack but a hack nevertheless.
+                        if v is d.in_message:
+                            d.in_message._type_info[k] = cls
 
             # Same as above, for the output type.
-            for k, v in d.out_message._type_info.items():
-                if v is d.out_message:
-                    d.out_message._type_info[k] = cls
+            if issubclass(d.out_message, ComplexModelBase):
+                for k, v in d.out_message._type_info.items():
+                    if v is d.out_message:
+                        d.out_message._type_info[k] = cls
+
+            if d.patterns is not None and not d.no_self:
+                d.name = '.'.join((cls.get_type_name(), d.name))
+                for p in d.patterns:
+                    if p.address is None:
+                        p.address = d.name
 
 
-def _gen_methods(cls_dict):
+def _gen_methods(cls, cls_dict):
     methods = _MethodsDict()
     for k, v in cls_dict.items():
         if not k.startswith('_') and hasattr(v, '_is_rpc'):
-            descriptor = v(_default_function_name=k)
-            cls_dict[k] = descriptor.function
+            descriptor = v(_default_function_name=k, _self_ref_replacement=cls)
+            setattr(cls, k, descriptor.function)
             methods[k] = descriptor
 
     return methods
@@ -425,17 +509,55 @@ def _sanitize_type_info(cls_name, _type_info, _type_info_alt):
             _type_info_alt[key] = v, k
 
 
+def _process_child_attrs(cls, retval, kwargs):
+    child_attrs_all = kwargs.get('child_attrs_all', None)
+    if child_attrs_all is not None:
+        ti = retval._type_info
+        logger.debug("processing child_attrs_all for %r", cls)
+        for k, v in ti.items():
+            logger.debug("  child_attrs_all set %r=%r", k, child_attrs_all)
+            ti[k] = ti[k].customize(**child_attrs_all)
+
+        if retval.__extends__ is not None:
+            retval.__extends__ = retval.__extends__.customize(
+                                            child_attrs_all=child_attrs_all)
+
+        retval.Attributes._delayed_child_attrs_all = child_attrs_all
+
+    child_attrs = copy(kwargs.get('child_attrs', None))
+    if child_attrs is not None:
+        ti = retval._type_info
+        logger.debug("processing child_attrs for %r", cls)
+        for k, v in list(child_attrs.items()):
+            if k in ti:
+                logger.debug("  child_attr set %r=%r", k, v)
+                ti[k] = ti[k].customize(**v)
+                del child_attrs[k]
+
+        base_fti = {}
+        if retval.__extends__ is not None:
+            retval.__extends__ = retval.__extends__.customize(
+                                                    child_attrs=child_attrs)
+            base_fti = retval.__extends__.get_flat_type_info(
+                                                         retval.__extends__)
+        for k, v in child_attrs.items():
+            if k not in base_fti:
+                logger.debug("  child_attr delayed %r=%r", k, v)
+                retval.Attributes._delayed_child_attrs[k] = v
+
+
 class ComplexModelMeta(with_metaclass(Prepareable, type(ModelBase))):
     """This metaclass sets ``_type_info``, ``__type_name__`` and ``__extends__``
     which are going to be used for (de)serialization and schema generation.
     """
 
     def __new__(cls, cls_name, cls_bases, cls_dict):
-        """This function initializes the class and registers attributes for
-        serialization.
-        """
+        """This function initializes the class and registers attributes."""
 
         attrs = _gen_attrs(cls_bases, cls_dict)
+        assert issubclass(attrs, ComplexModelBase.Attributes), \
+                   ("%r must be a ComplexModelBase.Attributes subclass" % attrs)
+
         cls_dict = _get_ordered_attributes(cls_name, cls_dict, attrs)
 
         type_name = cls_dict.get("__type_name__", None)
@@ -444,10 +566,6 @@ class ComplexModelMeta(with_metaclass(Prepareable, type(ModelBase))):
 
         _type_info = _get_type_info(cls, cls_name, cls_bases, cls_dict, attrs)
 
-        methods = _gen_methods(cls_dict)
-        if len(methods) > 0:
-            attrs.methods = methods
-
         # used for sub_name and sub_ns
         _type_info_alt = cls_dict['_type_info_alt'] = TypeInfo()
         for b in cls_bases:
@@ -472,12 +590,16 @@ class ComplexModelMeta(with_metaclass(Prepareable, type(ModelBase))):
             if self.Attributes._subclasses is eattr._subclasses:
                 self.Attributes._subclasses = None
 
-        for k,v in type_info.items():
+        for k, v in type_info.items():
             if issubclass(v, SelfReference):
-                type_info[k] = self
+                self._replace_field(k, self.customize(*v.customize_args,
+                                                          **v.customize_kwargs))
 
             elif issubclass(v, XmlData):
-                self.Attributes._xml_tag_body_as = k, v
+                if self.Attributes._xml_tag_body_as is None:
+                    self.Attributes._xml_tag_body_as = [(k, v)]
+                else:
+                    self.Attributes._xml_tag_body_as.append((k, v))
 
             elif issubclass(v, XmlAttribute):
                 a_of = v.attribute_of
@@ -510,6 +632,10 @@ class ComplexModelMeta(with_metaclass(Prepareable, type(ModelBase))):
         meta = self.Attributes.sqla_metadata
         t = self.Attributes.sqla_table
 
+        methods = _gen_methods(self, cls_dict)
+        if len(methods) > 0:
+            self.Attributes.methods = methods
+
         methods = self.Attributes.methods
         if methods is not None:
             assert isinstance(methods, _MethodsDict)
@@ -519,14 +645,14 @@ class ComplexModelMeta(with_metaclass(Prepareable, type(ModelBase))):
         if tn is None:
             if t is not None:
                 self.Attributes.sqla_metadata = t.metadata
-                from spyne.util.sqlalchemy import gen_spyne_info
+                from spyne.store.relational import gen_spyne_info
 
                 gen_spyne_info(self)
 
         # For spyne objects being converted to a sqlalchemy table
         elif meta is not None and (tn is not None or t is not None) and \
                                                        len(self._type_info) > 0:
-            from spyne.util.sqlalchemy import gen_sqla_info
+            from spyne.store.relational import gen_sqla_info
 
             gen_sqla_info(self, cls_bases)
 
@@ -544,45 +670,22 @@ class ComplexModelMeta(with_metaclass(Prepareable, type(ModelBase))):
     def __prepare__(mcs, name, bases, **kwds):
         return odict()
 
+    #
+    #  pydev debugger seems to secretly manipulate class dictionaries without
+    # doing proper checks -- not always do a class with a metaclass with a
+    # __prepare__ has to have it. this code has a very specific workaround to
+    # make debugging under pydev for python 3.x work.
+    #
+    # see https://github.com/arskom/spyne/issues/432 for details
+    #
+    # this can be removed once pydev fixes secret calls to class dict's
+    # __delitem__
+    if _is_under_pydev_debugger():
+        @classmethod
+        def __prepare__(mcs, name, bases, **kwds):
+            return odict((('__class__', mcs),))
+        print("ComplexModelMeta.__prepare__ substitution for PyDev successful")
 
-# FIXME: what an ugly hack.
-def _fill_empty_type_name(v, parent_ns, parent_tn, k):
-    v.__namespace__ = parent_ns
-    tn = "%s_%s%s" % (parent_tn, k, const.TYPE_SUFFIX)
-
-    if issubclass(v, Array):
-        child_v, = v._type_info.values()
-        child_v.__type_name__ = tn
-
-        v._type_info = TypeInfo({tn: child_v})
-        v.__type_name__ = '%s%s%s'% (const.ARRAY_PREFIX, tn, const.ARRAY_SUFFIX)
-
-        extends = child_v.__extends__
-        while extends is not None and extends.get_type_name() is v.Empty:
-            _fill_empty_type_name(extends, parent_ns, parent_tn,
-                                                        k + const.PARENT_SUFFIX)
-            extends = extends.__extends__
-
-    elif issubclass(v, XmlModifier):
-        child_v = v.type
-        child_v.__type_name__ = tn
-
-        v._type_info = TypeInfo({tn: child_v})
-        v.__type_name__ = '%s%s%s'% (const.ARRAY_PREFIX, tn, const.ARRAY_SUFFIX)
-
-        extends = child_v.__extends__
-        while extends is not None and extends.get_type_name() is v.Empty:
-            _fill_empty_type_name(extends, parent_ns, parent_tn,
-                                                        k + const.PARENT_SUFFIX)
-            extends = extends.__extends__
-
-    else:
-        v.__type_name__ = "%s_%s%s" % (parent_tn, k, const.TYPE_SUFFIX)
-        extends = v.__extends__
-        while extends is not None and extends.__type_name__ is ModelBase.Empty:
-            _fill_empty_type_name(v.__extends__, v.get_namespace(),
-                                     v.get_type_name(), k + const.PARENT_SUFFIX)
-            extends = extends.__extends__
 
 _is_array = lambda v: issubclass(v, Array) or (v.Attributes.min_occurs > 1)
 
@@ -629,6 +732,11 @@ class ComplexModelBase(ModelBase):
         """Customize child attributes in one go. It's a dict of dicts. This is
         ignored unless used via explicit customization."""
 
+        child_attrs_all = None
+        """Customize all child attributes. It's a dict. This is ignored unless
+        used via explicit customization. `child_attrs` always take precedence.
+        """
+
         declare_order = None
         """The order fields of the :class:``ComplexModel`` are to be declared
         in the SOAP WSDL. If this is left as None or explicitly set to
@@ -663,23 +771,31 @@ class ComplexModelBase(ModelBase):
         """FIXME: document me yo."""
 
         _variants = None
-        _xml_tag_body_as = None, None
+        _xml_tag_body_as = None
         _delayed_child_attrs = None
+        _delayed_child_attrs_all = None
         _subclasses = None
 
     def __init__(self, *args, **kwargs):
         cls = self.__class__
         fti = cls.get_flat_type_info(cls)
-        xtba_key, xtba_type = cls.Attributes._xml_tag_body_as
 
-        if xtba_key is not None and len(args) == 1:
-            self._safe_set(xtba_key, args[0], xtba_type)
-        elif len(args) > 0:
-            raise TypeError("Positional argument is only for ComplexModels "
-                            "with XmlData field. You must use keyword "
-                            "arguments in any other case.")
+        if cls.__orig__ is not None:
+            logger.warning("%r seems to be a customized class. It is not "
+                      "supposed to be instantiated. You have been warned.", cls)
 
-        for k,v in fti.items():
+        if cls.Attributes._xml_tag_body_as is not None:
+            for arg, (xtba_key, xtba_type) in \
+                                     zip(args, cls.Attributes._xml_tag_body_as):
+                if xtba_key is not None and len(args) == 1:
+                    self._safe_set(xtba_key, arg, xtba_type)
+                elif len(args) > 0:
+                    raise TypeError(
+                                "Positional argument is only for ComplexModels "
+                                "with XmlData field. You must use keyword "
+                                "arguments in any other case.")
+
+        for k, v in fti.items():
             if k in kwargs:
                 self._safe_set(k, kwargs[k], v)
 
@@ -689,18 +805,27 @@ class ComplexModelBase(ModelBase):
                 def_fac = attr.default_factory
 
                 if def_fac is not None:
+                    if six.PY2 and hasattr(def_fac, 'im_func'):
+                        # unbound-method error workaround. huh.
+                        def_fac = def_fac.im_func
+                    dval = def_fac()
+
                     # should not check for read-only for default values
-                    setattr(self, k, def_fac())
+                    setattr(self, k, dval)
 
                 elif def_val is not None:
                     # should not check for read-only for default values
                     setattr(self, k, def_val)
 
                 # sqlalchemy objects do their own init.
-                elif '_sa_class_manager' in cls.__dict__:
+                elif hasattr(cls, '_sa_class_manager'):
                     # except the attributes that sqlalchemy doesn't know about
                     if v.Attributes.exc_table:
-                        setattr(self, k, None)
+                        try:
+                            setattr(self, k, None)
+                        except AttributeError: # it could be a read-only property
+                            pass
+
                     elif issubclass(v, ComplexModelBase) and \
                                                   v.Attributes.store_as is None:
                         setattr(self, k, None)
@@ -729,13 +854,27 @@ class ComplexModelBase(ModelBase):
 
     def _safe_set(self, key, value, t):
         if t.Attributes.read_only:
-            pass
-        else:
+            return False
+
+        try:
             setattr(self, key, value)
+        except AttributeError as e:
+            logger.exception(e)
+            raise AttributeError("can't set %r attribute %s to %r" %
+                                                   (self.__class__, key, value))
+
+        return True
 
     def as_dict(self):
+        """Represent object as dict.
+
+        Null values are omitted from dict representation to support optional
+        not nullable attributes.
+
+        """
         return dict((
             (k, getattr(self, k)) for k in self.get_flat_type_info(self)
+            if getattr(self, k) is not None
         ))
 
     @classmethod
@@ -770,7 +909,10 @@ class ComplexModelBase(ModelBase):
                 setattr(inst, keys[i], value[i])
 
         elif isinstance(value, dict):
-            inst = cls()
+            cls_orig = cls
+            if cls.__orig__ is not None:
+                cls_orig = cls.__orig__
+            inst = cls_orig()
 
             for k in cls._type_info:
                 setattr(inst, k, value.get(k, None))
@@ -787,8 +929,18 @@ class ComplexModelBase(ModelBase):
         """
         if cls.__orig__ is None:
             return cls()
-        else:
-            return cls.__orig__()
+        return cls.__orig__()
+
+    @classmethod
+    @memoize_id
+    def get_subclasses(cls):
+        retval = []
+        subca = cls.Attributes._subclasses
+        if subca is not None:
+            retval.extend(subca)
+            for subc in subca:
+                retval.extend(subc.get_subclasses())
+        return retval
 
     @staticmethod
     @memoize
@@ -837,7 +989,12 @@ class ComplexModelBase(ModelBase):
             if issubclass(v, Array) and v.Attributes.max_occurs == 1:
                 v, = v._type_info.values()
 
+            key = hier_delim.join(prefix)
             if issubclass(v, ComplexModelBase):
+                retval[key] = _SimpleTypeInfoElement(path=tuple(prefix),
+                               parent=parent, type_=v, is_array=tuple(is_array),
+                                                              can_be_empty=True)
+
                 if not (v in tags):
                     tags.add(v)
                     queue.extend([
@@ -845,7 +1002,6 @@ class ComplexModelBase(ModelBase):
                             is_array + (v.Attributes.max_occurs > 1,), v)
                                    for k2, v2 in v.get_flat_type_info(v).items()])
             else:
-                key = hier_delim.join(prefix)
                 value = retval.get(key, None)
 
                 if value is not None:
@@ -853,7 +1009,8 @@ class ComplexModelBase(ModelBase):
                                                        (cls, k, value.path))
 
                 retval[key] = _SimpleTypeInfoElement(path=tuple(prefix),
-                               parent=parent, type_=v, is_array=tuple(is_array))
+                               parent=parent, type_=v, is_array=tuple(is_array),
+                                                             can_be_empty=False)
 
         return retval
 
@@ -872,7 +1029,7 @@ class ComplexModelBase(ModelBase):
                 continue
 
             if v.__type_name__ is ModelBase.Empty:
-                _fill_empty_type_name(v, cls.get_namespace(),
+                v._fill_empty_type_name(cls.get_namespace(),
                                                          cls.get_type_name(), k)
 
             v.resolve_namespace(v, default_ns, tags)
@@ -900,7 +1057,12 @@ class ComplexModelBase(ModelBase):
     @classmethod
     def customize(cls, **kwargs):
         """Duplicates cls and overwrites the values in ``cls.Attributes`` with
-        ``**kwargs`` and returns the new class."""
+        ``**kwargs`` and returns the new class.
+
+        Because each class is registered as a variant of the original (__orig__)
+        class, using this function to generate classes dynamically on-the-fly
+        could cause memory leaks. You have been warned.
+        """
 
         store_as = apply_pssm(kwargs.get('store_as', None), PSSM_VALUES)
         if store_as is not None:
@@ -908,6 +1070,8 @@ class ComplexModelBase(ModelBase):
 
         cls_name, cls_bases, cls_dict = cls._s_customize(cls, **kwargs)
         cls_dict['__module__'] = cls.__module__
+        if '__extends__' not in cls_dict:
+            cls_dict['__extends__'] = cls.__extends__
 
         retval = type(cls_name, cls_bases, cls_dict)
         retval._type_info = TypeInfo(cls._type_info)
@@ -921,15 +1085,6 @@ class ComplexModelBase(ModelBase):
         else:
             retval.Attributes._delayed_child_attrs = dict(dca.items())
 
-        child_attrs = kwargs.get('child_attrs', None)
-        if child_attrs is not None:
-            ti = retval._type_info
-            for k, v in child_attrs.items():
-                if k in ti:
-                    ti[k] = ti[k].customize(**v)
-                else:
-                    retval.Attributes._delayed_child_attrs[k] = v
-
         tn = kwargs.get("type_name", None)
         if tn is not None:
             retval.__type_name__ = tn
@@ -938,16 +1093,23 @@ class ComplexModelBase(ModelBase):
         if ns is not None:
             retval.__namespace__ = ns
 
-        if not cls is ComplexModel:
+        if cls is not ComplexModel:
             cls._process_variants(retval)
 
+        _process_child_attrs(cls, retval, kwargs)
+
+        # we could be smarter, but customize is supposed to be called only
+        # during daemon initialization, so it's not really necessary.
+        ComplexModelBase.get_subclasses.memo.clear()
+        ComplexModelBase.get_flat_type_info.memo.clear()
+        ComplexModelBase.get_simple_type_info.memo.clear()
+
         return retval
 
     @classmethod
     def _process_variants(cls, retval):
         orig = getattr(retval, '__orig__', None)
         if orig is not None:
-            retval.__extends__ = getattr(orig, '__extends__', None)
             if orig.Attributes._variants is None:
                 orig.Attributes._variants = WeakKeyDictionary()
             orig.Attributes._variants[retval] = True
@@ -955,9 +1117,13 @@ class ComplexModelBase(ModelBase):
             retval.Attributes._variants = None
 
     @classmethod
-    def append_field(cls, field_name, field_type):
+    def _append_field_impl(cls, field_name, field_type):
         assert isinstance(field_name, string_types)
 
+        dcaa = cls.Attributes._delayed_child_attrs_all
+        if dcaa is not None:
+            field_type = field_type.customize(**dcaa)
+
         dca = cls.Attributes._delayed_child_attrs
         if dca is not None:
             d_cust = dca.get(field_name, None)
@@ -965,17 +1131,36 @@ class ComplexModelBase(ModelBase):
                 field_type = field_type.customize(**d_cust)
 
         cls._type_info[field_name] = field_type
-        if cls.Attributes._variants is not None:
-            for c in cls.Attributes._variants:
-                c.append_field(field_name, field_type)
+
         ComplexModelBase.get_flat_type_info.memo.clear()
         ComplexModelBase.get_simple_type_info.memo.clear()
 
     @classmethod
-    def insert_field(cls, index, field_name, field_type):
+    def _append_to_variants(cls, field_name, field_type):
+        if cls.Attributes._variants is not None:
+            for c in cls.Attributes._variants:
+                c.append_field(field_name, field_type)
+
+    @classmethod
+    def append_field(cls, field_name, field_type):
+        cls._append_field_impl(field_name, field_type)
+        cls._append_to_variants(field_name, field_type)
+
+    @classmethod
+    def _insert_to_variants(cls, index, field_name, field_type):
+        if cls.Attributes._variants is not None:
+            for c in cls.Attributes._variants:
+                c.insert_field(index, field_name, field_type)
+
+    @classmethod
+    def _insert_field_impl(cls, index, field_name, field_type):
         assert isinstance(index, int)
         assert isinstance(field_name, string_types)
 
+        dcaa = cls.Attributes._delayed_child_attrs_all
+        if dcaa is not None:
+            field_type = field_type.customize(**dcaa)
+
         dca = cls.Attributes._delayed_child_attrs
         if dca is not None:
             if field_name in dca:
@@ -983,12 +1168,35 @@ class ComplexModelBase(ModelBase):
                 field_type = field_type.customize(**d_cust)
 
         cls._type_info.insert(index, (field_name, field_type))
-        if cls.Attributes._variants is not None:
-            for c in cls.Attributes._variants:
-                c.insert_field(index, field_name, field_type)
+
         ComplexModelBase.get_flat_type_info.memo.clear()
         ComplexModelBase.get_simple_type_info.memo.clear()
 
+    @classmethod
+    def insert_field(cls, index, field_name, field_type):
+        cls._insert_field_impl(index, field_name, field_type)
+        cls._insert_to_variants(index, field_name, field_type)
+
+    @classmethod
+    def _replace_in_variants(cls, field_name, field_type):
+        if cls.Attributes._variants is not None:
+            for c in cls.Attributes._variants:
+                c._replace_field(field_name, field_type)
+
+    @classmethod
+    def _replace_field_impl(cls, field_name, field_type):
+        assert isinstance(field_name, string_types)
+
+        cls._type_info[field_name] = field_type
+
+        ComplexModelBase.get_flat_type_info.memo.clear()
+        ComplexModelBase.get_simple_type_info.memo.clear()
+
+    @classmethod
+    def _replace_field(cls, field_name, field_type):
+        cls._replace_field_impl(field_name, field_type)
+        cls._replace_in_variants(field_name, field_type)
+
     @classmethod
     def store_as(cls, what):
         return cls.customize(store_as=what)
@@ -998,10 +1206,23 @@ class ComplexModelBase(ModelBase):
         return cls.customize(validate_freq=False)
 
     @classmethod
-    def init_from(cls, other):
-        retval = cls()
-        for k in cls._type_info:
-            setattr(retval, k, getattr(other, k, None))
+    def init_from(cls, other, **kwargs):
+        retval = (cls if cls.__orig__ is None else cls.__orig__)()
+
+        for k, v in cls._type_info.items():
+            if v.Attributes.read_only:
+                continue
+
+            try:
+                if k in kwargs:
+                    setattr(retval, k, kwargs[k])
+
+                elif hasattr(other, k):
+                    setattr(retval, k, getattr(other, k))
+
+            except AttributeError as e:
+                logger.warning("Error setting %s: %r", k, e)
+
         return retval
 
     @classmethod
@@ -1030,7 +1251,13 @@ class Array(ComplexModelBase):
     class Attributes(ComplexModelBase.Attributes):
         _wrapper = True
 
-    def __new__(cls, serializer, member_name=None, **kwargs):
+    def __new__(cls, serializer, member_name=None, wrapped=True, **kwargs):
+        if not wrapped:
+            if serializer.Attributes.max_occurs == 1:
+                kwargs['max_occurs'] = 'unbounded'
+
+            return serializer.customize(**kwargs)
+
         retval = cls.customize(**kwargs)
 
         _serializer = _get_spyne_type(cls.__name__, '__serializer__', serializer)
@@ -1050,6 +1277,24 @@ class Array(ComplexModelBase):
 
         return retval
 
+    @classmethod
+    def _fill_empty_type_name(cls, parent_ns, parent_tn, k):
+        cls.__namespace__ = parent_ns
+        tn = "%s_%s%s" % (parent_tn, k, const.TYPE_SUFFIX)
+
+        child_v, = cls._type_info.values()
+        child_v.__type_name__ = tn
+
+        cls._type_info = TypeInfo({tn: child_v})
+        cls.__type_name__ = '%s%s%s' % (const.ARRAY_PREFIX, tn,
+                                                             const.ARRAY_SUFFIX)
+
+        extends = child_v.__extends__
+        while extends is not None and extends.get_type_name() is cls.Empty:
+            extends._fill_empty_type_name(parent_ns, parent_tn,
+                                                    k + const.PARENT_SUFFIX)
+            extends = extends.__extends__
+
     @classmethod
     def customize(cls, **kwargs):
         serializer_attrs = kwargs.get('serializer_attrs', None)
@@ -1063,7 +1308,7 @@ class Array(ComplexModelBase):
 
     @classmethod
     def _set_serializer(cls, serializer, member_name=None):
-        if serializer.get_type_name() is ModelBase.Empty: # A customized class
+        if serializer.get_type_name() is ModelBase.Empty:  # A customized class
             member_name = "OhNoes"
             # mark array type name as "to be resolved later".
             cls.__type_name__ = ModelBase.Empty
@@ -1082,7 +1327,7 @@ class Array(ComplexModelBase):
             serializer = serializer.customize(max_occurs=decimal.Decimal('inf'))
 
         assert isinstance(member_name, string_types), member_name
-        cls._type_info = {member_name: serializer}
+        cls._type_info = TypeInfo({member_name: serializer})
 
     # the array belongs to its child's namespace, it doesn't have its own
     # namespace.
@@ -1135,22 +1380,30 @@ class Iterable(Array):
 
 @memoize
 def TTableModelBase():
-    from spyne.util.sqlalchemy import add_column
+    from spyne.store.relational import add_column
 
     class TableModelBase(ComplexModelBase):
         @classmethod
         def append_field(cls, field_name, field_type):
-            super(TableModelBase, cls).append_field(field_name, field_type)
+            cls._append_field_impl(field_name, field_type)
             # There could have been changes to field_type in ComplexModel so we
             # should not use field_type directly from above
-            add_column(cls, field_name, cls._type_info[field_name])
+            if cls.__table__ is not None:
+                add_column(cls, field_name, cls._type_info[field_name])
+            cls._append_to_variants(field_name, field_type)
+
+        @classmethod
+        def replace_field(cls, field_name, field_type):
+            raise NotImplementedError()
 
         @classmethod
         def insert_field(cls, index, field_name, field_type):
-            super(TableModelBase, cls).insert_field(index, field_name, field_type)
+            cls._insert_field_impl(index, field_name, field_type)
             # There could have been changes to field_type in ComplexModel so we
             # should not use field_type directly from above
-            add_column(cls, field_name, cls._type_info[field_name])
+            if cls.__table__ is not None:
+                add_column(cls, field_name, cls._type_info[field_name])
+            cls._insert_to_variants(index, field_name, field_type)
 
     return TableModelBase
 
