@@ -23,8 +23,6 @@ constant change. You have been warned.
 """
 
 from spyne.protocol.html._base import HtmlBase
-from spyne.protocol.html._base import E
-from spyne.protocol.html._base import NSMAP
 from spyne.protocol.html.table import HtmlColumnTable
 from spyne.protocol.html.table import HtmlRowTable
 from spyne.protocol.html.microformat import HtmlMicroFormat
