@@ -17,85 +17,50 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 #
 
+from __future__ import print_function
+
+import logging
+logger = logging.getLogger(__name__)
+
 from inspect import isgenerator
 
-from spyne.model import ModelBase, AnyHtml
-from spyne.model import ByteArray
-from spyne.model import ComplexModelBase
-from spyne.model import Array
-from spyne.model import AnyUri
-from spyne.model import ImageUri
-from spyne.model.binary import Attachment
-from spyne.protocol.html import HtmlBase
-from spyne.protocol.html import NSMAP
+from lxml import html
 from lxml.html.builder import E
+
+from spyne import ModelBase, ByteArray, ComplexModelBase, Array, AnyUri, \
+    ImageUri
+from spyne.protocol.html import HtmlBase
 from spyne.util import coroutine, Break
+from spyne.util.oset import oset
 from spyne.util.cdict import cdict
+from spyne.util.six.moves.urllib.parse import urlencode, quote
 
 
-def HtmlTable(app=None, ignore_uncap=False, ignore_wrappers=True,
-                     produce_header=True, table_name_attr='class',
-                     field_name_attr='class', border=0, fields_as='columns',
-                     row_class=None, cell_class=None, header_cell_class=None):
-    """Protocol that returns the response object as a html table.
-
-    The simple flavour is like the HtmlMicroFormatprotocol, but returns data
-    as a html table using the <table> tag.
-
-    :param app: A spyne.application.Application instance.
-    :param produce_header: Boolean value to determine whether to show field
-        names in the beginning of the table or not. Defaults to True. Set to
-        False to skip headers.
-    :param table_name_attr: The name of the attribute that will contain the
-        response name of the complex object in the table tag. Set to None to
-        disable.
-    :param field_name_attr: The name of the attribute that will contain the
-        field names of the complex object children for every table cell. Set
-        to None to disable.
-    :param fields_as: One of 'columns', 'rows'.
-    :param row_class: value that goes inside the <tr class="">
-    :param cell_class: value that goes inside the <td class="">
-    :param header_cell_class: value that goes inside the <th class="">
-
-    "Fields as rows" returns one record per table in a table with two
-    columns.
-
-    "Fields as columns" returns one record per table row in a table that
-    has as many columns as field names, just like a regular spreadsheet.
-    """
-
-    if fields_as == 'columns':
-        return HtmlColumnTable(app, ignore_uncap, ignore_wrappers,
-                    produce_header, table_name_attr, field_name_attr, border,
-                                    row_class, cell_class, header_cell_class)
-    elif fields_as == 'rows':
-        return HtmlRowTable(app, ignore_uncap, ignore_wrappers,
-                    produce_header, table_name_attr, field_name_attr, border,
-                                    row_class, cell_class, header_cell_class)
-
-    else:
-        raise ValueError(fields_as)
-
 class HtmlTableBase(HtmlBase):
     def __init__(self, app=None, ignore_uncap=False, ignore_wrappers=True,
-                       cloth=None, attr_name='spyne_id', root_attr_name='spyne',
-                                                              cloth_parser=None,
-                             produce_header=True, table_name_attr='class',
-                            field_name_attr='class', border=0, row_class=None,
-                                cell_class=None, header_cell_class=None):
+            cloth=None, cloth_parser=None, header=True, table_name_attr='class',
+                     table_name=None, table_class=None, field_name_attr='class',
+              border=0, row_class=None, cell_class=None, header_cell_class=None,
+                 polymorphic=True, hier_delim='.', doctype=None, link_gen=None,
+                 mrpc_delim_text='|', table_width=None):
 
         super(HtmlTableBase, self).__init__(app=app,
                      ignore_uncap=ignore_uncap, ignore_wrappers=ignore_wrappers,
-                cloth=cloth, attr_name=attr_name, root_attr_name=root_attr_name,
-                                                      cloth_parser=cloth_parser)
+                cloth=cloth, cloth_parser=cloth_parser, polymorphic=polymorphic,
+                                         hier_delim=hier_delim, doctype=doctype)
 
-        self.produce_header = produce_header
+        self.header = header
         self.table_name_attr = table_name_attr
+        self.table_name = table_name
         self.field_name_attr = field_name_attr
         self.border = border
         self.row_class = row_class
         self.cell_class = cell_class
         self.header_cell_class = header_cell_class
+        self.link_gen = link_gen
+        self.table_class = table_class
+        self.table_width = table_width
+        self.mrpc_delim_text = mrpc_delim_text
 
         if self.cell_class is not None and field_name_attr == 'class':
             raise Exception("Either 'cell_class' should be None or "
@@ -106,70 +71,124 @@ class HtmlTableBase(HtmlBase):
                             "field_name_attr should be != 'class'")
 
     def model_base_to_parent(self, ctx, cls, inst, parent, name,  **kwargs):
-        parent.write(self.to_string(cls, inst))
+        parent.write(self.to_unicode(cls, inst))
 
     def null_to_parent(self, ctx, cls, inst, parent, name, **kwargs):
         pass
 
 
 class HtmlColumnTable(HtmlTableBase):
+    """Protocol that returns the response object as a html table.
+
+    Returns one record per table row in a table that has as many columns as
+    field names, just like a regular spreadsheet.
+
+    The simple flavour is like the HtmlMicroFormatprotocol, but returns data
+    as a html table using the <table> tag.
+
+    :param app: A spyne.application.Application instance.
+    :param header: Boolean value to determine whether to show field
+        names in the beginning of the table or not. Defaults to True. Set to
+        False to skip headers.
+    :param table_name_attr: The name of the attribute that will contain the
+        response name of the complex object in the table tag. Set to None to
+        disable.
+    :param table_name: When not none, overrides what goes in `table_name_attr`.
+    :param table_class: When not none, specifies what goes in `class` attribute
+        in the `<table>` tag. Table name gets appended when
+        `table_name_attr == 'class'`
+    :param field_name_attr: The name of the attribute that will contain the
+        field names of the complex object children for every table cell. Set
+        to None to disable.
+    :param row_class: value that goes inside the <tr class="">
+    :param cell_class: value that goes inside the <td class="">
+    :param header_cell_class: value that goes inside the <th class="">
+    :param mrpc_delim_text: The text that goes between mrpc calls.
+    """
+
     def __init__(self, *args, **kwargs):
         super(HtmlColumnTable, self).__init__(*args, **kwargs)
 
-        self.serialization_handlers = cdict({
+        self.serialization_handlers.update({
             ModelBase: self.model_base_to_parent,
-            AnyUri: self.anyuri_to_parent,
-            AnyHtml: self.anyhtml_to_parent,
-            ImageUri: self.imageuri_to_parent,
-            ByteArray: self.not_supported,
-            Attachment: self.not_supported,
             ComplexModelBase: self.complex_model_to_parent,
             Array: self.array_to_parent,
         })
 
     def model_base_to_parent(self, ctx, cls, inst, parent, name,
                                                       from_arr=False, **kwargs):
+        inst_str = ''
+        if inst is not None:
+            inst_str = self.to_unicode(cls, inst)
+
         if from_arr:
             td_attrs = {}
             #if self.field_name_attr:
             #    td_attrs[self.field_name_attr] = name
             parent.write(E.tr(
                 E.td(
-                    self.to_string(cls, inst),
+                    inst_str,
                     **td_attrs
                 )
             ))
 
         else:
-            parent.write(self.to_string(cls, inst))
+            parent.write(inst_str)
 
     @coroutine
-    def _gen_row(self, ctx, cls, inst, parent, name, **kwargs):
+    def _gen_row(self, ctx, cls, inst, parent, name, from_arr=False,
+                                                    array_index=None, **kwargs):
+
+        # because HtmlForm* protocols don't use the global null handler, it's
+        # possible for null values to reach here.
+        if inst is None:
+            return
+
+        logger.debug("Generate row for %r", cls)
+
+        mrpc_delim_elt = ''
+        if self.mrpc_delim_text is not None:
+            mrpc_delim_elt = E.span(self.mrpc_delim_text,
+                                      **{'class': 'mrpc-delimiter'})
+            mrpc_delim_elt.tail = ' '
+
         with parent.element('tr'):
-            for k, v in cls.get_flat_type_info(cls).items():
-                # FIXME: To be fixed to work with prot_attrs and renamed to exc
-                if getattr(v.Attributes, 'exc_html', False) == True:
-                    continue
-                if getattr(v.Attributes, 'read', True) == False:
+            for k, v in self.sort_fields(cls):
+                attr = self.get_cls_attrs(v)
+                if attr.exc:
+                    logger.debug("\tExclude table cell %r type %r for %r",
+                                                                      k, v, cls)
                     continue
 
                 try:
                     sub_value = getattr(inst, k, None)
-                except: # to guard against e.g. SQLAlchemy throwing NoSuchColumnError
+                except:  # e.g. SQLAlchemy could throw NoSuchColumnError
                     sub_value = None
 
-                sub_name = v.Attributes.sub_name
+                sub_name = attr.sub_name
                 if sub_name is None:
                     sub_name = k
 
+                if self.hier_delim is not None:
+                    if array_index is None:
+                        sub_name = "%s%s%s" % (name, self.hier_delim, sub_name)
+                    else:
+                        sub_name = "%s[%d]%s%s" % (name, array_index,
+                                                     self.hier_delim, sub_name)
+
+                logger.debug("\tGenerate table cell %r type %r for %r",
+                                                               sub_name, v, cls)
+
                 td_attrs = {}
                 if self.field_name_attr is not None:
-                    td_attrs[self.field_name_attr] = sub_name
+                    td_attrs[self.field_name_attr] = attr.sub_name or k
+                if attr.hidden:
+                    td_attrs['style'] = 'display:None'
 
                 with parent.element('td', td_attrs):
-                    ret = self.to_parent(ctx, v, sub_value, parent, sub_name,
-                                                                       **kwargs)
-
+                    ret = self.to_parent(ctx, v, sub_value, parent,
+                                              sub_name, from_arr=from_arr,
+                                              array_index=array_index, **kwargs)
                     if isgenerator(ret):
                         try:
                             while True:
@@ -181,49 +200,105 @@ class HtmlColumnTable(HtmlTableBase):
                             except StopIteration:
                                 pass
 
-            self.extend_data_row(ctx, cls, inst, parent, name, **kwargs)
+            m = cls.Attributes.methods
+            if m is not None and len(m) > 0:
+                td_attrs = {'class': 'mrpc-cell'}
+
+                with parent.element('td', td_attrs):
+                    first = True
+
+                    for mn, md in self._methods(cls, inst):
+                        if first:
+                            first = False
+                        elif mrpc_delim_elt is not None:
+                            parent.write(" ")
+                            parent.write(mrpc_delim_elt)
+
+                        pd = { }
+                        for k, v in self.sort_fields(cls):
+                            if getattr(v.Attributes, 'primary_key', None):
+                                r = self.to_unicode(v, getattr(inst, k, None))
+                                if r is not None:
+                                    pd[k] = r
+
+                        params = urlencode(pd)
+
+                        mdid2key = ctx.app.interface.method_descriptor_id_to_key
+                        href = mdid2key[id(md)].rsplit("}", 1)[-1]
+                        text = md.translate(ctx.locale,
+                                                  md.in_message.get_type_name())
+                        parent.write(E.a(
+                            text,
+                            href="%s?%s" % (href, params),
+                            **{'class': 'mrpc-operation'}
+                        ))
+
+            logger.debug("Generate row for %r done.", cls)
+            self.extend_data_row(ctx, cls, inst, parent, name,
+                                              array_index=array_index, **kwargs)
+
+    def _gen_thead(self, ctx, cls, name, parent):
+        logger.debug("Generate header for %r", cls)
 
-    def _gen_header(self, ctx, cls, name, parent):
         with parent.element('thead'):
             with parent.element('tr'):
-                th_attrs = {}
-                if self.field_name_attr is not None:
-                    th_attrs[self.field_name_attr] = name
-
-                # fti is none when the type inside Array is not a ComplexModel.
                 if issubclass(cls, ComplexModelBase):
-                    fti = cls.get_flat_type_info(cls)
-                    if self.field_name_attr is None:
-                        for k, v in fti.items():
-                            if getattr(v.Attributes, 'exc_html', None):
-                                continue
-                            header_name = self.translate(v, ctx.locale, k)
-                            parent.write(E.th(header_name, **th_attrs))
-                    else:
-                        for k, v in fti.items():
-                            if getattr(v.Attributes, 'exc_html', None):
-                                continue
+                    fti = self.sort_fields(cls)
+                    for k, v in fti:
+                        attr = self.get_cls_attrs(v)
+                        if attr.exc:
+                            continue
+
+                        th_attrs = {}
+                        if self.field_name_attr is not None:
                             th_attrs[self.field_name_attr] = k
-                            header_name = self.translate(v, ctx.locale, k)
-                            parent.write(E.th(header_name, **th_attrs))
+                        if attr.hidden:
+                            th_attrs['style'] = 'display:None'
+
+                        header_name = self.trc(v, ctx.locale, k)
+                        parent.write(E.th(header_name, **th_attrs))
+
+                    m = cls.Attributes.methods
+                    if m is not None and len(m) > 0:
+                        parent.write(E.th())
 
                 else:
+                    th_attrs = {}
                     if self.field_name_attr is not None:
                         th_attrs[self.field_name_attr] = name
-                    header_name = self.translate(cls, ctx.locale, name)
+                    header_name = self.trc(cls, ctx.locale, name)
                     parent.write(E.th(header_name, **th_attrs))
 
-                self.extend_header_row(ctx, cls, name, parent)
+                self.extend_header_row(ctx, cls, parent, name)
 
     @coroutine
     def _gen_table(self, ctx, cls, inst, parent, name, gen_rows, **kwargs):
-        attrs = {}
-        if self.table_name_attr is not None:
-            attrs[self.table_name_attr] = cls.get_type_name()
+        logger.debug("Generate table for %r", cls)
 
-        with parent.element('table', attrs, nsmap=NSMAP):
-            if self.produce_header:
-                self._gen_header(ctx, cls, name, parent)
+        attrib = {}
+        table_class = oset()
+        if self.table_class is not None:
+            table_class.add(self.table_class)
+
+        if self.table_name_attr is not None:
+            tn = (self.table_name
+                        if self.table_name is not None else cls.get_type_name())
+
+            if self.table_name_attr == 'class':
+                table_class.add(tn)
+            else:
+                attrib[self.table_name_attr] = tn
+
+        attrib['class'] = ' '.join(table_class)
+        if self.table_width is not None:
+            attrib['width'] = self.table_width
+
+        self.event_manager.fire_event('before_table', ctx, cls, inst, parent,
+                                                                 name, **kwargs)
+
+        with parent.element('table', attrib):
+            if self.header:
+                self._gen_thead(ctx, cls, name, parent)
 
             with parent.element('tbody'):
                 ret = gen_rows(ctx, cls, inst, parent, name, **kwargs)
@@ -238,32 +313,26 @@ class HtmlColumnTable(HtmlTableBase):
                         except StopIteration:
                             pass
 
-                ret = self.extend_table(ctx, cls, parent, name, **kwargs)
-                if isgenerator(ret):
-                    try:
-                        while True:
-                            sv2 = (yield)
-                            ret.send(sv2)
-                    except Break as b:
-                        try:
-                            ret.throw(b)
-                        except StopIteration:
-                            pass
+                self.extend_table(ctx, cls, parent, name, **kwargs)
 
     def complex_model_to_parent(self, ctx, cls, inst, parent, name,
                                                       from_arr=False, **kwargs):
-        # If this is direct child of an array, table is already set up in the
+        # If this is direct child of an array, table is already set up in
         # array_to_parent.
         if from_arr:
             return self._gen_row(ctx, cls, inst, parent, name, **kwargs)
         else:
-            return self._gen_table(ctx, cls, inst, parent, name, self._gen_row,
+            return self.wrap_table(ctx, cls, inst, parent, name, self._gen_row,
                                                                        **kwargs)
 
     def array_to_parent(self, ctx, cls, inst, parent, name, **kwargs):
-        return self._gen_table(ctx, cls, inst, parent, name,
+        return self.wrap_table(ctx, cls, inst, parent, name,
                          super(HtmlColumnTable, self).array_to_parent, **kwargs)
 
+    def wrap_table(self, ctx, cls, inst, parent, name, gen_rows, **kwargs):
+        return self._gen_table(ctx, cls, inst, parent, name, gen_rows, **kwargs)
+
+    # FIXME: These three should be events as well.
     def extend_table(self, ctx, cls, parent, name, **kwargs):
         pass
 
@@ -275,6 +344,32 @@ class HtmlColumnTable(HtmlTableBase):
 
 
 class HtmlRowTable(HtmlTableBase):
+    """Protocol that returns the response object as a html table.
+
+    The simple flavour is like the HtmlMicroFormatprotocol, but returns data
+    as a html table using the <table> tag.
+
+    Returns one record per table in a table with two columns.
+
+    :param app: A spyne.application.Application instance.
+    :param header: Boolean value to determine whether to show field
+        names in the beginning of the table or not. Defaults to True. Set to
+        False to skip headers.
+    :param table_name_attr: The name of the attribute that will contain the
+        response name of the complex object in the table tag. Set to None to
+        disable.
+    :param table_name: When not none, overrides what goes in `table_name_attr`.
+    :param table_class: When not none, specifies what goes in `class` attribute
+        in the `<table>` tag. Table name gets appended when
+        `table_name_attr == 'class'`
+    :param field_name_attr: The name of the attribute that will contain the
+        field names of the complex object children for every table cell. Set
+        to None to disable.
+    :param row_class: value that goes inside the <tr class="">
+    :param cell_class: value that goes inside the <td class="">
+    :param header_cell_class: value that goes inside the <th class="">
+    """
+
     def __init__(self, *args, **kwargs):
         super(HtmlRowTable, self).__init__(*args, **kwargs)
 
@@ -283,34 +378,37 @@ class HtmlRowTable(HtmlTableBase):
             AnyUri: self.anyuri_to_parent,
             ImageUri: self.imageuri_to_parent,
             ByteArray: self.not_supported,
-            Attachment: self.not_supported,
             ComplexModelBase: self.complex_model_to_parent,
             Array: self.array_to_parent,
         })
 
-    def model_base_to_parent(self, ctx, cls, inst, parent, name, from_arr=False, **kwargs):
+    def model_base_to_parent(self, ctx, cls, inst, parent, name, from_arr=False,
+                                                                      **kwargs):
         if from_arr:
-            td_attrs = {}
+            td_attrib = {}
             if False and self.field_name_attr:
-                td_attrs[self.field_name_attr] = name
+                td_attrib[self.field_name_attr] = name
 
-            parent.write(E.tr(E.td(self.to_string(cls, inst), **td_attrs)))
+            parent.write(E.tr(E.td(self.to_unicode(cls, inst), **td_attrib)))
         else:
-            parent.write(self.to_string(cls, inst))
+            parent.write(self.to_unicode(cls, inst))
 
     @coroutine
     def complex_model_to_parent(self, ctx, cls, inst, parent, name,
                                                       from_arr=False, **kwargs):
-        attrs = {}
+        attrib = {}
         if self.table_name_attr is not None:
-            attrs[self.table_name_attr] = cls.get_type_name()
+            attrib[self.table_name_attr] = cls.get_type_name()
+        if self.table_width is not None:
+            attrib['width'] = self.table_width
 
-        with parent.element('table', attrs, nsmap=NSMAP):
+        with parent.element('table', attrib):
             with parent.element('tbody'):
-                for k, v in cls.get_flat_type_info(cls).items():
+                for k, v in self.sort_fields(cls):
+                    sub_attrs = self.get_cls_attrs(v)
                     try:
                         sub_value = getattr(inst, k, None)
-                    except: # to guard against e.g. SQLAlchemy throwing NoSuchColumnError
+                    except:  # e.g. SQLAlchemy could throw NoSuchColumnError
                         sub_value = None
 
                     sub_name = v.Attributes.sub_name
@@ -322,28 +420,32 @@ class HtmlRowTable(HtmlTableBase):
                                                              sub_name, **kwargs)
                         continue
 
-                    tr_attrs = {}
+                    tr_attrib = {}
                     if self.row_class is not None:
-                        tr_attrs['class'] = self.row_class
-                    with parent.element('tr', tr_attrs):
-                        th_attrs = {}
+                        tr_attrib['class'] = self.row_class
+                    with parent.element('tr', tr_attrib):
+                        th_attrib = {}
                         if self.header_cell_class is not None:
-                            th_attrs['class'] = self.header_cell_class
+                            th_attrib['class'] = self.header_cell_class
                         if self.field_name_attr is not None:
-                            th_attrs[self.field_name_attr] = sub_name
-                        if self.produce_header:
+                            th_attrib[self.field_name_attr] = sub_name
+                        if sub_attrs.hidden:
+                            th_attrib['style'] = 'display:None'
+                        if self.header:
                             parent.write(E.th(
-                                self.translate(v, ctx.locale, sub_name),
-                                **th_attrs
+                                self.trc(v, ctx.locale, sub_name),
+                                **th_attrib
                             ))
 
-                        td_attrs = {}
+                        td_attrib = {}
                         if self.cell_class is not None:
-                            td_attrs['class'] = self.cell_class
+                            td_attrib['class'] = self.cell_class
                         if self.field_name_attr is not None:
-                            td_attrs[self.field_name_attr] = sub_name
+                            td_attrib[self.field_name_attr] = sub_name
+                        if sub_attrs.hidden:
+                            td_attrib['style'] = 'display:None'
 
-                        with parent.element('td', td_attrs):
+                        with parent.element('td', td_attrib):
                             ret = self.to_parent(ctx, v, sub_value, parent,
                                                             sub_name, **kwargs)
                             if isgenerator(ret):
@@ -359,7 +461,7 @@ class HtmlRowTable(HtmlTableBase):
 
     @coroutine
     def array_to_parent(self, ctx, cls, inst, parent, name, **kwargs):
-        with parent.element('div', nsmap=NSMAP):
+        with parent.element('div'):
             if issubclass(cls, ComplexModelBase):
                 ret = super(HtmlRowTable, self).array_to_parent(
                                          ctx, cls, inst, parent, name, **kwargs)
@@ -374,22 +476,28 @@ class HtmlRowTable(HtmlTableBase):
                         except StopIteration:
                             pass
             else:
-                table_attrs = {}
+                table_attrib = {}
                 if self.table_name_attr:
-                    table_attrs = {self.table_name_attr: name}
+                    table_attrib = {self.table_name_attr: name}
+                if self.table_width is not None:
+                    table_attrib['width'] = self.table_width
 
-                with parent.element('table', table_attrs, nsmap=NSMAP):
-                    tr_attrs = {}
+                with parent.element('table', table_attrib):
+                    tr_attrib = {}
                     if self.row_class is not None:
-                        tr_attrs['class'] = self.row_class
-                    with parent.element('tr', tr_attrs):
-                        if self.produce_header:
-                            parent.write(E.th(self.translate(cls, ctx.locale,
+                        tr_attrib['class'] = self.row_class
+                    with parent.element('tr', tr_attrib):
+                        if self.header:
+                            parent.write(E.th(self.trc(cls, ctx.locale,
                                                           cls.get_type_name())))
-                        td_attrs = {}
+                        td_attrib = {}
+                        cls_attrs = self.get_cls_attrs(cls)
                         if self.cell_class is not None:
-                            td_attrs['class'] = self.cell_class
-                        with parent.element('td', td_attrs):
+                            td_attrib['class'] = self.cell_class
+                        if cls_attrs.hidden:
+                            td_attrib['style'] = 'display:None'
+
+                        with parent.element('td', td_attrib):
                             with parent.element('table'):
                                 ret = super(HtmlRowTable, self) \
                                     .array_to_parent(ctx, cls, inst, parent,
