@@ -19,6 +19,9 @@
 
 """The ``spyne.protocol.http`` module contains the HttpRpc protocol
 implementation.
+
+This module is EXPERIMENTAL. You may not recognize the code here next time you
+look at it.
 """
 
 import logging
@@ -28,11 +31,8 @@ import re
 import pytz
 import tempfile
 
-from spyne.util.six import string_types, BytesIO, PY3
-if PY3:
-    from http.cookies import SimpleCookie
-else:
-    from Cookie import SimpleCookie
+from spyne.util.six import string_types, BytesIO
+from spyne.util.six.moves.http_cookies import SimpleCookie
 
 from spyne import BODY_STYLE_WRAPPED, MethodDescriptor
 from spyne.error import ResourceNotFoundError
@@ -97,7 +97,8 @@ class HttpRpc(SimpleDictDocument):
 
     :param app: An :class:'spyne.application.Application` instance.
     :param validator: Validation method to use. One of (None, 'soft')
-    :param mime_type: Default mime type to set. Default is 'application/octet-stream'
+    :param mime_type: Default mime type to set. Default is
+        'application/octet-stream'
     :param tmp_dir: Temporary directory to store partial file uploads. Default
         is to use the OS default.
     :param tmp_delete_on_close: The ``delete`` argument to the
@@ -111,6 +112,7 @@ class HttpRpc(SimpleDictDocument):
 
     mime_type = 'text/plain'
     default_binary_encoding = BINARY_ENCODING_URLSAFE_BASE64
+    default_string_encoding = 'UTF-8'
 
     type = set(SimpleDictDocument.type)
     type.add('http')
@@ -183,6 +185,9 @@ class HttpRpc(SimpleDictDocument):
             raise ResourceNotFoundError(ctx.method_request_string)
 
         req_enc = getattr(ctx.transport, 'request_encoding', None)
+        if req_enc is None:
+            req_enc = ctx.in_protocol.default_string_encoding
+
         if ctx.descriptor.in_header is not None:
             # HttpRpc supports only one header class
             in_header_class = ctx.descriptor.in_header[0]
@@ -259,10 +264,14 @@ class HttpRpc(SimpleDictDocument):
 
         ctx.out_string = ctx.out_document
 
+    def boolean_from_string(self, cls, string):
+        return string.lower() in ('true', '1', 'checked', 'on')
+
 
 _fragment_pattern_re = re.compile('<([A-Za-z0-9_]+)>')
 _full_pattern_re = re.compile('{([A-Za-z0-9_]+)}')
 
+
 class HttpPattern(object):
     """Experimental. Stay away.
 
@@ -320,6 +329,9 @@ class HttpPattern(object):
 
     @address.setter
     def address(self, what):
+        if what is not None and not what.startswith('/'):
+            what = '/' + what
+
         self.__address = what
         self.address_re = self._compile_url_pattern(what)
 
@@ -358,5 +370,6 @@ class HttpPattern(object):
                                                                 methods=methods)
 
     def __repr__(self):
-        return "HttpPattern(address=%r, host=%r, verb=%r, endpoint=%r" % (
-                    self.address, self.host, self.verb, self.endpoint.name)
+        return "HttpPattern(address=%r, host=%r, verb=%r, endpoint=%r)" % (
+                    self.address, self.host, self.verb,
+                    None if self.endpoint is None else self.endpoint.name)
