@@ -22,9 +22,6 @@ Currently, only :class:`spyne.protocol.json.JsonDocument` is supported.
 
 Initially released in 2.8.0-rc.
 
-This module is EXPERIMENTAL. You may not recognize the code here next time you
-look at it.
-
 Missing Types
 =============
 
@@ -111,23 +108,23 @@ class JsonDocument(HierDictDocument):
                         ignore_uncap=False,
                         # DictDocument specific
                         ignore_wrappers=True, complex_as=dict, ordered=False,
-                        default_string_encoding=None,
+                        default_string_encoding=None, polymorphic=False,
                         **kwargs):
 
         super(JsonDocument, self).__init__(app, validator, mime_type, ignore_uncap,
-                                           ignore_wrappers, complex_as, ordered)
+                               ignore_wrappers, complex_as, ordered, polymorphic)
 
         # this is needed when we're overriding a regular instance attribute
         # with a property.
         self.__message = HierDictDocument.__getattribute__(self, 'message')
 
-        self._from_string_handlers[Double] = self._ret
-        self._from_string_handlers[Boolean] = self._ret
-        self._from_string_handlers[Integer] = self._ret
+        self._from_unicode_handlers[Double] = self._ret
+        self._from_unicode_handlers[Boolean] = self._ret
+        self._from_unicode_handlers[Integer] = self._ret
 
-        self._to_string_handlers[Double] = self._ret
-        self._to_string_handlers[Boolean] = self._ret
-        self._to_string_handlers[Integer] = self._ret
+        self._to_unicode_handlers[Double] = self._ret
+        self._to_unicode_handlers[Boolean] = self._ret
+        self._to_unicode_handlers[Integer] = self._ret
 
         self.default_string_encoding = default_string_encoding
         self.kwargs = kwargs
@@ -157,7 +154,7 @@ class JsonDocument(HierDictDocument):
         """Sets ``ctx.in_document``  using ``ctx.in_string``."""
 
         try:
-            in_string = ''.join(ctx.in_string)
+            in_string = b''.join(ctx.in_string)
             if not isinstance(in_string, six.text_type):
                 if in_string_encoding is None:
                     in_string_encoding = self.default_string_encoding
@@ -170,7 +167,7 @@ class JsonDocument(HierDictDocument):
 
     def create_out_string(self, ctx, out_string_encoding='utf8'):
         """Sets ``ctx.out_string`` using ``ctx.out_document``."""
-        ctx.out_string = (json.dumps(o, **self.kwargs) for o in ctx.out_document)
+        ctx.out_string = (json.dumps(o, **self.kwargs).encode(out_string_encoding) for o in ctx.out_document)
 
 
 class JsonP(JsonDocument):
