@@ -17,15 +17,17 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 #
 
+
 """The ``spyne.protocol.xml`` module contains an xml-based protocol that
 serializes python objects to xml using Xml Schema conventions.
 
-Logs valid documents to ``'%r'`` and invalid documents to ``'%r'``. Use the
-usual ``logging.getLogger()`` and friends to configure how these get logged.
+Logs valid documents to ``'spyne.protocol.xml'`` and invalid documents to
+``spyne.protocol.xml.invalid``. Use the usual ``logging.getLogger()`` and
+friends to configure how these get logged.
 
 Warning! You can get a lot of crap in the 'invalid' logger. You're not advised
 to turn it on for a production system.
-""" % ('spyne.protocol.xml', 'spyne.protocol.xml.invalid')
+"""
 
 
 import logging
@@ -43,11 +45,12 @@ from lxml.etree import XMLParser
 
 from spyne import BODY_STYLE_WRAPPED
 
-from spyne.util import _bytes_join, Break, coroutine
+from spyne.util import Break, coroutine
 from spyne.util.six import text_type, string_types
 from spyne.util.cdict import cdict
 from spyne.util.etreeconv import etree_to_dict, dict_to_etree,\
     root_dict_to_etree
+from spyne.const.xml import XSI
 
 from spyne.error import Fault
 from spyne.error import ValidationError
@@ -55,10 +58,10 @@ from spyne.const.ansi_color import LIGHT_GREEN
 from spyne.const.ansi_color import LIGHT_RED
 from spyne.const.ansi_color import END_COLOR
 from spyne.const.xml_ns import xsi as _ns_xsi
-from spyne.const.xml_ns import soap_env as _ns_soap_env
+from spyne.const.xml_ns import soap11_env
 from spyne.const.xml_ns import const_prefmap, DEFAULT_NS
-_pref_soap_env = const_prefmap[_ns_soap_env]
 
+from spyne.model import Any
 from spyne.model import ModelBase
 from spyne.model import Array
 from spyne.model import Iterable
@@ -72,13 +75,15 @@ from spyne.model import File
 from spyne.model import ByteArray
 from spyne.model import XmlData
 from spyne.model import XmlAttribute
-from spyne.model.binary import Attachment # deprecated
+from spyne.model.binary import Attachment  # deprecated
 from spyne.model.binary import BINARY_ENCODING_BASE64
 from spyne.model.enum import EnumBase
 
 from spyne.protocol import ProtocolBase
 
-NIL_ATTR = {'{%s}nil' % _ns_xsi: 'true'}
+
+NIL_ATTR = {XSI('nil'): 'true'}
+XSI_TYPE = XSI('type')
 
 
 def _append(parent, child_elt):
@@ -87,11 +92,13 @@ def _append(parent, child_elt):
     else:
         parent.write(child_elt)
 
+
 def _gen_tagname(ns, name):
     if ns is not None:
         name = "{%s}%s" % (ns, name)
     return name
 
+
 class SchemaValidationError(Fault):
     """Raised when the input stream could not be validated by the Xml Schema."""
 
@@ -153,6 +160,28 @@ class XmlDocument(SubXmlBase):
                 ProtocolBase.SOFT_VALIDATION, XmlDocument.SCHEMA_VALIDATION).
                 Both ``'lxml'`` and ``'schema'`` values are equivalent to
                 ``XmlDocument.SCHEMA_VALIDATION``.
+    :param replace_null_with_default: If False, does not replace incoming
+        explicit null values with denoted default values. This is against Xml
+        Schema standard but consistent with other Spyne protocol
+        implementations. Set this to False if you want cross-protocol
+        compatibility.
+
+        Relevant quote from xml schema primer
+        (http://www.w3.org/TR/xmlschema-0/):
+
+        ..
+            When a value is present and is null The schema processor treats
+            defaulted elements slightly differently. When an element is declared
+            with a default value, the value of the element is whatever value
+            appears as the element's content in the instance document; if the
+            element appears without any content, the schema processor provides
+            the element with a value equal to that of the default attribute.
+            However, if the element does not appear in the instance document,
+            the schema processor does not provide the element at all. In
+            summary, the differences between element and attribute defaults can
+            be stated as: Default attribute values apply when attributes are
+            missing, and default element values apply when elements are empty.
+
     :param xml_declaration: Whether to add xml_declaration to the responses
         Default is 'True'.
     :param cleanup_namespaces: Whether to add clean up namespace declarations
@@ -191,6 +220,8 @@ class XmlDocument(SubXmlBase):
     :param huge_tree: disable security restrictions and support very deep trees
         and very long text content. (only affects libxml2 2.7+) Off by default.
     :param compact: use compact storage for short text content. On by default.
+    :param parse_xsi_type: Set to ``False`` to disable parsing of ``xsi:type``
+        attribute, effectively disabling polymorphism. Defaults to True.
     """
 
     SCHEMA_VALIDATION = type("Schema", (object,), {})
@@ -201,7 +232,12 @@ class XmlDocument(SubXmlBase):
     type = set(ProtocolBase.type)
     type.add('xml')
 
-    def __init__(self, app=None, validator=None, xml_declaration=True,
+    soap_env = const_prefmap[soap11_env]
+    ns_soap_env = soap11_env
+
+    def __init__(self, app=None, validator=None,
+                replace_null_with_default=True,
+                xml_declaration=True,
                 cleanup_namespaces=True, encoding=None, pretty_print=False,
                 attribute_defaults=False,
                 dtd_validation=False,
@@ -215,20 +251,30 @@ class XmlDocument(SubXmlBase):
                 resolve_entities=False,
                 huge_tree=False,
                 compact=True,
+                binary_encoding=None,
+                parse_xsi_type=True,
+                polymorphic=False,
             ):
-        super(XmlDocument, self).__init__(app, validator)
+
+        super(XmlDocument, self).__init__(app, validator,
+                                                binary_encoding=binary_encoding)
+
         self.xml_declaration = xml_declaration
         self.cleanup_namespaces = cleanup_namespaces
+        self.replace_null_with_default = replace_null_with_default
 
         if encoding is None:
             self.encoding = 'UTF-8'
         else:
             self.encoding = encoding
 
+        self.polymorphic = polymorphic
         self.pretty_print = pretty_print
+        self.parse_xsi_type = parse_xsi_type
 
         self.serialization_handlers = cdict({
             AnyXml: self.xml_to_parent,
+            Any: self.xml_to_parent,
             Fault: self.fault_to_parent,
             AnyDict: self.dict_to_parent,
             AnyHtml: self.html_to_parent,
@@ -243,7 +289,9 @@ class XmlDocument(SubXmlBase):
         })
 
         self.deserialization_handlers = cdict({
+            AnyHtml: self.html_from_element,
             AnyXml: self.xml_from_element,
+            Any: self.xml_from_element,
             Array: self.array_from_element,
             Fault: self.fault_from_element,
             AnyDict: self.dict_from_element,
@@ -256,7 +304,6 @@ class XmlDocument(SubXmlBase):
             ComplexModelBase: self.complex_from_element,
         })
 
-        self.log_messages = (logger.level == logging.DEBUG)
         self.parser_kwargs = dict(
             attribute_defaults=attribute_defaults,
             dtd_validation=dtd_validation,
@@ -274,9 +321,6 @@ class XmlDocument(SubXmlBase):
             encoding=encoding,
         )
 
-    def subserialize(self, ctx, cls, inst, parent, ns=None, name=None):
-        return self.to_parent(ctx, cls, inst, parent, name)
-
     def set_validator(self, validator):
         if validator in ('lxml', 'schema') or \
                                     validator is self.SCHEMA_VALIDATION:
@@ -308,7 +352,7 @@ class XmlDocument(SubXmlBase):
             else:
                 line_header = LIGHT_RED + "Response:" + END_COLOR
         finally:
-            if self.log_messages:
+            if logger.level == logging.DEBUG:
                 logger.debug("%s %s" % (line_header, ctx.method_request_string))
                 logger.debug(etree.tostring(ctx.in_document, pretty_print=True))
 
@@ -317,7 +361,7 @@ class XmlDocument(SubXmlBase):
 
         self.validation_schema = None
 
-        if value:
+        if self.validator is self.SCHEMA_VALIDATION and value is not None:
             from spyne.interface.xml_schema import XmlSchema
 
             xml_schema = XmlSchema(value.interface)
@@ -338,7 +382,7 @@ class XmlDocument(SubXmlBase):
         """Uses the iterable of string fragments in ``ctx.in_string`` to set
         ``ctx.in_document``."""
 
-        string = _bytes_join(ctx.in_string)
+        string = b''.join(ctx.in_string)
         try:
             try:
                 ctx.in_document = etree.fromstring(string,
@@ -351,7 +395,7 @@ class XmlDocument(SubXmlBase):
                 ctx.in_document = etree.fromstring(string.decode(charset),
                                                                     self.parser)
         except XMLSyntaxError as e:
-            logger_invalid.error(string)
+            logger_invalid.error("%r in string %r", e, string)
             raise Fault('Client.XMLSyntaxError', str(e))
 
     def decompose_incoming_envelope(self, ctx, message):
@@ -367,12 +411,40 @@ class XmlDocument(SubXmlBase):
             if self.validator is self.SOFT_VALIDATION and not \
                                                         cls.Attributes.nillable:
                 raise ValidationError('')
-            return cls.Attributes.default
+
+            if self.replace_null_with_default:
+                return cls.Attributes.default
+            return None
+
+        # if present, use the xsi:type="ns0:ObjectName"
+        # attribute to instantiate subclass objects
+        if self.parse_xsi_type:
+            xsi_type = element.get(XSI_TYPE, None)
+            if xsi_type is not None:
+                if ":" in xsi_type:
+                    prefix, objtype = xsi_type.split(':', 1)
+                else:
+                    prefix, objtype = None, xsi_type
+
+                ns = element.nsmap.get(prefix)
+                if ns is not None:
+                    classkey = "{%s}%s" % (ns, objtype)
+                else:
+                    raise ValidationError(xsi_type)
+
+                newclass = ctx.app.interface.classes.get(classkey, None)
+                if newclass is None:
+                    raise ValidationError(xsi_type)
+
+                cls = newclass
+                logger.debug("xsi:type overrides %r to %r", cls, newclass)
 
         handler = self.deserialization_handlers[cls]
         return handler(ctx, cls, element)
 
     def to_parent(self, ctx, cls, inst, parent, ns, *args, **kwargs):
+        cls, add_type = self.get_polymorphic_target(cls, inst)
+
         subprot = getattr(cls.Attributes, 'prot', None)
         if subprot is not None:
             return subprot.subserialize(ctx, cls, inst, parent, ns,
@@ -386,6 +458,8 @@ class XmlDocument(SubXmlBase):
         if inst is None:
             return self.null_to_parent(ctx, cls, inst, parent, ns,
                                                                 *args, **kwargs)
+
+        kwargs['add_type'] = add_type
         return handler(ctx, cls, inst, parent, ns, *args, **kwargs)
 
     def deserialize(self, ctx, message):
@@ -419,11 +493,15 @@ class XmlDocument(SubXmlBase):
         else:
             ctx.in_object = self.from_element(ctx, body_class, ctx.in_body_doc)
 
-        if self.log_messages and message is self.REQUEST:
+        if logger.level == logging.DEBUG and message is self.REQUEST:
             line_header = '%sRequest%s' % (LIGHT_GREEN, END_COLOR)
 
-            logger.debug("%s %s" % (line_header, etree.tostring(ctx.out_document,
-                    xml_declaration=self.xml_declaration, pretty_print=True)))
+            outdoc_str = None
+            if ctx.out_document is not None:
+                outdoc_str = etree.tostring(ctx.out_document,
+                        xml_declaration=self.xml_declaration, pretty_print=True)
+
+            logger.debug("%s %s" % (line_header, outdoc_str))
 
         self.event_manager.fire_event('after_deserialize', ctx)
 
@@ -490,7 +568,7 @@ class XmlDocument(SubXmlBase):
                                           pretty_print=self.pretty_print,
                                           xml_declaration=self.xml_declaration)]
 
-        if self.log_messages:
+        if logger.level == logging.DEBUG:
             logger.debug('%sResponse%s %s' % (LIGHT_RED, END_COLOR,
                             etree.tostring(ctx.out_document,
                                           pretty_print=True, encoding='UTF-8')))
@@ -516,14 +594,30 @@ class XmlDocument(SubXmlBase):
         if hasattr(ctx.out_stream, 'finish'):
             ctx.out_stream.finish()
 
-    def byte_array_to_parent(self, ctx, cls, inst, parent, ns, name='retval'):
-        _append(parent, E(_gen_tagname(ns, name),
-                       self.to_string(cls, inst, self.default_binary_encoding)))
+    def _gen_tag(self, cls, ns, name, add_type=False, **_):
+        if ns is not None:
+            name = "{%s}%s" % (ns, name)
 
-    def modelbase_to_parent(self, ctx, cls, inst, parent, ns, name='retval'):
-        _append(parent, E(_gen_tagname(ns, name), self.to_string(cls, inst)))
+        retval = E(name)
+        if add_type:
+            retval.attrib[XSI_TYPE] = cls.get_type_name_ns(self.app.interface)
 
-    def null_to_parent(self, ctx, cls, inst, parent, ns, name='retval'):
+        return retval
+
+    def byte_array_to_parent(self, ctx, cls, inst, parent, ns, name='retval',
+                                                                      **kwargs):
+        elt = self._gen_tag(cls, ns, name, **kwargs)
+        elt.text = self.to_unicode(cls, inst, self.binary_encoding)
+        _append(parent, elt)
+
+    def modelbase_to_parent(self, ctx, cls, inst, parent, ns, name='retval',
+                                                                      **kwargs):
+        elt = self._gen_tag(cls, ns, name, **kwargs)
+        elt.text = self.to_unicode(cls, inst)
+        _append(parent, elt)
+
+    def null_to_parent(self, ctx, cls, inst, parent, ns, name='retval',
+                                                                      **kwargs):
         if issubclass(cls, XmlAttribute):
             return
 
@@ -531,21 +625,27 @@ class XmlDocument(SubXmlBase):
             parent.attrib.update(NIL_ATTR)
 
         else:
-            _append(parent, E(_gen_tagname(ns, name), **NIL_ATTR))
+            elt = self._gen_tag(cls, ns, name, **kwargs)
+            elt.attrib.update(NIL_ATTR)
+            _append(parent, elt)
 
     def null_from_element(self, ctx, cls, element):
         return None
 
-    def xmldata_to_parent(self, ctx, cls, inst, parent, ns, name):
+    def xmldata_to_parent(self, ctx, cls, inst, parent, ns, name,
+                                                      add_type=False, **_):
         ns = cls._ns
         if ns is None:
             ns = cls.Attributes.sub_ns
 
         name = _gen_tagname(ns, name)
 
+        if add_type:
+            parent.attrib[XSI_TYPE] = cls.get_type_name_ns(self.app.interface)
+
         cls.marshall(self, name, inst, parent)
 
-    def xmlattribute_to_parent(self, ctx, cls, inst, parent, ns, name):
+    def xmlattribute_to_parent(self, ctx, cls, inst, parent, ns, name, **_):
         ns = cls._ns
         if ns is None:
             ns = cls.Attributes.sub_ns
@@ -554,23 +654,26 @@ class XmlDocument(SubXmlBase):
 
         if inst is not None:
             if issubclass(cls.type, (ByteArray, File)):
-                parent.set(name, self.to_string(cls.type, inst,
-                                                 self.default_binary_encoding))
+                parent.set(name, self.to_unicode(cls.type, inst,
+                                                 self.binary_encoding))
             else:
-                parent.set(name, self.to_string(cls.type, inst))
+                parent.set(name, self.to_unicode(cls.type, inst))
 
-    def attachment_to_parent(self, cls, inst, ns, parent, name='retval'):
+    def attachment_to_parent(self, cls, inst, ns, parent, name='retval', **_):
         _append(parent, E(_gen_tagname(ns, name),
-                        ''.join([b.decode('ascii') for b in cls.to_base64(inst)])))
+                    ''.join([b.decode('ascii') for b in cls.to_base64(inst)])))
 
     @coroutine
-    def gen_members_parent(self, ctx, cls, inst, parent, tag_name, subelts):
-        delay = set()
+    def gen_members_parent(self, ctx, cls, inst, parent, tag_name, subelts,
+                                                                      add_type):
+        attrib = {}
+        if add_type:
+            attrib[XSI_TYPE] = cls.get_type_name_ns(self.app.interface)
 
         if isinstance(parent, etree._Element):
-            elt = etree.SubElement(parent, tag_name)
+            elt = etree.SubElement(parent, tag_name, attrib=attrib)
             elt.extend(subelts)
-            ret = self._get_members_etree(ctx, cls, inst, elt, delay)
+            ret = self._get_members_etree(ctx, cls, inst, elt)
 
             if isgenerator(ret):
                 try:
@@ -585,10 +688,10 @@ class XmlDocument(SubXmlBase):
                         pass
 
         else:
-            with parent.element(tag_name):
+            with parent.element(tag_name, attrib=attrib):
                 for e in subelts:
                     parent.write(e)
-                ret = self._get_members_etree(ctx, cls, inst, parent, delay)
+                ret = self._get_members_etree(ctx, cls, inst, parent)
                 if isgenerator(ret):
                     try:
                         while True:
@@ -602,12 +705,12 @@ class XmlDocument(SubXmlBase):
                             pass
 
     @coroutine
-    def _get_members_etree(self, ctx, cls, inst, parent, delay):
+    def _get_members_etree(self, ctx, cls, inst, parent):
         try:
             parent_cls = getattr(cls, '__extends__', None)
 
             if not (parent_cls is None):
-                ret = self._get_members_etree(ctx, parent_cls, inst, parent, delay)
+                ret = self._get_members_etree(ctx, parent_cls, inst, parent)
                 if ret is not None:
                     try:
                         while True:
@@ -637,11 +740,6 @@ class XmlDocument(SubXmlBase):
                 if sub_name is None:
                     sub_name = k
 
-                if issubclass(v, XmlAttribute) and \
-                                        v.attribute_of in cls._type_info.keys():
-                    delay.add(k)
-                    continue
-
                 mo = v.Attributes.max_occurs
                 if subvalue is not None and mo > 1:
                     if isinstance(subvalue, PushBase):
@@ -696,32 +794,8 @@ class XmlDocument(SubXmlBase):
         except Break:
             pass
 
-        if isinstance(parent, etree._Element):
-            # attribute_of won't work with async.
-            for k in delay:
-                v = cls._type_info[k]
-
-                subvalue = getattr(inst, k, None)
-                sub_name = v.Attributes.sub_name
-                if sub_name is None:
-                    sub_name = k
-
-                a_of = v.attribute_of
-                ns = cls.__namespace__
-                attr_parents = parent.findall("{%s}%s" % (ns, a_of))
-
-                if cls._type_info[a_of].Attributes.max_occurs > 1:
-                    for subsubvalue, attr_parent in zip(subvalue, attr_parents):
-                        self.to_parent(ctx, v, subsubvalue, attr_parent,
-                                                           v.get_namespace(), k)
-
-                else:
-                    for attr_parent in attr_parents:
-                        self.to_parent(ctx, v, subvalue, attr_parent,
-                                                           v.get_namespace(), k)
-
-
-    def complex_to_parent(self, ctx, cls, inst, parent, ns, name=None):
+    def complex_to_parent(self, ctx, cls, inst, parent, ns, name=None,
+                                                           add_type=False, **_):
         sub_name = cls.Attributes.sub_name
         if sub_name is not None:
             name = sub_name
@@ -735,36 +809,44 @@ class XmlDocument(SubXmlBase):
         tag_name = _gen_tagname(ns, name)
 
         inst = cls.get_serialization_instance(inst)
-        return self.gen_members_parent(ctx, cls, inst, parent, tag_name, [])
 
-    def fault_to_parent(self, ctx, cls, inst, parent, ns, *args, **kwargs):
-        tag_name = "{%s}Fault" % _ns_soap_env
+        return self.gen_members_parent(ctx, cls, inst, parent, tag_name, [],
+                                                                       add_type)
 
-        subelts = [
-            E("faultcode", '%s:%s' % (_pref_soap_env, inst.faultcode)),
-            E("faultstring", inst.faultstring),
-            E("faultactor", inst.faultactor),
-        ]
+    def _fault_to_parent_impl(self, ctx, cls, inst, parent, ns, subelts, **_):
+        tag_name = "{%s}Fault" % self.ns_soap_env
 
         # Accepting raw lxml objects as detail is deprecated. It's also not
         # documented. It's kept for backwards-compatibility purposes.
         if isinstance(inst.detail, string_types + (etree._Element,)):
             _append(subelts, E('detail', inst.detail))
+
         elif isinstance(inst.detail, dict):
-            _append(subelts, E('detail', root_dict_to_etree(inst.detail)))
+            if len(inst.detail) > 0:
+                _append(subelts, E('detail', root_dict_to_etree(inst.detail)))
+
         elif inst.detail is None:
             pass
+
         else:
             raise TypeError('Fault detail Must be dict, got', type(inst.detail))
 
         # add other nonstandard fault subelements with get_members_etree
-        return self.gen_members_parent(ctx, cls, inst, parent, tag_name, subelts)
-
-    def schema_validation_error_to_parent(self, ctx, cls, inst, parent, ns):
-        tag_name = "{%s}Fault" % _ns_soap_env
+        return self.gen_members_parent(ctx, cls, inst, parent, tag_name,
+                                                        subelts, add_type=False)
 
+    def fault_to_parent(self, ctx, cls, inst, parent, ns, *args, **kwargs):
         subelts = [
-            E("faultcode", '%s:%s' % (_pref_soap_env, inst.faultcode)),
+            E("faultcode", '%s:%s' % (self.soap_env, inst.faultcode)),
+            E("faultstring", inst.faultstring),
+            E("faultactor", inst.faultactor),
+        ]
+
+        return self._fault_to_parent_impl(ctx, cls, inst, parent, ns, subelts)
+
+    def schema_validation_error_to_parent(self, ctx, cls, inst, parent, ns,**_):
+        subelts = [
+            E("faultcode", '%s:%s' % (self.soap_env, inst.faultcode)),
             # HACK: Does anyone know a better way of injecting raw xml entities?
             E("faultstring", html.fromstring(inst.faultstring).text),
             E("faultactor", inst.faultactor),
@@ -773,24 +855,27 @@ class XmlDocument(SubXmlBase):
             _append(subelts, E('detail', inst.detail))
 
         # add other nonstandard fault subelements with get_members_etree
-        return self.gen_members_parent(ctx, cls, inst, parent, tag_name, subelts)
+        return self._fault_to_parent_impl(ctx, cls, inst, parent, ns, subelts)
 
-    def enum_to_parent(self, ctx, cls, inst, parent, ns, name='retval'):
-        self.modelbase_to_parent(ctx, cls, str(inst), parent, ns, name)
+    def enum_to_parent(self, ctx, cls, inst, parent, ns, name='retval', **kwargs):
+        self.modelbase_to_parent(ctx, cls, str(inst), parent, ns, name, **kwargs)
 
-    def xml_to_parent(self, ctx, cls, inst, parent, ns, name):
+    def xml_to_parent(self, ctx, cls, inst, parent, ns, name, **_):
         if isinstance(inst, str) or isinstance(inst, unicode):
             inst = etree.fromstring(inst)
 
         _append(parent, E(_gen_tagname(ns, name), inst))
 
-    def html_to_parent(self, ctx, cls, inst, parent, ns, name):
-        if isinstance(inst, str) or isinstance(inst, unicode):
+    def any_to_parent(self, ctx, cls, inst, parent, ns, name, **_):
+        _append(parent, E(_gen_tagname(ns, name), inst))
+
+    def html_to_parent(self, ctx, cls, inst, parent, ns, name, **_):
+        if isinstance(inst, string_types) and len(inst) > 0:
             inst = html.fromstring(inst)
 
         _append(parent, E(_gen_tagname(ns, name), inst))
 
-    def dict_to_parent(self, ctx, cls, inst, parent, ns, name):
+    def dict_to_parent(self, ctx, cls, inst, parent, ns, name, **_):
         elt = E(_gen_tagname(ns, name))
         dict_to_etree(inst, elt)
 
@@ -804,14 +889,14 @@ class XmlDocument(SubXmlBase):
         # this is for validating cls.Attributes.{min,max}_occurs
         frequencies = defaultdict(int)
 
-        xtba_key, xtba_type = cls.Attributes._xml_tag_body_as
-        if xtba_key is not None:
-            if issubclass(xtba_type.type, (ByteArray, File)):
-                value = self.from_string(xtba_type.type, elt.text,
-                                                    self.default_binary_encoding)
-            else:
-                value = self.from_string(xtba_type.type, elt.text)
-            setattr(inst, xtba_key, value)
+        if cls.Attributes._xml_tag_body_as is not None:
+            for xtba_key, xtba_type in cls.Attributes._xml_tag_body_as:
+                if issubclass(xtba_type.type, (ByteArray, File)):
+                    value = self.from_unicode(xtba_type.type, elt.text,
+                                                        self.binary_encoding)
+                else:
+                    value = self.from_unicode(xtba_type.type, elt.text)
+                setattr(inst, xtba_key, value)
 
         # parse input to set incoming data to related attributes.
         for c in elt:
@@ -846,19 +931,16 @@ class XmlDocument(SubXmlBase):
                     if member is None:
                         continue
 
-                if (not issubclass(member, XmlAttribute)) or \
-                                                         member.attribute_of == key:
-                    continue
-
+                mo = member.Attributes.max_occurs
                 if mo > 1:
                     value = getattr(inst, key, None)
                     if value is None:
                         value = []
 
-                    value.append(self.from_string(member.type, value_str))
+                    value.append(self.from_unicode(member.type, value_str))
 
                 else:
-                    value = self.from_string(member.type, value_str)
+                    value = self.from_unicode(member.type, value_str)
 
                 setattr(inst, key, value)
 
@@ -868,15 +950,14 @@ class XmlDocument(SubXmlBase):
                 member, key = cls._type_info_alt.get(key, (None, key))
                 if member is None:
                     continue
-
-            if (not issubclass(member, XmlAttribute)) or member.attribute_of == key:
+            if not issubclass(member, XmlAttribute):
                 continue
 
             if issubclass(member.type, (ByteArray, File)):
-                value = self.from_string(member.type, value_str,
-                                                       self.default_binary_encoding)
+                value = self.from_unicode(member.type, value_str,
+                                                       self.binary_encoding)
             else:
-                value = self.from_string(member.type, value_str)
+                value = self.from_unicode(member.type, value_str)
 
             setattr(inst, key, value)
 
@@ -931,6 +1012,19 @@ class XmlDocument(SubXmlBase):
 
         return retval
 
+    def html_from_element(self, ctx, cls, element):
+        children = element.getchildren()
+        retval = None
+
+        if len(children) == 1:
+            retval = children[0]
+        # this is actually a workaround to a case that should never exist --
+        # anyXml types should only have one child tag.
+        elif len(children) > 1:
+            retval = E.html(*children)
+
+        return retval
+
     def dict_from_element(self, ctx, cls, element):
         children = element.getchildren()
         if children:
@@ -947,7 +1041,7 @@ class XmlDocument(SubXmlBase):
         if s is None:
             s = ''
 
-        retval = self.from_string(cls, s)
+        retval = self.from_unicode(cls, s)
 
         if self.validator is self.SOFT_VALIDATION and not (
                                               cls.validate_native(cls, retval)):
@@ -960,7 +1054,7 @@ class XmlDocument(SubXmlBase):
                                         cls.validate_string(cls, element.text)):
             raise ValidationError(element.text)
 
-        retval = self.from_string(cls, element.text)
+        retval = self.from_unicode(cls, element.text)
 
         if self.validator is self.SOFT_VALIDATION and not (
                                             cls.validate_native(cls, retval)):
@@ -973,7 +1067,7 @@ class XmlDocument(SubXmlBase):
                                         cls.validate_string(cls, element.text)):
             raise ValidationError(element.text)
 
-        retval = self.from_string(cls, element.text, self.default_binary_encoding)
+        retval = self.from_unicode(cls, element.text, self.binary_encoding)
 
         if self.validator is self.SOFT_VALIDATION and not (
                                             cls.validate_native(cls, retval)):
