@@ -21,9 +21,6 @@
 Currently, only :class:`spyne.protocol.yaml.YamlDocument` is supported.
 
 Initially released in 2.10.0-rc.
-
-This module is EXPERIMENTAL. You may not recognize the code here next time you
-look at it.
 """
 
 from __future__ import absolute_import
@@ -86,19 +83,20 @@ class YamlDocument(HierDictDocument):
                                         ignore_wrappers=True,
                                         complex_as=dict,
                                         ordered=False,
+                                        polymorphic=False,
                                         # YamlDocument specific
                                         safe=True, **kwargs):
 
         super(YamlDocument, self).__init__(app, validator, mime_type,
-                            ignore_uncap, ignore_wrappers, complex_as, ordered)
+                ignore_uncap, ignore_wrappers, complex_as, ordered, polymorphic)
 
-        self._from_string_handlers[Double] = lambda cls, val: val
-        self._from_string_handlers[Boolean] = lambda cls, val: val
-        self._from_string_handlers[Integer] = lambda cls, val: val
+        self._from_unicode_handlers[Double] = lambda cls, val: val
+        self._from_unicode_handlers[Boolean] = lambda cls, val: val
+        self._from_unicode_handlers[Integer] = lambda cls, val: val
 
-        self._to_string_handlers[Double] = lambda cls, val: val
-        self._to_string_handlers[Boolean] = lambda cls, val: val
-        self._to_string_handlers[Integer] = lambda cls, val: val
+        self._to_unicode_handlers[Double] = lambda cls, val: val
+        self._to_unicode_handlers[Boolean] = lambda cls, val: val
+        self._to_unicode_handlers[Integer] = lambda cls, val: val
 
         self.in_kwargs = dict(kwargs)
         self.out_kwargs = dict(kwargs)
@@ -122,7 +120,7 @@ class YamlDocument(HierDictDocument):
             in_string_encoding = 'UTF-8'
 
         try:
-            ctx.in_document = yaml.load(''.join(ctx.in_string).decode(
+            ctx.in_document = yaml.load(b''.join(ctx.in_string).decode(
                          in_string_encoding), **self.in_kwargs)
 
         except ParserError as e:
