@@ -27,8 +27,20 @@ This module is EXPERIMENTAL. Tests and patches are welcome.
 from __future__ import absolute_import
 
 import logging
+logger = logging.getLogger(__name__)
+
 from functools import update_wrapper
-from django.conf import settings
+
+from spyne.application import get_fault_string_from_exception, Application
+from spyne.auxproc import process_contexts
+from spyne.interface import AllYourInterfaceDocuments
+from spyne.model.fault import Fault
+from spyne.protocol.soap import Soap11
+from spyne.protocol.http import HttpRpc
+from spyne.server.http import HttpBase, HttpMethodContext, HttpTransportContext
+from spyne.server.wsgi import WsgiApplication
+from spyne.util import _bytes_join
+
 from django.http import HttpResponse, HttpResponseNotAllowed, Http404
 from django.views.decorators.csrf import csrf_exempt
 
@@ -38,24 +50,14 @@ except ImportError as e:
     def StreamingHttpResponse(*args, **kwargs):
         raise e
 
-from spyne.application import get_fault_string_from_exception, Application
-from spyne.auxproc import process_contexts
-from spyne.interface import AllYourInterfaceDocuments
-from spyne.model.fault import Fault
-from spyne.protocol.soap import Soap11
-from spyne.protocol.http import HttpRpc
-from spyne.server.http import HttpBase, HttpMethodContext
-from spyne.server.wsgi import WsgiApplication
-
-
-logger = logging.getLogger(__name__)
-
-
 class DjangoApplication(WsgiApplication):
     """You should use this for regular RPC."""
 
     HttpResponseObject = HttpResponse
 
+    # noinspection PyMethodOverriding
+    # because this is VERY similar to a Wsgi app
+    # but not that much.
     def __call__(self, request):
         retval = self.HttpResponseObject()
 
@@ -88,7 +90,7 @@ class DjangoApplication(WsgiApplication):
         return retval
 
     def set_response(self, retval, response):
-        retval.content = ''.join(response)
+        retval.content = _bytes_join(response, b"")
 
 
 class StreamingDjangoApplication(DjangoApplication):
@@ -103,12 +105,34 @@ class StreamingDjangoApplication(DjangoApplication):
         retval.streaming_content = response
 
 
+class DjangoHttpTransportContext(HttpTransportContext):
+    def get_path(self):
+        return self.req.path
+
+    def get_request_method(self):
+        return self.req.method
+
+    def get_request_content_type(self):
+        return self.req.META['CONTENT_TYPE']
+
+    def get_path_and_qs(self):
+        return self.req.get_full_path()
+
+    def get_cookie(self, key):
+        return self.req.COOKIES[key]
+
+
+class DjangoHttpMethodContext(HttpMethodContext):
+    default_transport_context = DjangoHttpTransportContext
+
+
 class DjangoServer(HttpBase):
     """Server talking in Django request/response objects."""
 
-    def __init__(self, app, chunked=False):
+    def __init__(self, app, chunked=False, cache_wsdl=True):
         super(DjangoServer, self).__init__(app, chunked=chunked)
         self._wsdl = None
+        self._cache_wsdl = cache_wsdl
 
     def handle_rpc(self, request, *args, **kwargs):
         """Handle rpc request.
@@ -153,9 +177,7 @@ class DjangoServer(HttpBase):
         if self.chunked:
             response = StreamingHttpResponse(p_ctx.out_string)
         else:
-            response = HttpResponse(''.join(p_ctx.out_string))
-
-        p_ctx.close()
+            response = HttpResponse(b''.join(p_ctx.out_string))
 
         return self.response(response, p_ctx, others)
 
@@ -174,10 +196,14 @@ class DjangoServer(HttpBase):
             # section can be safely repeated in another concurrent thread.
             doc = AllYourInterfaceDocuments(self.app.interface)
             doc.wsdl11.build_interface_document(request.build_absolute_uri())
-            self._wsdl = doc.wsdl11.get_interface_document()
+            wsdl = doc.wsdl11.get_interface_document()
 
-        ctx.transport.wsdl = self._wsdl
-        ctx.close()
+            if self._cache_wsdl:
+                self._wsdl = wsdl
+        else:
+            wsdl = self._wsdl
+
+        ctx.transport.wsdl = wsdl
 
         response = HttpResponse(ctx.transport.wsdl)
         return self.response(response, ctx, ())
@@ -195,7 +221,7 @@ class DjangoServer(HttpBase):
                            p_ctx.out_protocol.fault_to_http_response_code(error)
 
         self.get_out_string(p_ctx)
-        resp = HttpResponse(''.join(p_ctx.out_string))
+        resp = HttpResponse(b''.join(p_ctx.out_string))
         return self.response(resp, p_ctx, others, error)
 
     def get_contexts(self, request):
@@ -205,13 +231,11 @@ class DjangoServer(HttpBase):
         :returns: generated contexts
         """
 
-        initial_ctx = HttpMethodContext(self, request,
-                                        self.app.out_protocol.mime_type)
+        initial_ctx = DjangoHttpMethodContext(self, request,
+                                                self.app.out_protocol.mime_type)
 
-        initial_ctx.in_string = request.body
-        in_string_charset = request.encoding or settings.DEFAULT_CHARSET
-
-        return self.generate_contexts(initial_ctx, in_string_charset)
+        initial_ctx.in_string = [request.body]
+        return self.generate_contexts(initial_ctx)
 
     def response(self, response, p_ctx, others, error=None):
         """Populate response with transport headers and finalize it.
@@ -236,6 +260,8 @@ class DjangoServer(HttpBase):
             # Report but ignore any exceptions from auxiliary methods.
             logger.exception(e)
 
+        p_ctx.close()
+
         return response
 
 
@@ -251,6 +277,7 @@ class DjangoView(object):
     out_protocol = Soap11()
     interface = None
     chunked = False
+    cache_wsdl = True
 
     http_method_names = ['get', 'post', 'put', 'patch', 'delete', 'head',
                          'options', 'trace']
@@ -266,7 +293,6 @@ class DjangoView(object):
         """Register application, server and create new view.
 
         :returns: callable view function
-
         """
 
         # sanitize keyword arguments
@@ -281,7 +307,9 @@ class DjangoView(object):
                                 "attributes of the class." % (cls.__name__,
                                                               key))
 
-        get = lambda key: initkwargs.get(key) or getattr(cls, key)
+        def get(key):
+            value = initkwargs.get(key)
+            return value if value is not None else getattr(cls, key)
 
         application = get('application') or Application(
             services=get('services'),
@@ -289,10 +317,10 @@ class DjangoView(object):
             name=get('name'),
             in_protocol=get('in_protocol'),
             out_protocol=get('out_protocol'),
-            interface=get('interface')
         )
         server = get('server') or DjangoServer(application,
-                                               chunked=get('chunked'))
+                                               chunked=get('chunked'),
+                                               cache_wsdl=get('cache_wsdl'))
 
         def view(request, *args, **kwargs):
             self = cls(server=server, **initkwargs)
@@ -329,7 +357,6 @@ class DjangoView(object):
     def post(self, request, *args, **kwargs):
         return self.server.handle_rpc(request, *args, **kwargs)
 
-
     def http_method_not_allowed(self, request, *args, **kwargs):
         logger.warning('Method Not Allowed (%s): %s', request.method,
                        request.path, extra={'status_code': 405, 'request':
