@@ -19,11 +19,21 @@
 
 from collections import defaultdict
 
-from spyne import TransportContext
-from spyne import MethodContext
-from spyne.protocol.http import HttpPattern
+from spyne import TransportContext, MethodDescriptor, MethodContext, Redirect
 from spyne.server import ServerBase
-from spyne.const.http import gen_body_redirect, HTTP_301, HTTP_302
+from spyne.const.http import gen_body_redirect, \
+    HTTP_301, HTTP_302, HTTP_303, HTTP_307
+from spyne.protocol.http import HttpPattern
+
+
+class HttpRedirect(Redirect):
+    def do_redirect(self):
+        if not isinstance(self.ctx.transport, HttpTransportContext):
+            if self.orig_exc is not None:
+                raise self.orig_exc
+            raise TypeError(self.ctx.transport)
+
+        self.ctx.transport.respond(HTTP_302, location=self.location)
 
 
 class HttpTransportContext(TransportContext):
@@ -59,7 +69,7 @@ class HttpTransportContext(TransportContext):
 
     def respond(self, resp_code, **kwargs):
         self.resp_code = resp_code
-        if resp_code in (HTTP_301, HTTP_302):
+        if resp_code in (HTTP_301, HTTP_302, HTTP_303, HTTP_307):
             l = kwargs.pop('location')
             self.resp_headers['Location'] = l
             self.parent.out_string = [gen_body_redirect(resp_code, l)]
@@ -69,6 +79,18 @@ class HttpTransportContext(TransportContext):
             # So that deserialization is skipped.
             self.parent.out_string = []
 
+    def get_path(self):
+        raise NotImplementedError()
+
+    def get_request_method(self):
+        raise NotImplementedError()
+
+    def get_request_content_type(self):
+        raise NotImplementedError()
+
+    def get_path_and_qs(self):
+        raise NotImplementedError()
+
     def get_cookie(self, key):
         raise NotImplementedError()
 
@@ -92,10 +114,10 @@ class HttpMethodContext(MethodContext):
     default_transport_context = HttpTransportContext
 
     def __init__(self, transport, req_env, content_type):
-        super(HttpMethodContext, self).__init__(transport)
+        super(HttpMethodContext, self).__init__(transport, MethodContext.SERVER)
 
-        self.transport = self.default_transport_context(self, transport, req_env,
-                                                                   content_type)
+        self.transport = self.default_transport_context(self, transport,
+                                                          req_env, content_type)
         """Holds the WSGI-specific information"""
 
     def set_out_protocol(self, what):
@@ -172,21 +194,23 @@ class HttpBase(ServerBase):
                 for k,v in match.groupdict().items():
                     params[k].append(v)
 
-            address = patt.address
-            if address is None:
-                address = ctx.descriptor.name
+            assert patt.address is not None
 
-            if address:
-                match = patt.address_re.match(path)
-                if match is None:
-                    continue
-                if not (match.span() == (0, len(path))):
-                    continue
-                for k,v in match.groupdict().items():
-                    params[k].append(v)
+            match = patt.address_re.match(path)
+            if match is None:
+                continue
+            if not (match.span() == (0, len(path))):
+                continue
+            for k,v in match.groupdict().items():
+                params[k].append(v)
 
-            ctx.method_request_string = '{%s}%s' % (self.app.interface.get_tns(),
-                                                    patt.endpoint.name)
+            d = patt.endpoint
+            assert isinstance(d, MethodDescriptor)
+            if d.parent_class is not None and d.in_message_name_override:
+                ctx.method_request_string = '%s.%s' % (
+                                           d.in_message.get_type_name(), d.name)
+            else:
+                ctx.method_request_string = patt.endpoint.name
             break
 
         return params
