@@ -60,6 +60,10 @@ from twisted.web.http import CACHED
 from twisted.python.log import err
 from twisted.internet.defer import Deferred
 
+from spyne import BODY_STYLE_BARE, BODY_STYLE_EMPTY, Redirect
+from spyne.application import logger_server
+from spyne.application import get_fault_string_from_exception
+
 from spyne.error import InternalError
 from spyne.auxproc import process_contexts
 from spyne.const.ansi_color import LIGHT_GREEN
@@ -99,7 +103,7 @@ def _render_file(file, request):
 
     try:
         fileForReading = file.openForReading()
-    except IOError, e:
+    except IOError as e:
         import errno
 
         if e[0] == errno.EACCES:
@@ -156,6 +160,18 @@ class TwistedHttpTransportContext(HttpTransportContext):
     def get_cookie(self, key):
         return self.req.getCookie(key)
 
+    def get_path(self):
+        return self.req.URLPath().path
+
+    def get_path_and_qs(self):
+        return self.req.uri
+
+    def get_request_method(self):
+        return self.req.method
+
+    def get_request_content_type(self):
+        return self.req.getHeader("Content-Type")
+
 
 class TwistedHttpMethodContext(HttpMethodContext):
     default_transport_context = TwistedHttpTransportContext
@@ -200,7 +216,7 @@ class TwistedHttpTransport(HttpBase):
 
         if ctx.method_request_string is None: # no pattern match
             ctx.method_request_string = '{%s}%s' % (self.app.interface.get_tns(),
-                                                    request.path.split('/')[-1])
+                                                request.path.rsplit('/', 1)[-1])
 
         logger.debug("%sMethod name: %r%s" % (LIGHT_GREEN,
                                           ctx.method_request_string, END_COLOR))
@@ -292,6 +308,7 @@ class TwistedWebResource(Resource):
     def __init__(self, app, chunked=False, max_content_length=2 * 1024 * 1024,
                                            block_length=8 * 1024, prepath=None):
         Resource.__init__(self)
+        self.app = app
 
         self.http_transport = TwistedHttpTransport(app, chunked,
                                             max_content_length, block_length)
@@ -310,8 +327,6 @@ class TwistedWebResource(Resource):
             else:
                 request.realprepath = self.prepath
 
-        request.realpostpath = request.path[len(request.realprepath):]
-
         if path in self.children:
             retval = self.children[path]
         else:
@@ -319,6 +334,8 @@ class TwistedWebResource(Resource):
 
         if isinstance(retval, NoResource):
             retval = self
+        else:
+            request.realpostpath = request.path[len(request.realprepath):]
 
         return retval
 
@@ -329,6 +346,8 @@ class TwistedWebResource(Resource):
         return self.handle_rpc(request)
 
     def handle_rpc_error(self, p_ctx, others, error, request):
+        logger.error(error)
+
         resp_code = p_ctx.transport.resp_code
         # If user code set its own response code, don't touch it.
         if resp_code is None:
@@ -451,8 +470,6 @@ def _eb_request_finished(retval, request, p_ctx):
 def _init_push(ret, request, p_ctx, others, resource):
     assert isinstance(ret, PushBase)
 
-    p_ctx.out_stream = request
-
     # fire events
     p_ctx.app.event_manager.fire_event('method_return_push', p_ctx)
     if p_ctx.service_class is not None:
@@ -501,16 +518,22 @@ def _cb_deferred(ret, request, p_ctx, others, resource, cb=True):
 
     om = p_ctx.descriptor.out_message
     single_class = None
-    if cb and ((not issubclass(om, ComplexModelBase)) or len(om._type_info) <= 1):
-        p_ctx.out_object = [ret]
-        if len(om._type_info) == 1:
-            single_class, = om._type_info.values()
+    if cb:
+        if p_ctx.descriptor.body_style in (BODY_STYLE_BARE, BODY_STYLE_EMPTY):
+            p_ctx.out_object = [ret]
 
+        elif (not issubclass(om, ComplexModelBase)) or len(om._type_info) <= 1:
+            p_ctx.out_object = [ret]
+            if len(om._type_info) == 1:
+                single_class, = om._type_info.values()
+        else:
+            p_ctx.out_object = ret
     else:
         p_ctx.out_object = ret
 
     retval = NOT_DONE_YET
 
+    p_ctx.out_stream = request
     if isinstance(ret, PushBase):
         retval = _init_push(ret, request, p_ctx, others, resource)
 
@@ -525,6 +548,13 @@ def _cb_deferred(ret, request, p_ctx, others, resource, cb=True):
         if retval != NOT_DONE_YET and cb:
             request.write(retval)
             request.finish()
+            p_ctx.close()
+        else:
+            def _close_only_context(ret):
+                p_ctx.close()
+
+            request.notifyFinish().addCallback(_close_only_context)
+            request.notifyFinish().addErrback(_eb_request_finished, request, p_ctx)
 
     else:
         resource.http_transport.get_out_string(p_ctx)
@@ -540,12 +570,58 @@ def _cb_deferred(ret, request, p_ctx, others, resource, cb=True):
     return retval
 
 
-def _eb_deferred(retval, request, p_ctx, others, resource):
-    p_ctx.out_error = retval.value
-    if not issubclass(retval.type, Fault):
-        retval.printTraceback()
-        p_ctx.out_error = InternalError(retval.value)
+def _eb_deferred(ret, request, p_ctx, others, resource):
+    app = p_ctx.app
+
+    # DRY this with what's in Application.process_request
+    if issubclass(ret.type, Redirect):
+        try:
+            ret.value.do_redirect()
+
+            # Now that the processing is switched to the outgoing message,
+            # point ctx.protocol to ctx.out_protocol
+            p_ctx.protocol = p_ctx.outprot_ctx
+
+            _cb_deferred(None, request, p_ctx, others, resource, cb=False)
+
+            # fire events
+            app.event_manager.fire_event('method_redirect', p_ctx)
+            if p_ctx.service_class is not None:
+                p_ctx.service_class.event_manager.fire_event(
+                    'method_redirect', p_ctx)
+
+        except Exception as e:
+            logger_server.exception(e)
+            p_ctx.out_error = Fault('Server', get_fault_string_from_exception(e))
+
+            # fire events
+            app.event_manager.fire_event('method_redirect_exception', p_ctx)
+            if p_ctx.service_class is not None:
+                p_ctx.service_class.event_manager.fire_event(
+                    'method_redirect_exception', p_ctx)
+
+    elif issubclass(ret.type, Fault):
+        p_ctx.out_error = ret.value
+
+        ret = resource.handle_rpc_error(p_ctx, others, p_ctx.out_error, request)
+
+        # fire events
+        app.event_manager.fire_event('method_exception_object', p_ctx)
+        if p_ctx.service_class is not None:
+            p_ctx.service_class.event_manager.fire_event(
+                                               'method_exception_object', p_ctx)
+
+        request.write(ret)
+
+    else:
+        p_ctx.out_error = ret.value
+        ret.printTraceback()
+        p_ctx.out_error = InternalError(ret.value)
+
+        # fire events
+        app.event_manager.fire_event('method_exception_object', p_ctx)
+        if p_ctx.service_class is not None:
+            p_ctx.service_class.event_manager.fire_event(
+                                               'method_exception_object', p_ctx)
 
-    ret = resource.handle_rpc_error(p_ctx, others, p_ctx.out_error, request)
-    request.write(ret)
     request.finish()
