@@ -30,10 +30,7 @@ import cgi
 import threading
 import itertools
 
-try:
-    from urllib.parse import unquote
-except ImportError: # Python 2
-    from urlparse import unquote
+from spyne.util.six.moves.urllib.parse import unquote, quote
 
 try:
     from werkzeug.formparser import parse_form_data
@@ -41,12 +38,9 @@ except ImportError as import_error:
     def parse_form_data(*args, **kwargs):
         raise import_error
 
-from spyne.util.six import string_types, BytesIO, PY3
-if PY3:
-    from http.cookies import SimpleCookie
-else:
-    from Cookie import SimpleCookie
 
+from spyne.util.six import string_types, BytesIO
+from spyne.util.six.moves.http_cookies import SimpleCookie
 
 from spyne.application import get_fault_string_from_exception
 from spyne.auxproc import process_contexts
@@ -100,6 +94,7 @@ def _parse_qs(qs):
 
     return retval
 
+
 def _get_http_headers(req_env):
     retval = {}
 
@@ -114,11 +109,11 @@ def _gen_http_headers(headers):
     retval = []
 
     for k,v in headers.items():
-        if isinstance(v, (list,tuple)):
+        if isinstance(v, (list, tuple)):
             for v2 in v:
-                retval.append((k,v2))
+                retval.append((k, v2))
         else:
-            retval.append((k,v))
+            retval.append((k, v))
 
     return retval
 
@@ -128,7 +123,8 @@ class WsgiTransportContext(HttpTransportContext):
     :class:`WsgiMethodContext` class."""
 
     def __init__(self, parent, transport, req_env, content_type):
-        super(WsgiTransportContext, self).__init__(parent, transport, req_env, content_type)
+        super(WsgiTransportContext, self).__init__(parent, transport,
+                                                          req_env, content_type)
 
         self.req_env = self.req
         """WSGI Request environment"""
@@ -136,6 +132,16 @@ class WsgiTransportContext(HttpTransportContext):
         self.req_method = req_env.get('REQUEST_METHOD', None)
         """HTTP Request verb, as a convenience to users."""
 
+    def get_path(self):
+        return self.req_env['PATH_INFO']
+
+    def get_path_and_qs(self):
+        retval = quote(self.req_env.get('PATH_INFO', ''))
+        qs = self.req_env.get('QUERY_STRING', None)
+        if qs is not None:
+            retval += '?' + qs
+        return retval
+
     def get_cookie(self, key):
         cookie_string = self.req_env.get('HTTP_COOKIE', None)
         if cookie_string is None:
@@ -146,6 +152,12 @@ class WsgiTransportContext(HttpTransportContext):
 
         return cookie.get(key, None).value
 
+    def get_request_method(self):
+        return self.req['REQUEST_METHOD'].upper()
+
+    def get_request_content_type(self):
+        return self.req.get("CONTENT_TYPE", None)
+
 
 class WsgiMethodContext(HttpMethodContext):
     """The WSGI-Specific method context. WSGI-Specific information is stored in
@@ -289,9 +301,11 @@ class WsgiApplication(HttpBase):
                                                     str(len(ctx.transport.wsdl))
         start_response(HTTP_200, _gen_http_headers(ctx.transport.resp_headers))
 
+        retval = ctx.transport.wsdl
+
         ctx.close()
 
-        return [ctx.transport.wsdl]
+        return [retval]
 
     def handle_error(self, p_ctx, others, error, start_response):
         """Serialize errors to an iterable of strings and return them.
@@ -397,7 +411,7 @@ class WsgiApplication(HttpBase):
         # code run until first yield, which lets it set response headers and
         # whatnot before calling start_response. Is there a better way?
         try:
-            len(p_ctx.out_string) # generator?
+            len(p_ctx.out_string)  # generator?
 
             # nope
             p_ctx.transport.resp_headers['Content-Length'] = \
@@ -469,29 +483,13 @@ class WsgiApplication(HttpBase):
                 raise RequestTooLongError()
 
             data = istream.read(bytes_to_read)
-            if data is None or data == '':
+            if data is None or len(data) == 0:
                 break
 
             bytes_read += len(data)
 
             yield data
 
-    def generate_map_adapter(self, ctx):
-        """This function runs on first request because it needs the
-        `'SERVER_NAME'` from the wsgi request environment.
-        """
-
-        try:
-            self._mtx_build_map_adapter.acquire()
-            if self._map_adapter is None:
-                # If url map is not bound before, bind url_map
-                req_env = ctx.transport.req_env
-                self._map_adapter = self._http_patterns.bind(
-                                                    req_env['SERVER_NAME'], "/")
-
-        finally:
-            self._mtx_build_map_adapter.release()
-
     def decompose_incoming_envelope(self, prot, ctx, message):
         """This function is only called by the HttpRpc protocol to have the wsgi
         environment parsed into ``ctx.in_body_doc`` and ``ctx.in_header_doc``.
