@@ -24,8 +24,10 @@ This module contains the :class:`ServiceBase` class and its helper objects.
 import logging
 logger = logging.getLogger(__name__)
 
-from spyne.util.six import add_metaclass, string_types
+import collections
+
 from spyne import EventManager
+from spyne.util import six
 from spyne.util.oset import oset
 
 
@@ -50,7 +52,12 @@ class ServiceBaseMeta(type):
                 setattr(self, k, staticmethod(descriptor.function))
                 descriptor.reset_function(getattr(self, k))
 
-                getattr(self, k).descriptor = descriptor
+                try:
+                    getattr(self, k).descriptor = descriptor
+                except AttributeError as e:
+                    pass
+                    # FIXME: this fails with builtins. Temporary hack while we
+                    # investigate whether we really need this or not
                 descriptor.service_class = self
 
                 self.public_methods[k] = descriptor
@@ -81,7 +88,7 @@ class ServiceBaseMeta(type):
         return self.__has_aux_methods
 
 
-@add_metaclass(ServiceBaseMeta)
+@six.add_metaclass(ServiceBaseMeta)
 class ServiceBase(object):
     """The ``ServiceBase`` class is the base class for all service definitions.
 
@@ -90,6 +97,10 @@ class ServiceBase(object):
     auxiliary processors. The :func:`spyne.decorator.srpc` decorator or its
     wrappers should be used to flag public methods.
 
+    This class is designed to be subclassed just once. You're supposed to
+    combine ServiceBase subclasses in order to get the public method mix you
+    want.
+
     It is a natural abstract base class, because it's of no use without any
     method definitions, hence the 'Base' suffix in the name.
 
@@ -125,11 +136,6 @@ class ServiceBase(object):
             the client.
     """
 
-    __tns__ = None
-    """For internal use only. You should use the ``tns`` argument to the
-    :class:`spyne.application.Application` constructor to define the target
-    namespace."""
-
     __in_header__ = None
     """The incoming header object that the methods under this service definition
     accept."""
@@ -155,8 +161,8 @@ class ServiceBase(object):
         return cls.__name__
 
     @classmethod
-    def get_service_key(cls):
-        return '{%s}%s' % (cls.get_tns(), cls.get_service_name())
+    def get_service_key(cls, app):
+        return '{%s}%s' % (app.tns, cls.get_service_name())
 
     @classmethod
     def get_service_name(cls):
@@ -169,19 +175,6 @@ class ServiceBase(object):
     def get_port_types(cls):
         return cls.__port_types__
 
-    @classmethod
-    def get_tns(cls):
-        if not (cls.__tns__ is None):
-            return cls.__tns__
-
-        retval = cls.__module__
-
-        if cls.__module__ == '__main__':
-            service_name = cls.get_service_class_name().split('.')[-1]
-            retval = '.'.join((service_name, service_name))
-
-        return retval
-
     @classmethod
     def _has_callbacks(cls):
         """Determines if this service definition has callback methods or not."""
@@ -203,7 +196,18 @@ class ServiceBase(object):
         """
 
         if ctx.function is not None:
+            args = ctx.in_object
+
+            # python3 wants a proper sequence as *args
+            assert not isinstance(args, six.string_types)
+            if not isinstance(args, collections.Sequence):
+                args = tuple(args)
+
             if ctx.descriptor.no_ctx:
-                return ctx.function(*ctx.in_object)
+                return ctx.function(*args)
             else:
-                return ctx.function(ctx, *ctx.in_object)
+                return ctx.function(ctx, *args)
+
+    @classmethod
+    def initialize(cls, app):
+        pass
