@@ -21,14 +21,13 @@ import unittest
 
 from spyne.client.http import HttpClient
 from spyne.test.interop._test_soap_client_base import SpyneClientTestBase
-from spyne.test.interop.server.soap_http_basic import soap_application
-from spyne.util.etreeconv import root_dict_to_etree
+from spyne.test.interop.server.soap11.soap_http_basic import soap11_application
 
 class TestSpyneHttpClient(SpyneClientTestBase, unittest.TestCase):
     def setUp(self):
         SpyneClientTestBase.setUp(self, 'http')
 
-        self.client = HttpClient('http://localhost:9754/', soap_application)
+        self.client = HttpClient('http://localhost:9754/', soap11_application)
         self.ns = "spyne.test.interop.server"
 
 
