@@ -19,8 +19,11 @@
 
 import unittest
 
+from suds.sax.parser import Parser
 from suds.client import Client
+from suds.plugin import MessagePlugin
 from suds import WebFault
+
 from datetime import datetime
 
 from spyne.test.interop._test_soap_client_base import SpyneClientTestBase
@@ -30,11 +33,17 @@ suds_logger = logging.getLogger('suds')
 suds_logger.setLevel(logging.INFO)
 
 
+class LastReceivedPlugin(MessagePlugin):
+    def received(self, context):
+        sax = Parser()
+        self.reply = sax.parse(string=context.reply)
+
+
 class TestSuds(SpyneClientTestBase, unittest.TestCase):
     def setUp(self):
         SpyneClientTestBase.setUp(self, 'http')
 
-        self.client = Client("http://localhost:9754/?wsdl", cache=None)
+        self.client = Client("http://localhost:9754/?wsdl", cache=None, plugins=[LastReceivedPlugin()])
         self.ns = "spyne.test.interop.server"
 
     def test_echo_datetime(self):
@@ -98,7 +107,7 @@ class TestSuds(SpyneClientTestBase, unittest.TestCase):
 
     def test_validation(self):
         non_nillable_class = self.client.factory.create(
-                                                "{hunk.sunk}NonNillableClass")
+                                                  "{hunk.sunk}NonNillableClass")
         non_nillable_class.i = 6
         non_nillable_class.s = None
 
@@ -179,7 +188,7 @@ class TestSuds(SpyneClientTestBase, unittest.TestCase):
         self.assertEquals(ret[1].returnDate, out_trace_header.returnDate)
         # Control the reply soap header (in an unelegant way but this is the
         # only way with suds)
-        soapheaders = self.client.last_received().getChild("Envelope").getChild("Header")
+        soapheaders = self.client.options.plugins[0].reply.getChild("Envelope").getChild("Header")
         soap_out_header = soapheaders.getChild('OutHeader')
         self.assertEquals('T'.join((out_header.dt.date().isoformat(),
                                     out_header.dt.time().isoformat())),
@@ -203,7 +212,7 @@ class TestSuds(SpyneClientTestBase, unittest.TestCase):
         return {
             "test_sub": {
                 "test_subsub1": {
-                    "test_subsubsub1" : ["subsubsub1 value"]
+                    "test_subsubsub1": ["subsubsub1 value"]
                 },
                 "test_subsub2": ["subsub2 value 1", "subsub2 value 2"],
                 "test_subsub3": [
@@ -276,12 +285,13 @@ class TestSuds(SpyneClientTestBase, unittest.TestCase):
         self.assertEqual(ret.other.dt, val.other.dt)
 
     def test_huge_number(self):
-        self.assertEquals(self.client.service.huge_number(), 2**int(1e5))
+        self.assertEquals(self.client.service.huge_number(), 2 ** int(1e5))
 
     def test_long_string(self):
-        self.assertEquals(self.client.service.long_string(), ('0123456789abcdef' * 16384))
+        self.assertEquals(self.client.service.long_string(),
+                                                   ('0123456789abcdef' * 16384))
 
-    def test_long_string(self):
+    def test_empty(self):
         self.client.service.test_empty()
 
     def test_echo_extension_class(self):
