@@ -17,6 +17,8 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 #
 
+from __future__ import print_function
+
 import pytz
 import datetime
 import unittest
@@ -26,11 +28,11 @@ from pprint import pprint
 from lxml import etree
 
 from base64 import b64encode
+from decimal import Decimal as D
 
-from spyne import Application
-from spyne import rpc
-from spyne import mrpc
-from spyne import ServiceBase
+from spyne import Application, rpc, mrpc, ServiceBase, ByteArray, Array, \
+    ComplexModel, SelfReference, XmlData, XmlAttribute, Unicode, DateTime, \
+    Float, Integer, String
 from spyne.const import xml_ns
 from spyne.error import ResourceNotFoundError
 from spyne.interface import Interface
@@ -38,17 +40,6 @@ from spyne.interface.wsdl import Wsdl11
 from spyne.protocol import ProtocolBase
 from spyne.protocol.soap import Soap11
 from spyne.server.null import NullServer
-from spyne.model import ByteArray
-from spyne.model import Array
-from spyne.model import ComplexModel
-from spyne.model import SelfReference
-from spyne.model import XmlData
-from spyne.model import XmlAttribute
-from spyne.model import Unicode
-from spyne.model import DateTime
-from spyne.model import Float
-from spyne.model import Integer
-from spyne.model import String
 
 from spyne.protocol.dictdoc import SimpleDictDocument
 from spyne.protocol.xml import XmlDocument
@@ -57,6 +48,7 @@ from spyne.test import FakeApp
 
 ns_test = 'test_namespace'
 
+
 class Address(ComplexModel):
     street = String
     city = String
@@ -67,6 +59,7 @@ class Address(ComplexModel):
 
 Address.resolve_namespace(Address, __name__)
 
+
 class Person(ComplexModel):
     name = String
     birthdate = DateTime
@@ -76,6 +69,7 @@ class Person(ComplexModel):
 
 Person.resolve_namespace(Person, __name__)
 
+
 class Employee(Person):
     employee_id = Integer
     salary = Float
@@ -88,16 +82,19 @@ class Level2(ComplexModel):
 
 Level2.resolve_namespace(Level2, __name__)
 
+
 class Level3(ComplexModel):
     arg1 = Integer
 
 Level3.resolve_namespace(Level3, __name__)
 
+
 class Level4(ComplexModel):
     arg1 = String
 
 Level4.resolve_namespace(Level4, __name__)
 
+
 class Level1(ComplexModel):
     level2 = Level2
     level3 = Array(Level3)
@@ -105,6 +102,7 @@ class Level1(ComplexModel):
 
 Level1.resolve_namespace(Level1, __name__)
 
+
 class TestComplexModel(unittest.TestCase):
     def test_simple_class(self):
         a = Address()
@@ -189,15 +187,15 @@ class TestComplexModel(unittest.TestCase):
                 a.addresses.append(addr)
             peeps.append(a)
 
-        type = Array(Person)
-        type.resolve_namespace(type, __name__)
+        arr = Array(Person)
+        arr.resolve_namespace(arr, __name__)
         element = etree.Element('test')
-        XmlDocument().to_parent(None, type, peeps, element, ns_test)
+        XmlDocument().to_parent(None, arr, peeps, element, ns_test)
         element = element[0]
 
         self.assertEquals(4, len(element.getchildren()))
 
-        peeps2 = XmlDocument().from_element(None, type, element)
+        peeps2 = XmlDocument().from_element(None, arr, element)
         for peep in peeps2:
             self.assertEquals(27, peep.age)
             self.assertEquals(25, len(peep.addresses))
@@ -350,7 +348,7 @@ class TestXmlAttribute(unittest.TestCase):
         Soap11().to_parent(None, PacketAttribute, gg, element, gg.get_namespace())
 
         element = element[0]
-        #print etree.tostring(element, pretty_print=True)
+        print(etree.tostring(element, pretty_print=True))
         self.assertEquals(element.attrib['Data'], b64string)
 
         s1 = Soap11().from_element(None, PacketAttribute, element)
@@ -483,27 +481,34 @@ class TestFlatDict(unittest.TestCase):
         d = SimpleDictDocument().object_to_simple_dict(CCM, val)
         pprint(d)
 
-        assert d['c[0].i'] == [0,1]
-        assert d['c[1].i'] == [0,1,2]
+        assert d['c[0].i'] == [0, 1]
+        assert d['c[1].i'] == [0, 1, 2]
 
         assert len(d) == 2
 
+    def test_array_nonwrapped(self):
+        i = Array(Integer, wrapped=False)
+
+        assert issubclass(i, Integer), i
+        assert i.Attributes.max_occurs == D('infinity')
+
 
 class TestSelfRefence(unittest.TestCase):
     def test_canonical_case(self):
         class TestSelfReference(ComplexModel):
             self_reference = SelfReference
 
-        assert (TestSelfReference._type_info['self_reference'] is TestSelfReference)
+        c = TestSelfReference._type_info['self_reference']
+        c = c.__orig__ or c
+
+        assert c is TestSelfReference
 
         class SoapService(ServiceBase):
             @rpc(_returns=TestSelfReference)
             def view_categories(ctx):
                 pass
 
-        Application([SoapService], 'service.soap',
-                            in_protocol=ProtocolBase(),
-                            out_protocol=ProtocolBase())
+        Application([SoapService], 'service.soap')
 
     def test_self_referential_array_workaround(self):
         from spyne.util.dictdoc import get_object_as_dict
@@ -651,7 +656,7 @@ class TestMemberRpc(unittest.TestCase):
 
         app = Application([SomeService], 'some_ns')
 
-        mmm = __name__ + '.SomeComplexModel.member_method'
+        mmm = __name__ + '.SomeComplexModel.SomeComplexModel.member_method'
         assert mmm in app.interface.method_id_map
 
     def test_interface_mult(self):
@@ -675,7 +680,7 @@ class TestMemberRpc(unittest.TestCase):
 
         app = Application([SomeService], 'some_ns')
 
-        mmm = __name__ + '.SomeComplexModel.member_method'
+        mmm = __name__ + '.SomeComplexModel.SomeComplexModel.member_method'
         assert mmm in app.interface.method_id_map
 
     def test_remote_call_error(self):
@@ -741,7 +746,7 @@ class TestMemberRpc(unittest.TestCase):
         null = NullServer(Application([SomeService], tns='some_tns'))
 
         v = SomeComplexModel(i=5)
-        assert null.service['SomeComplexModel.echo'](v) is v
+        assert null.service['SomeComplexModel.SomeComplexModel.echo'](v) is v
 
     def test_order(self):
         class CM(ComplexModel):
@@ -752,5 +757,246 @@ class TestMemberRpc(unittest.TestCase):
 
         assert CM._type_info.keys() == ['c', 'a']
 
+
+class TestDoc(unittest.TestCase):
+    def test_parent_doc(self):
+        class SomeComplexModel(ComplexModel):
+            """Some docstring"""
+            some_field = Unicode
+            class Annotations(ComplexModel.Annotations):
+                __use_parent_doc__ = True
+        assert "Some docstring" == SomeComplexModel.get_documentation()
+
+    def test_annotation(self):
+        class SomeComplexModel(ComplexModel):
+            """Some docstring"""
+            class Annotations(ComplexModel.Annotations):
+                doc = "Some annotations"
+
+            some_field = Unicode
+        assert "Some annotations" == SomeComplexModel.get_documentation()
+
+    def test_no_parent_doc(self):
+        class SomeComplexModel(ComplexModel):
+            """Some docstring"""
+            class Annotations(ComplexModel.Annotations):
+                __use_parent_doc__ = False
+
+            some_field = Unicode
+        assert "" == SomeComplexModel.get_documentation()
+
+    def test_parent_doc_customize(self):
+        """Check that we keep the documentation when we use customize"""
+        class SomeComplexModel(ComplexModel):
+            """Some docstring"""
+            some_field = Unicode
+            class Annotations(ComplexModel.Annotations):
+                __use_parent_doc__ = True
+        assert "Some docstring" == SomeComplexModel.customize().get_documentation()
+
+
+class TestCustomize(unittest.TestCase):
+    def test_base_class(self):
+        class A(ComplexModel):
+            s = Unicode
+
+        assert A.customize().__extends__ is None
+
+        class B(A):
+            i = Integer
+
+        assert B.__orig__ is None
+
+        B2 = B.customize()
+
+        assert B2.__orig__ is B
+        assert B2.__extends__ is A
+
+        B3 = B2.customize()
+
+        assert B3.__orig__ is B
+        assert B3.__extends__ is A
+
+    def test_noop(self):
+        class A(ComplexModel):
+            s = Unicode
+
+        assert A.get_flat_type_info(A)['s'].Attributes.max_len == D('inf')
+
+    def test_cust_simple(self):
+        # simple types are different from complex ones for __extends__ handling.
+        # simple types set __orig__ and __extends__ on customization.
+        # complex types set __orig__ but not extend.
+        # for complex types, __extend__ is set only on explicit inheritance
+
+        t = Unicode(max_len=10)
+
+        assert t.Attributes.max_len == 10
+        assert t.__extends__ is Unicode
+        assert t.__orig__ is Unicode
+
+    def test_cust_simple_again(self):
+        t = Unicode(max_len=10)
+        t2 = t(min_len=5)
+
+        assert t2.Attributes.max_len == 10
+        assert t2.Attributes.min_len == 5
+        assert t2.__extends__ is t
+        assert t2.__orig__ is Unicode
+
+    def test_cust_complex(self):
+        class A(ComplexModel):
+            s = Unicode
+
+        A2 = A.customize(
+            child_attrs=dict(
+                s=dict(
+                    max_len=10
+                )
+            )
+        )
+
+        assert A2.get_flat_type_info(A2)['s'].Attributes.max_len == 10
+
+    def test_cust_base_class(self):
+        class A(ComplexModel):
+            s = Unicode
+
+        class B(A):
+            i = Integer
+
+        B2 = B.customize(
+            child_attrs=dict(
+                s=dict(
+                    max_len=10,
+                ),
+            ),
+        )
+
+        assert B2.get_flat_type_info(B2)['s'].Attributes.max_len == 10
+
+    def test_cust_again_base_class(self):
+        class A(ComplexModel):
+            s = Unicode
+
+        A2 = A.customize()
+        try:
+            class B(A2):
+                i = Integer
+        except AssertionError:
+            pass
+        else:
+            raise Exception("must fail")
+
+    def test_cust_array(self):
+        A = Array(Unicode)
+
+        assert A.__orig__ is Array
+        assert A.__extends__ is None
+        assert issubclass(A, Array)
+
+    def test_cust_array_again(self):
+        A = Array(Unicode)
+
+        A = A.customize(foo='bar')
+
+        assert A.Attributes.foo == 'bar'
+        assert A.__orig__ is Array
+        assert A.__extends__ is None
+        assert issubclass(A, Array)
+
+    def test_cust_array_serializer(self):
+        A = Array(Unicode)
+
+        A = A.customize(
+            serializer_attrs=dict(
+                max_len=10,
+            ),
+        )
+
+        serializer, = A._type_info.values()
+
+        assert serializer.Attributes.max_len == 10
+        assert serializer.__orig__ is Unicode
+        assert issubclass(serializer, Unicode)
+
+    def test_cust_sub_array(self):
+        """vanilla class is passed as base"""
+        class A(ComplexModel):
+            s = Array(Unicode)
+
+        d = dict(
+            child_attrs=dict(
+                s=dict(
+                    serializer_attrs=dict(
+                        max_len=10,
+                    ),
+                ),
+            ),
+        )
+
+        A2 = A.customize(**d)
+
+        ser, = A2._type_info['s']._type_info.values()
+        assert ser.Attributes.max_len == 10
+
+        class B(A):
+            i = Integer
+
+        B2 = B.customize(**d)
+
+        b2_fti = B2.get_flat_type_info(B2)
+        ser, = b2_fti['s']._type_info.values()
+
+        assert ser.Attributes.max_len == 10
+
+    def test_cust_side_effect(self):
+        class A(ComplexModel):
+            s = Unicode
+            i = Integer
+
+        class B(A):
+            d = DateTime
+
+        B2 = B.customize(child_attrs=dict(s=dict(max_len=10)))
+        assert B2.get_flat_type_info(B2)['s'].Attributes.max_len == 10
+
+        B3 = B2.customize(child_attrs=dict(d=dict(format="%y")))
+        assert B3.get_flat_type_info(B3)['s'].Attributes.max_len == 10
+
+
+    def test_complex_type_name_clashes(self):
+        class TestComplexModel(ComplexModel):
+            attr1 = String
+
+        TestComplexModel1 = TestComplexModel
+
+        class TestComplexModel(ComplexModel):
+            attr2 = String
+
+        TestComplexModel2 = TestComplexModel
+
+        class TestService(ServiceBase):
+            @rpc(TestComplexModel1)
+            def test1(ctx, obj):
+                pass
+
+            @rpc(TestComplexModel2)
+            def test2(ctx, obj):
+                pass
+
+        try:
+            Application([TestService], 'tns')
+        except Exception as e:
+            print(e)
+        else:
+            raise Exception("must fail with: "
+                "ValueError: classes "
+                    "<class 'spyne.test.model.test_complex.TestComplexModel'> "
+                    "and "
+                    "<class 'spyne.test.model.test_complex.TestComplexModel'> "
+                    "have conflicting names.")
+
+
 if __name__ == '__main__':
     unittest.main()
