@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+# coding=utf-8
 #
 # spyne - Copyright (C) Spyne contributors.
 #
@@ -18,10 +19,11 @@
 #
 
 import re
+import uuid
 import datetime
 import unittest
+
 import pytz
-import uuid
 
 from datetime import timedelta
 
@@ -29,29 +31,22 @@ from lxml import etree
 
 from spyne.util import total_seconds
 from spyne.const import xml_ns as ns
-from spyne.model import Null, AnyDict, Uuid
-from spyne.model.complex import Array
-from spyne.model.complex import ComplexModel
-from spyne.model.primitive import Date
-from spyne.model.primitive import Time
-from spyne.model.primitive import Boolean
-from spyne.model.primitive import DateTime
-from spyne.model.primitive import Duration
-from spyne.model.primitive import Float
-from spyne.model.primitive import Integer
-from spyne.model.primitive import UnsignedInteger
-from spyne.model.primitive import Unicode
-from spyne.model.primitive import String
-from spyne.model.primitive import Decimal
+
+from spyne import Null, AnyDict, Uuid, Array, ComplexModel, Date, Time, \
+    Boolean, DateTime, Duration, Float, Integer, UnsignedInteger, Unicode, \
+    String, Decimal
+from spyne.model import ModelBase
 
 from spyne.protocol import ProtocolBase
 from spyne.protocol.xml import XmlDocument
 
 ns_test = 'test_namespace'
 
-from spyne.model import ModelBase
 
 class TestPrimitive(unittest.TestCase):
+    def test_getitem_cust(self):
+        assert Unicode[dict(max_len=2)].Attributes.max_len
+
     def test_nillable_quirks(self):
         assert ModelBase.Attributes.nillable == True
         class Attributes(ModelBase.Attributes):
@@ -154,6 +149,18 @@ class TestPrimitive(unittest.TestCase):
         dt = XmlDocument().from_element(None, DateTime(format=format), element)
         assert n == dt
 
+    def test_datetime_unicode_format(self):
+        n = datetime.datetime.now().replace(microsecond=0)
+        format = u"%Y %m %d\u00a0%H %M %S"
+
+        element = etree.Element('test')
+        XmlDocument().to_parent(None, DateTime(format=format), n, element, ns_test)
+        element = element[0]
+
+        assert element.text == n.strftime(format.encode('utf8')).decode('utf8')
+        dt = XmlDocument().from_element(None, DateTime(format=format), element)
+        assert n == dt
+
     def test_date_format(self):
         t = datetime.date.today()
         format = "%Y %m %d"
@@ -198,6 +205,33 @@ class TestPrimitive(unittest.TestCase):
         dt = ProtocolBase().from_string(Time, ret)
         self.assertEquals(n, dt)
 
+    def test_time_usec(self):
+        # python's datetime and time only accept ints between [0, 1e6[
+        # if the incoming data is 999999.8 microseconds, rounding it up means
+        # adding 1 second to time. For many reasons, we want to avoid that. (see
+        # http://bugs.python.org/issue1487389) That's why 999999.8 usec is
+        # rounded to 999999.
+
+        # rounding 0.1 µsec down
+        t = ProtocolBase().from_string(Time, "12:12:12.0000001")
+        self.assertEquals(datetime.time(12, 12, 12), t)
+
+        # rounding 0.5 µsec up
+        t = ProtocolBase().from_string(Time, "12:12:12.0000005")
+        self.assertEquals(datetime.time(12, 12, 12, 1), t)
+
+        # rounding 999998.8 µsec up
+        t = ProtocolBase().from_string(Time, "12:12:12.9999988")
+        self.assertEquals(datetime.time(12, 12, 12, 999999), t)
+
+        # rounding 999999.1 µsec down
+        t = ProtocolBase().from_string(Time, "12:12:12.9999991")
+        self.assertEquals(datetime.time(12, 12, 12, 999999), t)
+
+        # rounding 999999.8 µsec down, not up.
+        t = ProtocolBase().from_string(Time, "12:12:12.9999998")
+        self.assertEquals(datetime.time(12, 12, 12, 999999), t)
+
     def test_date(self):
         n = datetime.date(2011,12,13)
 
@@ -251,7 +285,7 @@ class TestPrimitive(unittest.TestCase):
         ProtocolBase().from_string(UnsignedInteger, "-1") # This is not supposed to fail.
 
         try:
-            UnsignedInteger.validate_native(-1) # This is supposed to fail.
+            UnsignedInteger.validate_native(-1)  # This is supposed to fail.
         except:
             pass
         else:
@@ -324,6 +358,14 @@ class TestPrimitive(unittest.TestCase):
         assert Unicode(pattern='a').Attributes.pattern == 'a'
         assert Unicode(pattern='a')(5).Attributes.pattern == 'a'
 
+    def test_unicode_upattern(self):
+        patt = r'[\w .-]+'
+        attr = Unicode(unicode_pattern=patt).Attributes
+        assert attr.pattern == patt
+        assert attr._pattern_re.flags & re.UNICODE
+        assert attr._pattern_re.match(u"Ğ Ğ ç .-")
+        assert attr._pattern_re.match(u"\t") is None
+
     def test_unicode_nullable_mult_cust_false(self):
         assert Unicode(nullable=False).Attributes.nullable == False
         assert Unicode(nullable=False)(5).Attributes.nullable == False
@@ -343,7 +385,7 @@ class TestPrimitive(unittest.TestCase):
         self.assertEquals(None, value)
 
     def test_point(self):
-        from spyne.model.primitive import _get_point_pattern
+        from spyne.model.primitive.spatial import _get_point_pattern
 
         a=re.compile(_get_point_pattern(2))
         assert a.match('POINT (10 40)') is not None
@@ -353,7 +395,7 @@ class TestPrimitive(unittest.TestCase):
         assert a.match('POINT(1.310e4 40)') is not None
 
     def test_multipoint(self):
-        from spyne.model.primitive import _get_multipoint_pattern
+        from spyne.model.primitive.spatial import _get_multipoint_pattern
 
         a=re.compile(_get_multipoint_pattern(2))
         assert a.match('MULTIPOINT (10 40, 40 30, 20 20, 30 10)') is not None
@@ -361,26 +403,26 @@ class TestPrimitive(unittest.TestCase):
         #assert a.match('MULTIPOINT ((10 40), (40 30), (20 20), (30 10))') is not None
 
     def test_linestring(self):
-        from spyne.model.primitive import _get_linestring_pattern
+        from spyne.model.primitive.spatial import _get_linestring_pattern
 
         a=re.compile(_get_linestring_pattern(2))
         assert a.match('LINESTRING (30 10, 10 30, 40 40)') is not None
 
     def test_multilinestring(self):
-        from spyne.model.primitive import _get_multilinestring_pattern
+        from spyne.model.primitive.spatial import _get_multilinestring_pattern
 
         a=re.compile(_get_multilinestring_pattern(2))
         assert a.match('''MULTILINESTRING ((10 10, 20 20, 10 40),
                                 (40 40, 30 30, 40 20, 30 10))''') is not None
 
     def test_polygon(self):
-        from spyne.model.primitive import _get_polygon_pattern
+        from spyne.model.primitive.spatial import _get_polygon_pattern
 
         a=re.compile(_get_polygon_pattern(2))
         assert a.match('POLYGON ((30 10, 10 20, 20 40, 40 40, 30 10))') is not None
 
     def test_multipolygon(self):
-        from spyne.model.primitive import _get_multipolygon_pattern
+        from spyne.model.primitive.spatial import _get_multipolygon_pattern
 
         a=re.compile(_get_multipolygon_pattern(2))
         assert a.match('''MULTIPOLYGON (((30 20, 10 40, 45 40, 30 20)),
@@ -485,6 +527,12 @@ class TestPrimitive(unittest.TestCase):
         assert ProtocolBase().from_string(Uuid(serialize_as='int'),
                 24197857161011715162171839636988778104) == value
 
+    def test_uuid_validate(self):
+        assert Uuid.validate_string(Uuid,
+                          '12345678-1234-5678-1234-567812345678')
+        assert Uuid.validate_native(Uuid,
+                uuid.UUID('12345678-1234-5678-1234-567812345678'))
+
     def test_datetime_serialize_as(self):
         i = 1234567890123456
         v = datetime.datetime.fromtimestamp(i / 1e6)
@@ -512,13 +560,35 @@ class TestPrimitive(unittest.TestCase):
 
         assert ProtocolBase().from_string(
                     DateTime(serialize_as='msec'), i//1e3) == \
-                                     datetime.datetime.fromtimestamp(i/1e3//1000)
+                                    datetime.datetime.fromtimestamp(i/1e3//1000)
         assert ProtocolBase().from_string(
                     DateTime(serialize_as='msec_float'), i/1e3) == v
 
         assert ProtocolBase().from_string(
                     DateTime(serialize_as='usec'), i) == v
 
+    def test_datetime_usec(self):
+        # see the comments on time test for why the rounding here is weird
+
+        # rounding 0.1 µsec down
+        dt = ProtocolBase().from_string(DateTime, "2015-01-01 12:12:12.0000001")
+        self.assertEquals(datetime.datetime(2015, 1, 1, 12, 12, 12), dt)
+
+        # rounding 0.5 µsec up
+        dt = ProtocolBase().from_string(DateTime, "2015-01-01 12:12:12.0000005")
+        self.assertEquals(datetime.datetime(2015, 1, 1, 12, 12, 12, 1), dt)
+
+        # rounding 999998.8 µsec up
+        dt = ProtocolBase().from_string(DateTime, "2015-01-01 12:12:12.9999988")
+        self.assertEquals(datetime.datetime(2015, 1, 1, 12, 12, 12, 999999), dt)
+
+        # rounding 999999.1 µsec down
+        dt = ProtocolBase().from_string(DateTime, "2015-01-01 12:12:12.9999991")
+        self.assertEquals(datetime.datetime(2015, 1, 1, 12, 12, 12, 999999), dt)
+
+        # rounding 999999.8 µsec down, not up.
+        dt = ProtocolBase().from_string(DateTime, "2015-01-01 12:12:12.9999998")
+        self.assertEquals(datetime.datetime(2015, 1, 1, 12, 12, 12, 999999), dt)
 
 ### Duration Data Type
 ## http://www.w3schools.com/schema/schema_dtypes_date.asp
@@ -676,5 +746,6 @@ class TestDurationPrimitive(unittest.TestCase):
         self.assertEquals(dur, ProtocolBase().from_string(Duration,
                                ProtocolBase().to_string(Duration, dur)))
 
+
 if __name__ == '__main__':
     unittest.main()
