@@ -81,16 +81,21 @@ def TDry(serializer, _DictDocumentChild, dumps_kwargs=None):
         dumps_kwargs = {}
 
     def _dry_me(services, d, ignore_wrappers=False, complex_as=dict,
-                         just_ctx=False, just_in_object=False, validator=None):
+                         just_ctx=False, just_in_object=False, validator=None,
+                         polymorphic=False):
 
         app = Application(services, 'tns',
-                in_protocol=_DictDocumentChild(validator=validator),
+                in_protocol=_DictDocumentChild(
+                    ignore_wrappers=ignore_wrappers, complex_as=complex_as,
+                    polymorphic=polymorphic, validator=validator,
+                ),
                 out_protocol=_DictDocumentChild(
-                        ignore_wrappers=ignore_wrappers, complex_as=complex_as),
+                     ignore_wrappers=ignore_wrappers, complex_as=complex_as,
+                                                       polymorphic=polymorphic),
             )
 
         server = ServerBase(app)
-        initial_ctx = MethodContext(server)
+        initial_ctx = MethodContext(server, MethodContext.SERVER)
         initial_ctx.in_string = [serializer.dumps(d, **dumps_kwargs)]
 
         ctx, = server.generate_contexts(initial_ctx)
@@ -121,7 +126,7 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
 
             ctx = _dry_me([SomeService], {"some_call":[]})
 
-            s = ''.join(ctx.out_string)
+            s = b''.join(ctx.out_string)
             d = serializer.dumps({"some_callResponse": {"some_callResult":
                     {"SomeComplexModel": {"i": 5, "s": "5x"}}}}, **dumps_kwargs)
             print(s)
@@ -144,7 +149,7 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
                     return CCM(c=ccm.c, i=ccm.i, s=ccm.s)
 
             ctx = _dry_me([SomeService], {"some_call":
-                    {"ccm": {"c":{"i":3, "s": "3x"}, "i":4, "s": "4x"}}
+                    {"ccm": {"CCM":{"c":{"CM":{"i":3, "s": "3x"}}, "i":4, "s": "4x"}}}
                 })
 
             ret = serializer.loads(''.join(ctx.out_string))
@@ -213,12 +218,12 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
                     return ecm
 
             ctx = _dry_me([SomeService], {
-                "some_call": {"ecm": [{
-                        "c": {"i":3, "s": "3x"},
+                "some_call": {"ecm": [{"ECM": {
+                        "c": {"CM":{"i":3, "s": "3x"}},
                         "i":4,
                         "s": "4x",
                         "d": "2011-12-13T14:15:16Z"
-                    }]
+                    }}]
                 }})
 
             print(ctx.in_object)
@@ -234,7 +239,6 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             assert ret["some_callResponse"]['some_callResult'][0]["ECM"]["s"] == "4x"
             assert ret["some_callResponse"]['some_callResult'][0]["ECM"]["d"] == "2011-12-13T14:15:16+00:00"
 
-
         def test_invalid_request(self):
             class SomeService(ServiceBase):
                 @srpc(Integer, String, DateTime)
@@ -316,7 +320,7 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ret = serializer.loads(''.join(ctx.out_string))
 
             assert ret == {"some_callResponse": {'some_callResult':
-                                                    {'SomeObject':{'s': None}}}}
+                                                   {'SomeObject': {'s': None}}}}
 
         def test_any_xml(self):
             d = lxml.etree.tostring(E('{ns1}x', E('{ns2}Y', "some data")))
@@ -332,8 +336,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -351,14 +355,14 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
 
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
-            s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            s = b''.join(ctx.out_string)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
+
             print(s)
             print(d)
             assert s == d
 
-
         def test_any_dict(self):
             d = {'helo': 213, 'data': {'nested': [12, 0.3]}}
 
@@ -373,8 +377,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -393,8 +397,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -413,8 +417,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -433,8 +437,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -453,8 +457,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -495,8 +499,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -515,8 +519,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -537,8 +541,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
 
             print(s)
             print(d)
@@ -560,8 +564,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -581,8 +585,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -602,8 +606,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -623,8 +627,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]}, validator='soft')
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -643,15 +647,15 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
 
             ctx = _dry_me([SomeService], {"some_call":[d]}, validator='soft')
 
-            s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            s = b''.join(ctx.out_string)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
 
         def test_duration(self):
-            d = ProtocolBase().to_string(Duration, timedelta(0, 45))
+            d = ProtocolBase().to_unicode(Duration, timedelta(0, 45))
 
             class SomeService(ServiceBase):
                 @srpc(Duration, _returns=Duration)
@@ -664,8 +668,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -684,8 +688,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -704,8 +708,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -724,8 +728,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -744,8 +748,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -764,8 +768,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -784,8 +788,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -804,8 +808,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -824,8 +828,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -844,8 +848,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -864,8 +868,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -884,8 +888,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -904,8 +908,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -924,8 +928,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -946,8 +950,8 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             ctx = _dry_me([SomeService], {"some_call":[d]})
 
             s = ''.join(ctx.out_string)
-            d = serializer.dumps({"some_callResponse": {"some_callResult":
-                                                        d}}, **dumps_kwargs)
+            d = serializer.dumps({"some_callResponse": {"some_callResult": d}},
+                                                                 **dumps_kwargs)
             print(s)
             print(d)
             assert s == d
@@ -1048,10 +1052,10 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
                     assert p.name == v.name
                     return p
 
-            d = get_object_as_dict(v, File)
-            assert d['name'] == v.name
-            assert d['type'] == v.type
-            assert d['data'] == v.data
+            d = get_object_as_dict(v, File, ignore_wrappers=False)
+            assert d[File.get_type_name()]['name'] == v.name
+            assert d[File.get_type_name()]['type'] == v.type
+            assert d[File.get_type_name()]['data'] == v.data
 
             ctx = _dry_me([SomeService], {"some_call": {'p': d}})
             s = ''.join(ctx.out_string)
@@ -1209,4 +1213,95 @@ def TDictDocumentTest(serializer, _DictDocumentChild, dumps_kwargs=None):
             print(serializer.loads(d))
             assert s == d
 
+        def test_polymorphic_deserialization(self):
+            class P(ComplexModel):
+                sig = Unicode
+
+            class C(P):
+                foo = Unicode
+
+            class D(P):
+                bar = Integer
+
+            class SomeService(ServiceBase):
+                @rpc(P, _returns=Unicode)
+                def typeof(ctx, p):
+                    return type(p).__name__
+
+            ctx = _dry_me([SomeService],
+                            {"typeof": [{'C':{'sig':'a', 'foo': 'f'}}]},
+                                                               polymorphic=True)
+
+            s = ''.join(ctx.out_string)
+            d = serializer.dumps({"typeofResponse": {"typeofResult":
+                'C'
+            }}, **dumps_kwargs)
+
+            print(serializer.loads(s))
+            print(serializer.loads(d))
+            assert s == d
+
+            ctx = _dry_me([SomeService],
+                                  {"typeof": [{'D':{'sig':'b', 'bar': 5}}]},
+                                                               polymorphic=True)
+
+            s = ''.join(ctx.out_string)
+            d = serializer.dumps({"typeofResponse": {"typeofResult":
+                'D'
+            }}, **dumps_kwargs)
+
+            print(serializer.loads(s))
+            print(serializer.loads(d))
+            assert s == d
+
+        def test_default(self):
+            class SomeComplexModel(ComplexModel):
+                _type_info = [
+                    ('a', Unicode),
+                    ('b', Unicode(default='default')),
+                ]
+
+            class SomeService(ServiceBase):
+                @srpc(SomeComplexModel)
+                def some_method(s):
+                    pass
+
+            ctx = _dry_me([SomeService],
+                            {"some_method": [{"s": {"a": "x", "b": None}}]},
+                                                               polymorphic=True)
+
+            assert ctx.in_object.s.b == None
+            assert ctx.in_error is None
+
+            ctx = _dry_me([SomeService], {"some_method": {"s": {"a": "x"}}},
+                                                               polymorphic=True)
+
+            assert ctx.in_object.s.b == 'default'
+            assert ctx.in_error is None
+
+        def test_nillable_default(self):
+            class SomeComplexModel(ComplexModel):
+                _type_info = [
+                    ('a', Unicode),
+                    ('b', Unicode(min_occurs=1, default='default', nillable=True)),
+                ]
+
+            class SomeService(ServiceBase):
+                @srpc(SomeComplexModel)
+                def some_method(s):
+                    pass
+
+            ctx = _dry_me([SomeService],
+                            {"some_method": [{"s": {"a": "x", "b": None}}]},
+                                           polymorphic=True, validator='soft')
+
+            assert ctx.in_object.s.b == None
+            assert ctx.in_error is None
+
+            ctx = _dry_me([SomeService], {"some_method": {"s": {"a": "x"}}},
+                                                               polymorphic=True)
+
+            assert ctx.in_object.s.b == 'default'
+            assert ctx.in_error is None
+
     return Test
