@@ -23,15 +23,8 @@ logging.basicConfig(level=logging.DEBUG)
 
 import unittest
 
-from spyne.util import six
-
-if six.PY3:
-    from io import StringIO
-    from http.cookies import SimpleCookie
-
-else:
-    from StringIO import StringIO
-    from Cookie import SimpleCookie
+from spyne.util.six import StringIO
+from spyne.util.six.moves.http_cookies import SimpleCookie
 
 from datetime import datetime
 from wsgiref.validate import validator as wsgiref_validator
@@ -279,14 +272,13 @@ class TestValidation(unittest.TestCase):
         ret = _test([SomeService], 'p[2].i=2&p[0].i=0&', strict_arrays=False)
         assert ret.out_object[0] == '[C(i=0), C(i=2)]'
 
-
     def test_validation_nested_array(self):
         class CC(ComplexModel):
             d = DateTime
 
         class C(ComplexModel):
-            i=Integer(min_occurs=1)
-            cc=Array(CC)
+            i = Integer(min_occurs=1)
+            cc = Array(CC)
 
         class SomeService(ServiceBase):
             @srpc(Array(C))
@@ -428,7 +420,7 @@ class Test(unittest.TestCase):
 
         assert ctx.out_string[0] == "CCM(i=1, c=CM(i=3, s='cs'), s='s')"
 
-    def test_multiple(self):
+    def test_simple_array(self):
         class SomeService(ServiceBase):
             @srpc(String(max_occurs='unbounded'), _returns=String)
             def some_call(s):
@@ -437,6 +429,60 @@ class Test(unittest.TestCase):
         ctx = _test([SomeService], '&s=1&s=2')
         assert ''.join(ctx.out_string) == '1\n2'
 
+    def test_complex_array(self):
+        class CM(ComplexModel):
+            _type_info = [
+                ("i", Integer),
+                ("s", String),
+            ]
+
+        class SomeService(ServiceBase):
+            @srpc(Array(CM), _returns=String)
+            def some_call(cs):
+                return '\n'.join([repr(c) for c in cs])
+
+        ctx = _test([SomeService],
+             'cs[0].i=1&cs[0].s=x'
+            '&cs[1].i=2&cs[1].s=y'
+            '&cs[2].i=3&cs[2].s=z')
+
+        assert ''.join(ctx.out_string) == \
+           "CM(i=1, s='x')\n" \
+           "CM(i=2, s='y')\n" \
+           "CM(i=3, s='z')"
+
+    def test_complex_array_empty(self):
+        class CM(ComplexModel):
+            _type_info = [
+                ("i", Integer),
+                ("s", String),
+            ]
+
+        class SomeService(ServiceBase):
+            @srpc(Array(CM), _returns=String)
+            def some_call(cs):
+                return repr(cs)
+
+        ctx = _test([SomeService], 'cs=empty')
+
+        assert ''.join(ctx.out_string) == '[]'
+
+    def test_complex_object_empty(self):
+        class CM(ComplexModel):
+            _type_info = [
+                ("i", Integer),
+                ("s", String),
+            ]
+
+        class SomeService(ServiceBase):
+            @srpc(CM, _returns=String)
+            def some_call(c):
+                return repr(c)
+
+        ctx = _test([SomeService], 'c=empty')
+
+        assert ''.join(ctx.out_string) == 'CM()'
+
     def test_nested_flatten(self):
         class CM(ComplexModel):
             _type_info = [
@@ -559,6 +605,33 @@ class Test(unittest.TestCase):
         s = ''.join(list(ctx.out_string))
         assert s == "[CCM(i=1, c=['a', 'b'], s='s')]"
 
+    def test_default(self):
+        class CM(ComplexModel):
+            _type_info = [
+                ("i", Integer),
+                ("s", String(default='default')),
+            ]
+
+        class SomeService(ServiceBase):
+            @srpc(CM, _returns=String)
+            def some_call(cm):
+                return repr(cm)
+
+        # s is missing
+        ctx = _test([SomeService], 'cm.i=1')
+        s = ''.join(ctx.out_string)
+        assert s == "CM(i=1, s='default')"
+
+        # s is None
+        ctx = _test([SomeService], 'cm.i=1&cm.s')
+        s = ''.join(ctx.out_string)
+        assert s == "CM(i=1)"
+
+        # s is empty
+        ctx = _test([SomeService], 'cm.i=1&cm.s=')
+        s = ''.join(ctx.out_string)
+        assert s == "CM(i=1, s='')"
+
     def test_nested_flatten_with_primitive_array(self):
         class CCM(ComplexModel):
             _type_info = [
