@@ -97,6 +97,7 @@ class TestSpyneJsonRpc1(unittest.TestCase):
         assert ctx.out_document == {"ver": 1, "fault": {
                         'faultcode': 'Server', 'faultstring': 'Internal Error'}}
 
+
 class TestJsonDocument(unittest.TestCase):
     def test_out_kwargs(self):
         class SomeService(ServiceBase):
@@ -130,7 +131,7 @@ class TestJsonDocument(unittest.TestCase):
 
         server = ServerBase(app)
 
-        initial_ctx = MethodContext(server)
+        initial_ctx = MethodContext(server, MethodContext.SERVER)
         initial_ctx.in_string = ['{']
         ctx, = server.generate_contexts(initial_ctx)
         assert ctx.in_error.faultcode == 'Client.JsonDecodeError'
@@ -179,5 +180,6 @@ class TestJsonP(unittest.TestCase):
         print(''.join(server.service.yay()))
         # assert false
 
+
 if __name__ == '__main__':
     unittest.main()
