@@ -26,34 +26,24 @@ logging.basicConfig(level=logging.DEBUG)
 
 import unittest
 
-from spyne.util import six
+from spyne.util.six import BytesIO
 
 from lxml import etree
 
-if six.PY3:
-    from io import StringIO
-else:
-    from StringIO import StringIO
-
 from spyne.const import RESPONSE_SUFFIX
 from spyne.model.primitive import NATIVE_MAP
 
+from spyne.service import ServiceBase
+from spyne.decorator import rpc, srpc
 from spyne.application import Application
 from spyne.auxproc.sync import SyncAuxProc
 from spyne.auxproc.thread import ThreadAuxProc
-from spyne.decorator import rpc
-from spyne.decorator import srpc
-from spyne.model import Array
-from spyne.model import SelfReference
-from spyne.model import Iterable
-from spyne.model import ComplexModel
-from spyne.model import String
-from spyne.model import Unicode
 from spyne.protocol.http import HttpRpc
 from spyne.protocol.soap import Soap11
 from spyne.server.null import NullServer
 from spyne.server.wsgi import WsgiApplication
-from spyne.service import ServiceBase
+from spyne.model import Array, SelfReference, Iterable, ComplexModel, String, \
+    Unicode
 
 
 Application.transport = 'test'
@@ -131,13 +121,16 @@ class TestMultipleMethods(unittest.TestCase):
             def call(s):
                 data.append(s)
 
-        app = Application([Service, AuxService], 'tns', in_protocol=HttpRpc(), out_protocol=HttpRpc())
+        app = Application([Service, AuxService], 'tns', in_protocol=HttpRpc(),
+                                                         out_protocol=HttpRpc())
         server = WsgiApplication(app)
         server({
             'QUERY_STRING': 's=hey',
             'PATH_INFO': '/call',
-            'REQUEST_METHOD': 'GET',
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'text/xml',
             'SERVER_NAME': 'localhost',
+            'wsgi.input': BytesIO(),
         }, start_response, "http://null")
 
         assert data == ['hey', 'hey']
@@ -164,8 +157,10 @@ class TestMultipleMethods(unittest.TestCase):
         server({
             'QUERY_STRING': 's=hey',
             'PATH_INFO': '/call',
-            'REQUEST_METHOD': 'GET',
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'text/xml',
             'SERVER_NAME': 'localhost',
+            'wsgi.input': BytesIO(),
         }, start_response, "http://null")
 
         import time
@@ -190,13 +185,17 @@ class TestMultipleMethods(unittest.TestCase):
                             "non-aux methods in a single service definition.'")
 
     def __run_service(self, service):
-        app = Application([service], 'tns', in_protocol=HttpRpc(), out_protocol=Soap11())
+        app = Application([service], 'tns', in_protocol=HttpRpc(),
+                                                          out_protocol=Soap11())
         server = WsgiApplication(app)
+
         return_string = ''.join(server({
             'QUERY_STRING': '',
             'PATH_INFO': '/some_call',
-            'REQUEST_METHOD': 'GET',
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'text/xml',
             'SERVER_NAME': 'localhost',
+            'wsgi.input': BytesIO(""),
         }, start_response, "http://null"))
 
         elt = etree.fromstring(''.join(return_string))
@@ -219,7 +218,7 @@ class TestMultipleMethods(unittest.TestCase):
                 ctx.out_header.Elem1 = 'Test1'
 
         elt, nsmap = self.__run_service(SomeService)
-        query = '/senv:Envelope/senv:Header/tns:RespHeader/tns:Elem1/text()'
+        query = '/soap11env:Envelope/soap11env:Header/tns:RespHeader/tns:Elem1/text()'
         assert elt.xpath(query, namespaces=nsmap)[0] == 'Test1'
 
         # test header in decorator
@@ -230,7 +229,7 @@ class TestMultipleMethods(unittest.TestCase):
                 ctx.out_header.Elem1 = 'Test1'
 
         elt, nsmap = self.__run_service(SomeService)
-        query = '/senv:Envelope/senv:Header/tns:RespHeader/tns:Elem1/text()'
+        query = '/soap11env:Envelope/soap11env:Header/tns:RespHeader/tns:Elem1/text()'
         assert elt.xpath(query, namespaces=nsmap)[0] == 'Test1'
 
         # test no header
@@ -241,7 +240,7 @@ class TestMultipleMethods(unittest.TestCase):
                 ctx.out_header.Elem1 = 'Test1'
 
         elt, nsmap = self.__run_service(SomeService)
-        query = '/senv:Envelope/senv:Header/tns:RespHeader/tns:Elem1/text()'
+        query = '/soap11env:Envelope/soap11env:Header/tns:RespHeader/tns:Elem1/text()'
         assert len(elt.xpath(query, namespaces=nsmap)) == 0
 
 
@@ -278,21 +277,22 @@ class TestBodyStyle(unittest.TestCase):
                                    out_protocol=Soap11(cleanup_namespaces=True))
 
         req = """
-        <senv:Envelope  xmlns:senv="http://schemas.xmlsoap.org/soap/envelope/"
+        <soap11env:Envelope  xmlns:soap11env="http://schemas.xmlsoap.org/soap/envelope/"
                         xmlns:tns="tns">
-            <senv:Body>
+            <soap11env:Body>
                 <tns:some_call>abc</tns:some_call>
-            </senv:Body>
-        </senv:Envelope>
+            </soap11env:Body>
+        </soap11env:Envelope>
         """
 
         server = WsgiApplication(app)
         resp = etree.fromstring(''.join(server({
             'QUERY_STRING': '',
             'PATH_INFO': '/call',
-            'REQUEST_METHOD': 'GET',
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'text/xml',
             'SERVER_NAME': 'localhost',
-            'wsgi.input': StringIO(req)
+            'wsgi.input': BytesIO(req),
         }, start_response, "http://null")))
 
         print(etree.tostring(resp, pretty_print=True))
@@ -309,24 +309,26 @@ class TestBodyStyle(unittest.TestCase):
                 return 'abc'
 
         app = Application([SomeService], 'tns', in_protocol=Soap11(),
-                                                out_protocol=Soap11(cleanup_namespaces=True))
+                                   out_protocol=Soap11(cleanup_namespaces=True))
 
         req = """
-        <senv:Envelope  xmlns:senv="http://schemas.xmlsoap.org/soap/envelope/"
-                        xmlns:tns="tns">
-            <senv:Body>
+        <soap11env:Envelope
+                    xmlns:soap11env="http://schemas.xmlsoap.org/soap/envelope/"
+                    xmlns:tns="tns">
+            <soap11env:Body>
                 <tns:some_call/>
-            </senv:Body>
-        </senv:Envelope>
+            </soap11env:Body>
+        </soap11env:Envelope>
         """
 
         server = WsgiApplication(app)
         resp = etree.fromstring(''.join(server({
             'QUERY_STRING': '',
             'PATH_INFO': '/call',
-            'REQUEST_METHOD': 'GET',
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'text/xml',
             'SERVER_NAME': 'localhost',
-            'wsgi.input': StringIO(req)
+            'wsgi.input': BytesIO(req)
         }, start_response, "http://null")))
 
         print(etree.tostring(resp, pretty_print=True))
@@ -335,6 +337,20 @@ class TestBodyStyle(unittest.TestCase):
         assert resp[0][0].tag == '{tns}some_call' + RESPONSE_SUFFIX
         assert resp[0][0].text == 'abc'
 
+    def test_soap_bare_empty_model_input_method_name(self):
+        class EmptyRequest(ComplexModel):
+            pass
+        try:
+            class SomeService(ServiceBase):
+                @rpc(EmptyRequest, _body_style='bare', _returns=String)
+                def some_call(ctx, request):
+                    return 'abc'
+        except Exception:
+            pass
+        else:
+            raise Exception("Must fail with exception: body_style='bare' does "
+                            "not allow empty model as param")
+
     def test_implicit_class_conflict(self):
         class someCallResponse(ComplexModel):
             __namespace__ = 'tns'
@@ -363,20 +379,21 @@ class TestBodyStyle(unittest.TestCase):
                                 out_protocol=Soap11(cleanup_namespaces=True))
 
         req = """
-        <senv:Envelope  xmlns:senv="http://schemas.xmlsoap.org/soap/envelope/"
+        <soap11env:Envelope  xmlns:soap11env="http://schemas.xmlsoap.org/soap/envelope/"
                         xmlns:tns="tns">
-            <senv:Body>
+            <soap11env:Body>
                 <tns:some_call/>
-            </senv:Body>
-        </senv:Envelope>
+            </soap11env:Body>
+        </soap11env:Envelope>
         """
 
         server = WsgiApplication(app)
         resp = etree.fromstring(''.join(server({
             'QUERY_STRING': '',
             'PATH_INFO': '/call',
-            'REQUEST_METHOD': 'GET',
-            'wsgi.input': StringIO(req)
+            'REQUEST_METHOD': 'POST',
+            'CONTENT_TYPE': 'text/xml',
+            'wsgi.input': BytesIO(req)
         }, start_response, "http://null")))
 
         print(etree.tostring(resp, pretty_print=True))
@@ -408,7 +425,5 @@ class TestBodyStyle(unittest.TestCase):
         else:
             raise Exception("Must fail with: "
                         "'SelfReference can't be used inside @rpc and its ilk'")
-
-
 if __name__ == '__main__':
     unittest.main()
