@@ -1,5 +1,7 @@
 #!/usr/bin/env python
 
+from __future__ import print_function
+
 import unittest
 import pytz
 import decimal
@@ -32,6 +34,7 @@ from spyne.util.protocol import deserialize_request_string
 from spyne.util.dictdoc import get_dict_as_object, get_object_as_yaml, \
     get_object_as_json
 from spyne.util.dictdoc import get_object_as_dict
+from spyne.util.tdict import tdict
 
 from spyne.util.xml import get_object_as_xml
 from spyne.util.xml import get_xml_as_object
@@ -158,6 +161,45 @@ class TestCDict(unittest.TestCase):
         else:
             raise Exception("Must fail.")
 
+class TestTDict(unittest.TestCase):
+    def test_tdict_notype(self):
+        d = tdict()
+        d[0] = 1
+        assert d[0] == 1
+
+        d = tdict()
+        d.update({0:1})
+        assert d[0] == 1
+
+        d = tdict.fromkeys([0], 1)
+        assert d[0] == 1
+
+    def test_tdict_k(self):
+        d = tdict(str)
+        try:
+            d[0] = 1
+        except TypeError:
+            pass
+        else:
+            raise Exception("must fail")
+
+        d = tdict(str)
+        d['s'] = 1
+        assert d['s'] == 1
+
+    def test_tdict_v(self):
+        d = tdict(vt=str)
+        try:
+            d[0] = 1
+        except TypeError:
+            pass
+        else:
+            raise Exception("must fail")
+
+        d = tdict(vt=str)
+        d[0] = 's'
+        assert d[0] == 's'
+
 
 class TestLogRepr(unittest.TestCase):
     def test_log_repr_simple(self):
@@ -174,7 +216,7 @@ class TestLogRepr(unittest.TestCase):
 
         assert log_repr(Z(z="a" * l)) == "Z(z='%s'(...))" % \
                                                 ('a' * MAX_STRING_FIELD_LENGTH)
-        assert log_repr(['a','b','c'], Array(String)) ==  "['a', 'b', 'c']"
+        assert log_repr(['a','b','c'], Array(String)) ==  "['a', 'b', (...)]"
 
     def test_log_repr_complex(self):
         from spyne.model import ByteArray
@@ -193,8 +235,8 @@ class TestLogRepr(unittest.TestCase):
         l = MAX_STRING_FIELD_LENGTH + 100
         val = Z(z=["abc"] * l, t=['t'], f=File.Value(name='aaa', data=['t']))
         print(repr(val))
-        print
-        assert log_repr(val) == "Z(z=['abc', 'abc', 'abc', 'abc', (...)])"
+
+        assert log_repr(val) == "Z(z=['abc', 'abc', (...)])"
 
 
 class TestDeserialize(unittest.TestCase):
