@@ -18,13 +18,15 @@
 #
 
 import logging
+import functools
+
 logger = logging.getLogger(__name__)
 
 import sys
 import datetime
 
-from collections import deque
 from inspect import isgeneratorfunction
+from spyne.util.six import PY3
 
 
 try:
@@ -107,12 +109,17 @@ def coroutine(func):
     assert isgeneratorfunction(func)
 
     def start(*args, **kwargs):
-        ret = func(*args, **kwargs)
+        try:
+            ret = func(*args, **kwargs)
+        except TypeError as e:
+            logger.error("Function %r at %s:%d got error %r", func.func_name,
+                         func.__module__, func.__code__.co_firstlineno, e)
+            raise
 
         try:
             next(ret)
 
-        except StopIteration:
+        except StopIteration as e:
             return None
 
         except Exception as e:
@@ -149,7 +156,18 @@ class memoize_id(memoize):
 
     def get_key(self, args, kwargs):
         return tuple([id(a) for a in args]), \
-                                   tuple([ (k,id(v)) for k,v in kwargs.items()])
+                                  tuple([(k, id(v)) for k, v in kwargs.items()])
+
+
+class memoize_id_method(memoize_id):
+    """A memoization decorator that keeps caching until reset for unhashable
+    types on instance methods. It works on id()'s of objects instead."""
+
+    def __get__(self, obj, objtype):
+        """Support instance methods."""
+        fn = functools.partial(self.__call__, obj)
+        fn.reset = self.reset
+        return fn
 
 
 def sanitize_args(a):
@@ -176,9 +194,9 @@ def sanitize_args(a):
     return args, kwargs
 
 
-if sys.version > '3':
-    def _bytes_join(val, joiner=''):
-        return bytes(joiner).join(val)
+if PY3:
+    def _bytes_join(val, joiner=b''):
+        return joiner.join(val)
 else:
     def _bytes_join(val, joiner=''):
         return joiner.join(val)
